@@ -1,24 +1,38 @@
 ;; Nat-listp.
 ;;
-;; This program is free software; you can redistribute it and/or modify it
-;; under the terms of the GNU General Public License as published by the Free
-;; Software Foundation; either version 2 of the License, or (at your option)
-;; any later version.
-;;
-;; This program is distributed in the hope that it will be useful but WITHOUT
-;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-;; more details.
-;;
-;; You should have received a copy of the GNU General Public License along with
-;; this program; if not, write to the Free Software Foundation, Inc., 59 Temple
-;; Place - Suite 330, Boston, MA 02111-1307, USA.
 
 ;; Note: Contributed initially by Sol Swords; modified by Matt Kaufmann.
 ;; Adapted from unicode/nat-listp.lisp, but INCOMPATIBLE with it.  This
 ;; version of nat-listp is similar to built-in ACL2 functions integer-listp,
 ;; symbol-listp, etc, in that it implies true-listp.
 
+; Copyright (C) 2014 Centaur Technology
+;
+; Contact:
+;   Centaur Technology Formal Verification Group
+;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
+;   http://www.centtech.com/
+;
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
+
 (in-package "ACL2")
 (include-book "xdoc/top" :dir :system)
 
@@ -29,9 +43,10 @@
   :short "Lemmas about @(see nat-listp) available in the @('arithmetic/nat-listp')
 book."
 
-  :long "<p>BOZO additional lemmas about @('nat-listp') are available in the
-book @(see std/typed-lists/nat-listp), which subsumes this one.  Should we get
-rid of this book?</p>"
+  :long "<p>Note: this book is pretty minimal.  You should probably generally
+instead see @(see std/typed-lists/nat-listp).</p>
+
+<p>BOZO Should we get rid of this book?</p>"
 
   (local (in-theory (enable nat-listp)))
 
