@@ -6,15 +6,25 @@
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 ;
 ; Original author: Jared Davis <jared@centtech.com>
 
@@ -144,17 +154,17 @@ sexprs.</p>")
                  x)
      :hints((witness)))
 
-   (defthm car-of-repeat
-     (equal (car (repeat a n))
+   (defthm car-of-replicate
+     (equal (car (replicate n a))
             (if (zp n)
                 nil
               a))
-     :hints(("Goal" :in-theory (enable repeat))))
+     :hints(("Goal" :in-theory (enable replicate))))
 
-   (defthm cdr-of-repeat
-     (equal (cdr (repeat a n))
-            (repeat a (- n 1)))
-     :hints(("Goal" :in-theory (enable repeat))))
+   (defthm cdr-of-replicate
+     (equal (cdr (replicate n a))
+            (replicate (- n 1) a))
+     :hints(("Goal" :in-theory (enable replicate))))
 
    (defthm hons-assoc-equal-of-atom
      (implies (atom alist)
@@ -1033,7 +1043,7 @@ implement.</p>"
     (declare (xargs :guard t :verify-guards nil))
     (b* (((when (atom vars))
           (fast-alist-free false-bindings)
-          (repeat (4vs-x) (len sexprs)))
+          (replicate (len sexprs) (4vs-x)))
 
          (var             (car vars))
          (bindings        (hons-acons var (4vs-t) false-bindings))
@@ -1238,7 +1248,7 @@ that don't mention any of the variables we are assuming to be one-hot.</p>"
            (new-sexprs
             (4vs-ite*-list-dumb (4vs-onehot vars)
                            (4v-onehot-sexpr-list-prime vars sexprs)
-                           (repeat (4vs-x) (len sexprs)))))
+                           (replicate (len sexprs) (4vs-x)))))
       (pairlis$ names new-sexprs)))
 
   (defund 4v-onehot-rw-sexpr-alist-aux (vars sexpr-alist)
