@@ -6,15 +6,25 @@
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 ;
 ; Original authors: Sol Swords <sswords@centtech.com>
 ;                   Jared Davis <jared@centtech.com>
@@ -29,6 +39,7 @@
 (include-book "4v-logic")
 (include-book "centaur/misc/fast-alists" :dir :system)
 (include-book "centaur/misc/hons-extra" :dir :system)
+(local (include-book "std/lists/nth" :dir :system))
 
 (defsection 4v-sexpr-ind
   :parents (4v-sexprs)
@@ -108,7 +119,7 @@ logic constants.  It must be a fast alist.</p>
 Moreover, the main theorems about other 4v-sexpr operations are usually stated
 in terms of the evaluations of their results.</p>
 
-<p>We @(see memoize) evaluation to avoid having to recomputing shared
+<p>We @(see memoize) evaluation to avoid having to recompute shared
 subexpressions.  Note that we do not memoize with @(':forget t') because you
 frequently want to evaluate several related expressions under the same
 environment, as in @(see 4v-sexpr-eval-alist).  As a consequence, you'll
@@ -129,7 +140,7 @@ of fixnums to hold the values, or similar.</p>"
     :long "<p>See @(see 4v-sexpr-eval); this is just its mutually recursive
 counterpart.</p>")
 
-  (defun 4v-sexpr-apply (fn args)
+  (defund 4v-sexpr-apply (fn args)
     (declare (xargs :guard (true-listp args)))
     (b* (((when (or (eq fn (4vt))
                     (eq fn (4vf))
@@ -148,6 +159,7 @@ counterpart.</p>")
         (ite*      (4v-ite*     arg1 arg2 arg3))
         (zif       (4v-zif      arg1 arg2 arg3))
         (buf       (4v-unfloat  arg1))
+        (xdet      (4v-xdet     arg1))
         (res       (4v-res      arg1 arg2))
         (tristate  (4v-tristate arg1 arg2))
         (ite       (4v-ite      arg1 arg2 arg3))
@@ -157,7 +169,7 @@ counterpart.</p>")
         ;; (wor      (4v-wor   arg1 arg2))
         (otherwise (4vx)))))
 
-
+  (local (in-theory (enable 4v-sexpr-apply)))
 ; [Jared] I was tempted to make a 4v-sexpr-eval1 function in the style of
 ; 4v-sexpr-eval-alist1, but this seems troublesome because it would mean
 ; changing all of our clear-memoize-table call functions.  I'm just going to
@@ -193,6 +205,7 @@ counterpart.</p>")
           (ite*      (4v-ite*     arg1 arg2 arg3))
           (zif       (4v-zif      arg1 arg2 arg3))
           (buf       (4v-unfloat  arg1))
+          (xdet      (4v-xdet     arg1))
           (res       (4v-res      arg1 arg2))
           (tristate  (4v-tristate arg1 arg2))
           (ite       (4v-ite      arg1 arg2 arg3))
@@ -259,12 +272,17 @@ counterpart.</p>")
       :flag sexpr-list)
     :hints (("goal"
              :in-theory (disable* (:ruleset 4v-op-defs) 4v-<= 4v-lookup
-                                  default-car default-cdr nth-when-zp nth-add1 nth))
+                                  default-car default-cdr nth-when-zp nth-add1 nth
+                                  ))
             (and stable-under-simplificationp
                  '(:use ((:instance 4v-alist-<=-necc
                                     (k x)
                                     (x env)
-                                    (y env1))))))))
+                                    (y env1)))))))
+
+  (defthm nth-of-4v-sexpr-eval-list
+    (4v-equiv (nth n (4v-sexpr-eval-list x env))
+              (4v-sexpr-eval (nth n x) env))))
 
 
 (defsection 4v-sexpr-eval-alist
@@ -276,7 +294,7 @@ bind names to sexprs.  It evaluates the sexprs under @('env') and returns a new
 alist that binds the same names to the resulting four-valued constants.  The
 new alist is an ordinary, non-fast alist.</p>
 
-<p>It is beneficial for @('env') to be a fast alist; if it is not fast, we we
+<p>It is beneficial for @('env') to be a fast alist; if it is not fast, we
 temporarily make it fast.</p>"
 
   (defund 4v-sexpr-eval-alist1 (x env)
@@ -702,3 +720,130 @@ by @('a'):</p>
   (verify-guards 4v-sexpr-alist-extract)
 
   (defcong alist-equiv equal (4v-sexpr-alist-extract keys al) 2))
+
+
+
+
+
+(defsection 4v-sexpr-eval-default
+  (defsection 4v-lookup-default
+
+    (defun 4v-lookup-default (k env default)
+      (declare (xargs :guard t))
+      (let ((look (hons-get k env)))
+        (prog2$ (and (not look) (4v-lookup-not-found k))
+                (4v-fix (if look (cdr look) default)))))
+
+    (defthm 4vp-of-4v-lookup-default
+      (4vp (4v-lookup-default k env default)))
+
+    (defthm 4v-fix-4v-lookup-default
+      (equal (4v-fix (4v-lookup-default k env default))
+             (4v-lookup-default k env default))))
+
+
+  (mutual-recursion
+   (defun 4v-sexpr-eval-default (x env default)
+     (declare (xargs :guard t))
+     (b* (((when (atom x))
+           ;; NIL is regarded as X, which is important in 4v-sexpr-compose and
+           ;; perhaps other places.  Any other atom is a variable and we look
+           ;; up its value in env.
+           (if x
+               (4v-lookup-default x env default)
+             (4vx)))
+          (fn   (car x))
+          (args (4v-sexpr-eval-default-list (cdr x) env default)))
+       (4v-sexpr-apply fn args)))
+
+   (defun 4v-sexpr-eval-default-list (x env default)
+     (declare (xargs :guard t))
+     (if (atom x)
+         nil
+       (cons (4v-sexpr-eval-default (car x) env default)
+             (4v-sexpr-eval-default-list (cdr x) env default)))))
+
+  (memoize '4v-sexpr-eval-default :condition '(and (consp x) (consp (cdr x))))
+
+  (in-theory (disable 4v-sexpr-eval-default)))
+
+
+(defsection 4v-sexpr-eval-default-alist
+
+  (defund 4v-sexpr-eval-default-alist1 (x env default)
+    "Assumes ENV is fast"
+    (declare (xargs :guard t))
+    (cond ((atom x)
+           nil)
+          ((atom (car x))
+           (4v-sexpr-eval-default-alist1 (cdr x) env default))
+          (t
+           (cons (cons (caar x) (4v-sexpr-eval-default (cdar x) env default))
+                 (4v-sexpr-eval-default-alist1 (cdr x) env default)))))
+
+  (defund 4v-sexpr-eval-default-alist (x env default)
+    "Makes ENV fast if necessary"
+    (declare (xargs :guard t :verify-guards nil))
+    (mbe :logic
+         (cond ((atom x)
+                nil)
+               ((atom (car x))
+                (4v-sexpr-eval-default-alist (cdr x) env default))
+               (t
+                (cons (cons (caar x) (4v-sexpr-eval-default (cdar x) env default))
+                      (4v-sexpr-eval-default-alist (cdr x) env default))))
+         :exec
+         (with-fast-alist env (4v-sexpr-eval-default-alist1 x env default))))
+
+  (local (in-theory (enable 4v-sexpr-eval-default-alist
+                            4v-sexpr-eval-default-alist1)))
+
+
+  (defund 4v-sexpr-eval-default-alists1 (x env default)
+    (declare (xargs :guard t))
+    (if (atom x)
+        nil
+      (cons (4v-sexpr-eval-default-alist1 (car x) env default)
+            (4v-sexpr-eval-default-alists1 (cdr x) env default))))
+
+  (defund 4v-sexpr-eval-default-alists (x env default)
+    "Makes ENV fast if necessary"
+    (declare (xargs :guard t :verify-guards nil))
+    (mbe :logic
+         (if (atom x)
+             nil
+           (cons (4v-sexpr-eval-default-alist (car x) env default)
+                 (4v-sexpr-eval-default-alists (cdr x) env default)))
+         :exec
+         (with-fast-alist env (4v-sexpr-eval-default-alists1 x env default))))
+
+  (local (in-theory (enable 4v-sexpr-eval-default-alists
+                            4v-sexpr-eval-default-alists1)))
+
+
+  (defthm 4v-sexpr-eval-default-alist1-removal
+    (equal (4v-sexpr-eval-default-alist1 x env default)
+           (4v-sexpr-eval-default-alist x env default)))
+
+  (defthm 4v-sexpr-eval-default-alists1-removal
+    (equal (4v-sexpr-eval-default-alists1 x env default)
+           (4v-sexpr-eval-default-alists x env default))
+    :hints(("Goal" :in-theory (e/d ()
+                                   (4v-sexpr-eval-default)))))
+
+  (verify-guards 4v-sexpr-eval-default-alist)
+  (verify-guards 4v-sexpr-eval-default-alists)
+
+  (defthm lookup-sexpr-eval-default-alist
+    (equal (hons-assoc-equal x (4v-sexpr-eval-default-alist al env default))
+           (and (hons-assoc-equal x al)
+                (cons x (4v-sexpr-eval-default (cdr (hons-assoc-equal x al)) env default)))))
+
+  (defthm 4v-sexpr-eval-default-alist-append
+    (equal (4v-sexpr-eval-default-alist (append a b) env default)
+           (append (4v-sexpr-eval-default-alist a env default)
+                   (4v-sexpr-eval-default-alist b env default))))
+
+  (defthm alist-keys-4v-sexpr-eval-default-alist
+    (equal (alist-keys (4v-sexpr-eval-default-alist a env default))
+           (alist-keys a))))
