@@ -6,15 +6,25 @@
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 ;
 ; AIG input variable collecting -- specialized/aux functions
 ; Original author: Sol Swords <sswords@centtech.com>
@@ -1108,7 +1118,7 @@ NIL
 (defun map-mergesort (x)
   (if (Atom x)
       nil
-    (cons (sets::mergesort (car x))
+    (cons (set::mergesort (car x))
           (map-mergesort (cdr x)))))
 
 (defun equal-mergesorts (x y)
@@ -1124,7 +1134,7 @@ NIL
 
 
 
-(sets::mergesort
+(set::mergesort
  (n-counts (map-len (map-aig-vars-smarter7 (alist-vals (@m
                                                         :reduced-prop-aigtab))))
            nil))
@@ -1135,9 +1145,9 @@ NIL
 (len *var-sets*)
 
 (defconst *sorted-var-set-set*
-  (sets::mergesort (map-mergesort *var-sets*)))
+  (set::mergesort (map-mergesort *var-sets*)))
 
-(sets::mergesort (n-counts (map-len *sorted-var-set-set*) nil))
+(set::mergesort (n-counts (map-len *sorted-var-set-set*) nil))
 
 
 
