@@ -6,15 +6,25 @@
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 ;
 ; Original author: Sol Swords <sswords@centtech.com>
 ;
@@ -37,15 +47,6 @@
 (set-default-hints '('(:do-not-induct t)))
 
 
-(defwitness alist-equiv-witnessing
-  :predicate (not (alist-equiv al1 al2))
-  :expr (not (let ((bg (alist-equiv-bad-guy al1 al2)))
-               (equal (hons-assoc-equal bg al1)
-                      (hons-assoc-equal bg al2))))
-  :hints('(:use alist-equiv-when-agree-on-bad-guy
-                 :in-theory nil))
-  :generalize (((alist-equiv-bad-guy al1 al2) . aeqkey)))
-
 
 
 (defexample aig-equiv-example
@@ -640,7 +641,7 @@
 
 
 
-(def-universal-equiv faig-fix-equiv
+(def-universal-equiv faig-onoff-equiv
   :equiv-terms ((iff (consp x))
                 (faig-equiv x))
   :parents (faig)
@@ -661,32 +662,32 @@ let @('y') be @('(t . t)').  Then, @('x') and @('y') are @('faig-equiv'), but
 their cars are not @('aig-equiv') because the car of @('x') is @('nil'),
 constant false, whereas the car of @('y') is @('t'), constant true.</p>
 
-<p>So, @(call faig-fix-equiv) corrects for this by insisting that @('x') and
+<p>So, @(call faig-onoff-equiv) corrects for this by insisting that @('x') and
 @('y') are either both atoms or both conses.  This way, the car/cdr of
-@('faig-fix-equiv') objects are always @(see aig-equiv).</p>")
+@('faig-onoff-equiv') objects are always @(see aig-equiv).</p>")
 
-(defsection faig-fix-equiv-thms
-  :extension faig-fix-equiv
+(defsection faig-onoff-equiv-thms
+  :extension faig-onoff-equiv
 
-  (defrefinement faig-fix-equiv faig-equiv
-    :hints(("Goal" :in-theory (enable faig-fix-equiv))))
+  (defrefinement faig-onoff-equiv faig-equiv
+    :hints(("Goal" :in-theory (enable faig-onoff-equiv))))
 
-  (defcong faig-equiv faig-fix-equiv (faig-fix x) 1
-    :hints(("Goal" :in-theory (enable faig-fix faig-fix-equiv
+  (defcong faig-equiv faig-onoff-equiv (faig-fix x) 1
+    :hints(("Goal" :in-theory (enable faig-fix faig-onoff-equiv
                                       faig-eval))
            (witness :ruleset (faig-equiv-witnessing
                               faig-equiv-instancing
                               faig-equiv-example)))
     :otf-flg t)
 
-  (defcong faig-fix-equiv aig-equiv (car x) 1
-    :hints(("Goal" :in-theory (enable faig-eval faig-fix-equiv))
+  (defcong faig-onoff-equiv aig-equiv (car x) 1
+    :hints(("Goal" :in-theory (enable faig-eval faig-onoff-equiv))
            (witness :ruleset (aig-equiv-witnessing
                               faig-equiv-instancing
                               faig-equiv-aig-eval-ex))))
 
-  (defcong faig-fix-equiv aig-equiv (cdr x) 1
-    :hints(("Goal" :in-theory (enable faig-eval faig-fix-equiv))
+  (defcong faig-onoff-equiv aig-equiv (cdr x) 1
+    :hints(("Goal" :in-theory (enable faig-eval faig-onoff-equiv))
            (witness :ruleset (aig-equiv-witnessing
                               faig-equiv-instancing
                               faig-equiv-aig-eval-ex)))))
