@@ -6,15 +6,25 @@
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 ;
 ; Original author: Sol Swords <sswords@centtech.com>
 ;
@@ -135,7 +145,10 @@ FAIGs.</p>"
     (cons (aig-not (aig-and a0 (aig-not a1)))
           (aig-not (aig-and a1 (aig-not a0)))))
   ///
-  (prove-faig-op-commutes f-aig-unfloat (a)))
+  (prove-faig-op-commutes f-aig-unfloat (a))
+
+  (defcong faig-fix-equiv equal (f-aig-unfloat a) 1
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix)))))
 
 
 (define t-aig-not (a)
@@ -147,7 +160,10 @@ evaluate to Z."
   (b* (((faig a1 a0) a))
     (cons a0 a1))
   ///
-  (prove-faig-op-commutes t-aig-not (a)))
+  (prove-faig-op-commutes t-aig-not (a))
+
+  (defcong faig-fix-equiv equal (t-aig-not a) 1
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix)))))
 
 (define f-aig-not (a)
   :parents (faig-constructors)
@@ -157,7 +173,44 @@ evaluate to Z."
     (cons (aig-not (aig-and a1 (aig-not a0)))
           (aig-not (aig-and a0 (aig-not a1)))))
   ///
-  (prove-faig-op-commutes f-aig-not (a)))
+  (prove-faig-op-commutes f-aig-not (a))
+
+  (defcong faig-fix-equiv equal (f-aig-not a) 1
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix)))))
+
+(define t-aig-xdet (a)
+  :parents (faig-constructors)
+  :short "@(call t-aig-xdet) constructs an FAIG for @(see 4v-xdet), assuming
+that the argument @('a') cannot evaluate to Z."
+  :inline t
+  :enabled t
+  (b* (((faig a1 a0) a))
+    ;; Want 0 if it's boolean, or X if it's not.
+    ;; Offset must always be 1.
+    ;; Onset is: (and a1 a0)
+    (cons (aig-and a1 a0) t))
+  ///
+  (prove-faig-op-commutes t-aig-xdet (a))
+
+  (defcong faig-fix-equiv equal (t-aig-xdet a) 1
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix)))))
+
+
+(define f-aig-xdet (a)
+  :parents (faig-constructors)
+  :short "@(call f-aig-xdet) constructs an FAIG for @(see 4v-xdet)."
+  :inline t
+  :enabled t
+  (b* (((faig a1 a0) a))
+    ;; Want 0 if it's boolean, or X if it's not.
+    ;; Offset must always be 1.
+    ;; Onset is: (iff a1 a0)
+    (cons (aig-iff a1 a0) t))
+  ///
+  (prove-faig-op-commutes f-aig-xdet (a))
+
+  (defcong faig-fix-equiv equal (f-aig-xdet a) 1
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix)))))
 
 
 (define t-aig-and (a b)
@@ -170,7 +223,12 @@ assuming they cannot evaluate to Z."
     (cons (aig-and a1 b1)
           (aig-or  a0 b0)))
   ///
-  (prove-faig-op-commutes t-aig-and (a b)))
+  (prove-faig-op-commutes t-aig-and (a b))
+
+  (defcong faig-fix-equiv equal (t-aig-and a b) 1
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix))))
+  (defcong faig-fix-equiv equal (t-aig-and a b) 2
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix)))))
 
 (define f-aig-and (a b)
   :parents (faig-constructors)
@@ -180,7 +238,12 @@ assuming they cannot evaluate to Z."
        (b (f-aig-unfloat b)))
     (t-aig-and a b))
   ///
-  (prove-faig-op-commutes f-aig-and (a b)))
+  (prove-faig-op-commutes f-aig-and (a b))
+
+  (defcong faig-fix-equiv equal (f-aig-and a b) 1
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix))))
+  (defcong faig-fix-equiv equal (f-aig-and a b) 2
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix)))))
 
 
 (define t-aig-or (a b)
@@ -193,7 +256,12 @@ assuming they cannot evaluate to Z."
     (cons (aig-or  a1 b1)
           (aig-and a0 b0)))
   ///
-  (prove-faig-op-commutes t-aig-or (a b)))
+  (prove-faig-op-commutes t-aig-or (a b))
+  
+  (defcong faig-fix-equiv equal (t-aig-or a b) 1
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix))))
+  (defcong faig-fix-equiv equal (t-aig-or a b) 2
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix)))))
 
 (define f-aig-or (a b)
   :parents (faig-constructors)
@@ -203,7 +271,12 @@ assuming they cannot evaluate to Z."
        (b (f-aig-unfloat b)))
     (t-aig-or a b))
   ///
-  (prove-faig-op-commutes f-aig-or (a b)))
+  (prove-faig-op-commutes f-aig-or (a b))
+  
+  (defcong faig-fix-equiv equal (f-aig-or a b) 1
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix))))
+  (defcong faig-fix-equiv equal (f-aig-or a b) 2
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix)))))
 
 
 (define t-aig-xor (a b)
@@ -214,7 +287,12 @@ assuming they cannot evaluate to Z."
   (t-aig-or (t-aig-and a (t-aig-not b))
             (t-aig-and (t-aig-not a) b))
   ///
-  (prove-faig-op-commutes t-aig-xor (a b)))
+  (prove-faig-op-commutes t-aig-xor (a b))
+
+  (defcong faig-fix-equiv equal (t-aig-xor a b) 1
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix))))
+  (defcong faig-fix-equiv equal (t-aig-xor a b) 2
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix)))))
 
 (define f-aig-xor (a b)
   :parents (faig-constructors)
@@ -224,7 +302,12 @@ assuming they cannot evaluate to Z."
        (b (f-aig-unfloat b)))
     (t-aig-xor a b))
   ///
-  (prove-faig-op-commutes f-aig-xor (a b)))
+  (prove-faig-op-commutes f-aig-xor (a b))
+
+  (defcong faig-fix-equiv equal (f-aig-xor a b) 1
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix))))
+  (defcong faig-fix-equiv equal (f-aig-xor a b) 2
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix)))))
 
 
 (define t-aig-iff (a b)
@@ -235,7 +318,12 @@ assuming they cannot evaluate to Z."
   (t-aig-or (t-aig-and a b)
             (t-aig-and (t-aig-not a) (t-aig-not b)))
   ///
-  (prove-faig-op-commutes t-aig-iff (a b)))
+  (prove-faig-op-commutes t-aig-iff (a b))
+
+  (defcong faig-fix-equiv equal (t-aig-iff a b) 1
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix))))
+  (defcong faig-fix-equiv equal (t-aig-iff a b) 2
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix)))))
 
 (define f-aig-iff (a b)
   :parents (faig-constructors)
@@ -245,7 +333,12 @@ assuming they cannot evaluate to Z."
        (b (f-aig-unfloat b)))
     (t-aig-iff a b))
   ///
-  (prove-faig-op-commutes f-aig-iff (a b)))
+  (prove-faig-op-commutes f-aig-iff (a b))
+
+  (defcong faig-fix-equiv equal (f-aig-iff a b) 1
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix))))
+  (defcong faig-fix-equiv equal (f-aig-iff a b) 2
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix)))))
 
 
 (define t-aig-ite (c a b)
@@ -262,7 +355,14 @@ for discussion related to this issue.</p>"
     (cons (aig-or (aig-and c1 a1) (aig-and c0 b1))
           (aig-or (aig-and c1 a0) (aig-and c0 b0))))
   ///
-  (prove-faig-op-commutes t-aig-ite (c a b)))
+  (prove-faig-op-commutes t-aig-ite (c a b))
+
+  (defcong faig-fix-equiv equal (t-aig-ite c a b) 1
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix))))
+  (defcong faig-fix-equiv equal (t-aig-ite c a b) 2
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix))))
+  (defcong faig-fix-equiv equal (t-aig-ite c a b) 3
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix)))))
 
 (define f-aig-ite (c a b)
   :parents (faig-constructors)
@@ -277,7 +377,14 @@ for discussion related to this issue.</p>"
        (b (f-aig-unfloat b)))
     (t-aig-ite c a b))
   ///
-  (prove-faig-op-commutes f-aig-ite (c a b)))
+  (prove-faig-op-commutes f-aig-ite (c a b))
+
+  (defcong faig-fix-equiv equal (f-aig-ite c a b) 1
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix))))
+  (defcong faig-fix-equiv equal (f-aig-ite c a b) 2
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix))))
+  (defcong faig-fix-equiv equal (f-aig-ite c a b) 3
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix)))))
 
 
 (define t-aig-ite* (c a b)
@@ -295,7 +402,13 @@ for discussion related to this issue.</p>"
     (cons (aig-or x (aig-or (aig-and c1 a1) (aig-and c0 b1)))
           (aig-or x (aig-or (aig-and c1 a0) (aig-and c0 b0)))))
   ///
-  (prove-faig-op-commutes t-aig-ite* (c a b)))
+  (prove-faig-op-commutes t-aig-ite* (c a b))
+  (defcong faig-fix-equiv equal (t-aig-ite* c a b) 1
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix))))
+  (defcong faig-fix-equiv equal (t-aig-ite* c a b) 2
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix))))
+  (defcong faig-fix-equiv equal (t-aig-ite* c a b) 3
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix)))))
 
 (define f-aig-ite* (c a b)
   :parents (faig-constructors)
@@ -310,7 +423,14 @@ for discussion related to this issue.</p>"
        (b (f-aig-unfloat b)))
     (t-aig-ite* c a b))
   ///
-  (prove-faig-op-commutes f-aig-ite* (c a b)))
+  (prove-faig-op-commutes f-aig-ite* (c a b))
+
+  (defcong faig-fix-equiv equal (f-aig-ite* c a b) 1
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix))))
+  (defcong faig-fix-equiv equal (f-aig-ite* c a b) 2
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix))))
+  (defcong faig-fix-equiv equal (f-aig-ite* c a b) 3
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix)))))
 
 (define f-aig-zif (c a b)
   :parents (faig-constructors)
@@ -322,7 +442,14 @@ conversion.</p>"
   (b* ((c (f-aig-unfloat c)))
     (t-aig-ite* c a b))
   ///
-  (prove-faig-op-commutes f-aig-zif (c a b)))
+  (prove-faig-op-commutes f-aig-zif (c a b))
+
+  (defcong faig-fix-equiv equal (f-aig-zif c a b) 1
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix))))
+  (defcong faig-fix-equiv equal (f-aig-zif c a b) 2
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix))))
+  (defcong faig-fix-equiv equal (f-aig-zif c a b) 3
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix)))))
 
 (define f-aig-res (x y)
   :parents (faig-constructors)
@@ -335,7 +462,12 @@ the @(see faig) equivalent.</p>"
        ((faig y1 y0) y))
     (cons (aig-or x1 y1) (aig-or x0 y0)))
   ///
-  (prove-faig-op-commutes f-aig-res (a b)))
+  (prove-faig-op-commutes f-aig-res (a b))
+
+  (defcong faig-fix-equiv equal (f-aig-res x y) 1
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix))))
+  (defcong faig-fix-equiv equal (f-aig-res x y) 2
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix)))))
 
 
 ;; Theorem: no F-AIG-UNFLOAT needed around the answer of f-aig-ite.
@@ -384,7 +516,12 @@ buffer whose inputs are known to be non-X."
     (cons (aig-and (aig-not float) (aig-not (aig-and set off)))
           (aig-and (aig-not float) (aig-not (aig-and set on)))))
   ///
-  (prove-faig-op-commutes t-aig-tristate (c a)))
+  (prove-faig-op-commutes t-aig-tristate (c a))
+
+  (defcong faig-fix-equiv equal (t-aig-tristate c a) 1
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix))))
+  (defcong faig-fix-equiv equal (t-aig-tristate c a) 2
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix)))))
 
 (define f-aig-pullup (a)
   :parents (faig-constructors)
@@ -395,12 +532,16 @@ resistor."
        (a-floating (aig-not a-not-aig-floating)))
     (cons (aig-or a-floating a1) a0))
   ///
-  (prove-faig-op-commutes f-aig-pullup (a)))
+  (prove-faig-op-commutes f-aig-pullup (a))
+
+  (defcong faig-fix-equiv equal (f-aig-pullup a) 1
+    :hints(("Goal" :in-theory (enable faig-fix-equiv faig-fix)))))
 
 
 (def-ruleset f-aig-defs
   '(f-aig-unfloat
     f-aig-not
+    f-aig-xdet
     f-aig-and
     f-aig-or
     f-aig-xor
@@ -418,6 +559,7 @@ resistor."
     t-aig-ite
     t-aig-ite*
     t-aig-not
+    t-aig-xdet
     t-aig-or
     t-aig-xor))
 
