@@ -6,15 +6,25 @@
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 ;
 ; Original author: Sol Swords <sswords@centtech.com>
 
@@ -37,14 +47,14 @@
 (local (include-book "arithmetic/top-with-meta" :dir :system))
 (local (include-book "centaur/bitops/ihsext-basics" :dir :system))
 (local (in-theory (enable* acl2::arith-equiv-forwarding)))
-(local (in-theory (disable sets::double-containment)))
+(local (in-theory (disable set::double-containment)))
 (local (in-theory (disable nth update-nth
                            acl2::nfix-when-not-natp
                            resize-list
                            acl2::resize-list-when-empty
                            acl2::make-list-ac-redef
-                           sets::double-containment
-                           sets::sets-are-true-lists
+                           set::double-containment
+                           set::sets-are-true-lists
                            make-list-ac)))
 
 
@@ -94,8 +104,6 @@
       x
     (const-stype))
   ///
-  (defthm stypep-of-stype-fix
-    (stypep (stype-fix x)))
   (defthm stype-fix-when-stypep
     (implies (stypep x)
              (equal (stype-fix x) x))))
@@ -152,8 +160,6 @@
       x
     (const-ctype))
   ///
-  (defthm ctypep-of-ctype-fix
-    (ctypep (ctype-fix x)))
   (defthm ctype-fix-when-ctypep
     (implies (ctypep x)
              (equal (ctype-fix x) x))))
@@ -1292,10 +1298,10 @@
                            (lookup-id id y)))
            :hints(("Goal" :in-theory (enable aignet-extension-p lookup-id)))))
 
-  (defthm <=-minus-1-rewrite
-    (implies (and (natp x) (natp y))
-             (equal (< (+ -1 y) x)
-                    (not (< x y)))))
+  (local (defthm <=-minus-1-rewrite
+           (implies (and (natp x) (natp y))
+                    (equal (< (+ -1 y) x)
+                           (not (< x y))))))
 
   (defthm aignet-lit-fix-id-val-linear
     (<= (lit-id (aignet-lit-fix lit aignet))
