@@ -6,15 +6,25 @@
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 ;
 ; Original author: Sol Swords <sswords@centtech.com>
 
@@ -36,8 +46,8 @@
                            acl2::nthcdr-of-cdr
                            acl2::resize-list-when-empty
                            acl2::make-list-ac-redef
-                           sets::double-containment
-                           sets::sets-are-true-lists
+                           set::double-containment
+                           set::sets-are-true-lists
                            make-list-ac)))
 
 (set-waterfall-parallelism nil) ; currently unknown why we need to disable
@@ -425,7 +435,21 @@
 
   (local (in-theory (disable id-eval
                              acl2::nth-with-large-index
-                             sets::double-containment)))
+                             set::double-containment)))
+
+  (defund aiglist-to-aignet-aux (x xmemo varmap gatesimp strash aignet lits)
+    (declare (xargs :stobjs (aignet strash)
+                    :guard (and (natp gatesimp)
+                                (good-varmap-p varmap aignet)
+                                (xmemo-well-formedp xmemo aignet)
+                                (true-listp lits))
+                    :verify-guards nil))
+    (b* (((when (atom x))
+          (mv (reverse lits) xmemo strash aignet))
+         ((mv lit xmemo strash aignet)
+          (aig-to-aignet (car x) xmemo varmap gatesimp strash aignet)))
+      (aiglist-to-aignet-aux (cdr x) xmemo varmap gatesimp strash aignet
+                             (cons lit lits))))
 
   ;; just puts the gates in and adds inputs/regs as needed, doesn't set up
   ;; outputs or reg inputs
@@ -435,16 +459,26 @@
                                 (good-varmap-p varmap aignet)
                                 (xmemo-well-formedp xmemo aignet))
                     :verify-guards nil))
-    (b* (((when (atom x))
-          (mv nil xmemo strash aignet))
-         ((mv lit xmemo strash aignet)
-          (aig-to-aignet (car x) xmemo varmap gatesimp strash aignet))
-         ((mv rest xmemo strash aignet)
-          (aiglist-to-aignet (cdr x) xmemo varmap gatesimp strash aignet)))
-      (mv (cons lit rest)
-          xmemo strash aignet)))
+    (mbe :logic (b* (((when (atom x))
+                      (mv nil xmemo strash aignet))
+                     ((mv lit xmemo strash aignet)
+                      (aig-to-aignet (car x) xmemo varmap gatesimp strash aignet))
+                     ((mv rest xmemo strash aignet)
+                      (aiglist-to-aignet (cdr x) xmemo varmap gatesimp strash aignet)))
+                  (mv (cons lit rest)
+                      xmemo strash aignet))
+         :exec
+         (aiglist-to-aignet-aux x xmemo varmap gatesimp strash aignet nil)))
 
-  (local (in-theory (enable aiglist-to-aignet)))
+  (local (in-theory (enable aiglist-to-aignet
+                            aiglist-to-aignet-aux)))
+
+  (defthm aiglist-to-aignet-aux-elim
+    (implies (true-listp lits)
+             (equal (aiglist-to-aignet-aux x xmemo varmap gatesimp strash aignet lits)
+                    (mv-let (rest-lits xmemo strash aignet)
+                      (aiglist-to-aignet x xmemo varmap gatesimp strash aignet)
+                      (mv (revappend lits rest-lits) xmemo strash aignet)))))
 
   (def-aignet-preservation-thms aiglist-to-aignet)
 
@@ -482,6 +516,7 @@
                     (equal (aignet-eval-to-env varmap in-vals reg-vals aignet1)
                            (aignet-eval-to-env varmap in-vals reg-vals aignet))))))
 
+  (verify-guards aiglist-to-aignet-aux)
   (verify-guards aiglist-to-aignet)
 
   (defun aiglist-to-aignet-top (x varmap gatesimp strash aignet)
@@ -1027,7 +1062,7 @@
                        (<= (nfix n) (len x)))
                   (lit-listp (take n x)))
          :hints(("Goal" :in-theory (enable take
-                                           acl2::repeat)))))
+                                           acl2::replicate)))))
 
 (local (defthm lit-listp-of-nthcdr
          (implies (lit-listp x)
@@ -1046,7 +1081,7 @@
                   (aignet-lit-listp (take n x) aignet))
          :hints(("Goal" :in-theory (enable aignet-lit-listp
                                            take
-                                           acl2::repeat)))))
+                                           acl2::replicate)))))
 
 (local (defthm aignet-lit-listp-of-nthcdr
          (implies (aignet-lit-listp x aignet)
@@ -1066,7 +1101,7 @@
   (implies (<= (nfix n) (len x))
            (equal (lit-eval-list (take n x) in-vals reg-vals aignet)
                   (take n (lit-eval-list x in-vals reg-vals aignet))))
-  :hints(("Goal" :in-theory (enable take acl2::repeat
+  :hints(("Goal" :in-theory (enable take acl2::replicate
                                     acl2::nfix))))
 
 (defthm aig-eval-list-of-append
@@ -1097,11 +1132,12 @@
 (encapsulate nil
   (local
    (progn
-     (defthm nthcdr-of-update-nth
-       (equal (nthcdr n (update-nth n v x))
-              (cons v
-                    (nthcdr (1+ (nfix n)) x)))
-       :hints(("Goal" :in-theory (e/d (update-nth)))))
+     ;; no longer necessary with std/lists/update-nth
+     ;; (defthm nthcdr-of-update-nth
+     ;;   (equal (nthcdr n (update-nth n v x))
+     ;;          (cons v
+     ;;                (nthcdr (1+ (nfix n)) x)))
+     ;;   :hints(("Goal" :in-theory (e/d (update-nth)))))
      (defthm nthcdr-empty
        (implies (and (true-listp lst)
                      (<= (len lst) (nfix n)))
@@ -1250,12 +1286,6 @@
            (implies (lit-listp x)
                     (true-listp x))))
 
-  (defthm repeat-when-zp
-    (implies (zp n)
-             (equal (acl2::repeat x n) nil))
-    :hints(("Goal" :in-theory (enable acl2::repeat))))
-
-
 
   (defthm add-to-varmap-preserves-lookup
     (implies (hons-assoc-equal x varmap)
@@ -1410,7 +1440,7 @@
   ;;                                  set-regs-logic
   ;;                                  set-outs-logic
   ;;                                  update-lit-list
-  ;;                                  sets::double-containment
+  ;;                                  set::double-containment
   ;;                                  resize-list))
   ;;         :do-not-induct t)))
 
@@ -1431,6 +1461,6 @@
                                    pairlis$
                                    hons-assoc-equal
                                    acl2::make-fal
-                                   sets::double-containment
+                                   set::double-containment
                                    resize-list))
           :do-not-induct t))))
