@@ -6,15 +6,25 @@
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 ;
 ; Original author: Sol Swords <sswords@centtech.com>
 
@@ -32,14 +42,14 @@
 (local (include-book "arithmetic/top-with-meta" :dir :system))
 (local (include-book "centaur/bitops/ihsext-basics" :dir :system))
 (local (in-theory (enable* acl2::arith-equiv-forwarding)))
-(local (in-theory (disable sets::double-containment)))
+(local (in-theory (disable set::double-containment)))
 (local (in-theory (disable nth update-nth
                            acl2::nfix-when-not-natp
                            resize-list
                            acl2::resize-list-when-empty
                            acl2::make-list-ac-redef
-                           sets::double-containment
-                           sets::sets-are-true-lists
+                           set::double-containment
+                           set::sets-are-true-lists
                            make-list-ac)))
 
 (local (include-book "std/lists/nth" :dir :system))
@@ -1459,3 +1469,86 @@ same outputs when run starting at that initial state.</p>"
 
 
 
+(defsection aignet-print
+  (local (in-theory (disable len)))
+  (defund aignet-print-lit (lit aignet)
+    (declare (xargs :stobjs aignet
+                    :guard (and (litp lit)
+                                (fanin-litp lit aignet))))
+    (b* ((id (lit-id lit))
+         (type (id->type id aignet))
+         ((when (int= type (const-type)))
+          (if (int= (lit-neg lit) 1) "1" "0")))
+      (acl2::msg "~s0~s1~x2"
+                 (if (int= (lit-neg lit) 1) "~" "")
+                 (if (int= type (in-type))
+                     (if (int= (io-id->regp id aignet) 1) "r" "i")
+                   "g")
+                 (if (int= type (in-type))
+                     (io-id->ionum id aignet)
+                   id))))
+
+  (defund aignet-print-gate (n aignet)
+    (declare (Xargs :stobjs aignet
+                    :guard (and (natp n)
+                                (id-existsp n aignet)
+                                (int= (id->type n aignet) (gate-type)))))
+    (b* ((f0 (gate-id->fanin0 n aignet))
+         (f1 (gate-id->fanin1 n aignet)))
+      (acl2::msg "g~x0 = ~@1 & ~@2"
+                 n
+                 (aignet-print-lit f0 aignet)
+                 (aignet-print-lit f1 aignet))))
+
+
+  (local (set-default-hints nil))
+  
+  (defund aignet-print-gates (n aignet)
+    (declare (Xargs :stobjs aignet
+                    :guard (and (natp n)
+                                (<= n (num-nodes aignet)))
+                    :guard-hints (("goal" :in-theory (enable aignet-idp)))
+                    :measure (nfix (- (nfix (num-nodes aignet)) (nfix n)))))
+    (b* (((when (mbe :logic (zp (- (nfix (num-nodes aignet)) (nfix n)))
+                     :exec (= (num-nodes aignet) n)))
+          nil)
+         ((unless (int= (id->type n aignet) (gate-type)))
+          (aignet-print-gates (1+ (lnfix n)) aignet))
+         (- (cw "~@0~%" (aignet-print-gate n aignet))))
+      (aignet-print-gates (1+ (lnfix n)) aignet)))
+
+  (defund aignet-print-outs (n aignet)
+    (declare (Xargs :stobjs aignet
+                    :guard (and (natp n)
+                                (<= n (num-outs aignet)))
+                    :guard-hints (("goal" :in-theory (e/d (lookup-stype-in-bounds))))
+                    :measure (nfix (- (nfix (num-outs aignet)) (nfix n)))))
+    (b* (((when (mbe :logic (zp (- (nfix (num-outs aignet)) (nfix n)))
+                     :exec (= (num-outs aignet) n)))
+          nil)
+         (- (cw "o~x0 = ~@1~%" n (aignet-print-lit
+                                  (co-id->fanin (outnum->id n aignet) aignet)
+                                  aignet))))
+      (aignet-print-outs (1+ (lnfix n)) aignet)))
+
+  (defund aignet-print-regs (n aignet)
+    (declare (Xargs :stobjs aignet
+                    :guard (and (natp n)
+                                (<= n (num-regs aignet)))
+                    :guard-hints (("goal" :in-theory (e/d (lookup-stype-in-bounds))))
+                    :measure (nfix (- (nfix (num-regs aignet)) (nfix n)))))
+    (b* (((when (mbe :logic (zp (- (nfix (num-regs aignet)) (nfix n)))
+                     :exec (= (num-regs aignet) n)))
+          nil)
+         (ri (reg-id->nxst (regnum->id n aignet) aignet))
+         ((when (int= ri 0))
+          (aignet-print-regs (1+ (lnfix n)) aignet))
+         (- (cw "r~x0 = ~@1~%" n
+                (aignet-print-lit (co-id->fanin ri aignet) aignet))))
+      (aignet-print-regs (1+ (lnfix n)) aignet)))
+
+  (defund aignet-print (aignet)
+    (declare (xargs :stobjs aignet))
+    (progn$ (aignet-print-gates 0 aignet)
+            (aignet-print-outs 0 aignet)
+            (aignet-print-regs 0 aignet))))
