@@ -6,26 +6,39 @@
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 ;
 ; Original author: Jared Davis <jared@centtech.com>
 
 (in-package "RSTOBJ")
 (include-book "def-typed-record")
 (include-book "generic")
-(include-book "misc/definline" :dir :system)
 (include-book "misc/records" :dir :system)
 (include-book "tools/bstar" :dir :system)
 (include-book "centaur/misc/arith-equivs" :dir :system)
 (include-book "centaur/misc/absstobjs" :dir :system)
+(include-book "std/lists/nth" :dir :system)
+(include-book "std/lists/resize-list" :dir :system)
+(include-book "std/lists/len" :dir :system)
+
 
 (defsection defrstobj
   :parents (stobj macro-libraries)
@@ -216,12 +229,31 @@ records book.  See @(see def-typed-record).</p>")
 ;   (2) we extend it with the :typed-record field, etc.
 
 
+(defmacro access-defstobj-field-template (x field)
+  `(acl2::access acl2::defstobj-field-template ,x ,field))
+
+(defun destructure-defstobj-field-template (x)
+
+  ;; BOZO this has to be kept in sync with defstobj-field-template.
+
+  (let ((fieldp-name (access-defstobj-field-template x :fieldp-name))
+        (type (access-defstobj-field-template x :type))
+        (init (access-defstobj-field-template x :init))
+        (accessor-name (access-defstobj-field-template x :accessor-name))
+        (updater-name (access-defstobj-field-template x :updater-name))
+        (length-name (access-defstobj-field-template x :length-name))
+        (resize-name (access-defstobj-field-template x :resize-name))
+        (resizable (access-defstobj-field-template x :resizable))
+        (other (access-defstobj-field-template x :other)))
+    (list fieldp-name type init accessor-name updater-name length-name
+          resize-name resizable other)))
+
 (defun make-fta (field-name typed-rec fix defstobj-fld-template mksym-pkg tr-table)
   (declare (xargs :mode :program))
   (b* (((list recog-name type init accessor-name updater-name length-name
-              resize-name resizable)
-        ;; BOZO this has to be kept in sync with defstobj-template
-        defstobj-fld-template)
+              resize-name resizable other)
+        ;; BOZO this has to be kept in sync with defstobj-field-template
+        (destructure-defstobj-field-template defstobj-fld-template))
 
 
        ;; We could just override the init settting with the typed-record init
@@ -285,6 +317,7 @@ records book.  See @(see def-typed-record).</p>")
       (:length-name$c   . ,length-name)
       (:resize-name$c   . ,resize-name)
       (:resizable     . ,resizable)
+      (:other         . ,other)
       (:offset        . ,(acl2::defconst-name accessor-name))
       (:typed-record  . ,typed-rec)
       (:accessor-name . ,(mksym 'get- field-name))
@@ -568,11 +601,11 @@ records book.  See @(see def-typed-record).</p>")
        (recog (cdr (assoc :recog-name$c fta)))
        (hyp (translate-declaration-to-guard (second type) 'v w)))
     (append
-     `((defthm ,(mksym recog '-of-repeat)
+     `((defthm ,(mksym recog '-of-replicate)
          (implies ,hyp
-                  (,recog (acl2::repeat v n)))
-         :hints(("Goal" :in-theory (enable acl2::repeat)
-                 :induct (acl2::repeat v n))))
+                  (,recog (acl2::replicate n v)))
+         :hints(("Goal" :in-theory (enable acl2::replicate)
+                 :induct (acl2::replicate n v))))
 
        (defthm ,(mksym recog '-of-update-nth)
          (implies (and ,hyp
@@ -631,7 +664,27 @@ records book.  See @(see def-typed-record).</p>")
 
 
 
+(defmacro access-defstobj-template (x field)
+  `(acl2::access acl2::defstobj-template ,x ,field))
 
+(defun destructure-defstobj-template (x)
+
+  ;; BOZO this has to be kept in sync with defstobj-template.
+
+  (let ((recognizer (access-defstobj-template x :recognizer))
+        (creator (access-defstobj-template x :creator))
+        (field-templates (access-defstobj-template x :field-templates))
+        (doc (access-defstobj-template x :doc))
+        (inline (access-defstobj-template x :inline))
+        (congruent-to (access-defstobj-template x :congruent-to))
+        (non-memoizable (access-defstobj-template x :non-memoizable)))
+    (list recognizer
+          creator
+          field-templates
+          doc
+          inline
+          congruent-to
+          non-memoizable)))
 
 (defun defrstobj-fn (name args wrld)
   (declare (ignorable wrld)
@@ -652,9 +705,10 @@ records book.  See @(see def-typed-record).</p>")
        (name$c        (mksym name '$c))
        (st-fields$c   (make-$c-fields st-fields mksym-pkg))
        (st-template   (defstobj-template name$c (append st-fields$c st-kw-part) wrld))
-       ((list recog$c create$c st$c-fld-templates ?doc ?inline)
+       ((list recog$c create$c st$c-fld-templates ?doc ?inline ?congruent-to
+              ?non-memoizable)
         ;; BOZO this has to be kept in sync with defstobj-template.
-        st-template)
+        (destructure-defstobj-template st-template))
 
        (tr-table      (table-alist 'typed-records wrld))
 
@@ -693,10 +747,10 @@ records book.  See @(see def-typed-record).</p>")
                             ((:t nfix)
                              natp-compound-recognizer
                              zp-compound-recognizer
-                             nth-of-repeat
-                             len-of-repeat
+                             nth-of-replicate
+                             len-of-replicate
                              append-to-nil
-                             (:t repeat)
+                             (:t replicate)
                              make-list-ac-removal
                              nth-when-atom
                              nth-0-cons
@@ -714,7 +768,7 @@ records book.  See @(see def-typed-record).</p>")
                              resize-list-when-zp
                              (:rules-of-class :executable-counterpart :here))
                             ((make-list-ac)
-                             (repeat)))))
+                             (replicate)))))
 
          (local (in-theory (e/d
                             (plus-collect-consts
