@@ -6,15 +6,25 @@
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 
 
 ; esim-paths.lisp -- path and well-formedness definitions for esim-new-probe
@@ -24,9 +34,9 @@
 
 (in-package "ACL2")
 (include-book "esim-sexpr-support")
-(local (include-book "esim-sexpr-support-thms"))
 (include-book "std/util/deflist" :dir :system)
 (include-book "std/util/defmvtypes" :dir :system)
+(local (include-book "esim-sexpr-support-thms"))
 
 (make-event
 
@@ -275,7 +285,8 @@ fast-canonicalize-path).</p>"
 
 (defthm good-esim-occs-of-occs-when-good-esim-modulep
   (implies (good-esim-modulep mod)
-           (good-esim-occsp (gpl :occs mod))))
+           (good-esim-occsp (gpl :occs mod)))
+  :hints(("Goal" :in-theory (enable good-esim-modulep))))
 
 (defthm good-esim-modulep-of-op-when-good-esim-occ
   (implies (good-esim-occp occ)
@@ -340,7 +351,7 @@ describes the same wire:</p>
 })
 
 <p>@('Fast-canonicalize-path') should be given a valid path in the sense of
-@(see mod-all-paths), and tries to return the canonical equivalent.  Failure
+@('mod-all-paths'), and tries to return the canonical equivalent.  Failure
 indicates that the given path is invalid, but we do only minimal error
 checking.  In other words, you should not rely on this function to see whether
 a path is valid.</p>"
