@@ -6,15 +6,25 @@
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 
 ; esim-sexpr-support -- Functions that support esim-faig
 ;
@@ -26,7 +36,8 @@
 (include-book "centaur/4v-sexpr/sexpr-vars" :dir :system)
 (include-book "std/misc/two-nats-measure" :dir :system)
 (include-book "arithmetic/nat-listp" :dir :system)
-(local (include-book "str/explode-atom" :dir :system))
+(include-book "std/lists/remove-duplicates" :dir :system)
+(local (include-book "std/strings/explode-atom" :dir :system))
 (set-well-founded-relation nat-list-<)
 
 (make-event
@@ -391,6 +402,8 @@
 
 (memoize 'good-esim-modulep :condition '(gpl :occs x))
 
+(in-theory (disable good-esim-modulep))
+
 
 
 
@@ -567,8 +580,9 @@
          (not (good-esim-occsp x)))
     :flag occs)
   :hints(("Goal"
-          :in-theory (disable hons-intersect-p hons-dups-p
-                              hons-member-equal))))
+          :in-theory (e/d (good-esim-modulep)
+                          (hons-intersect-p hons-dups-p
+                                            hons-member-equal)))))
 
 
 
@@ -596,15 +610,6 @@
 ;; These functions are useful for EMOD compatibility.
 
 
-
-(local
- (defthm hons-remove-duplicates1-acl2-count
-   (<= (acl2-count (hons-remove-duplicates1 x y))
-       (acl2-count x))
-   :hints(("Goal" :in-theory (e/d (acl2-count))))
-   :rule-classes :linear))
-
-
 ;; Removes duplicate occurrences in the module and reduces the fields to the
 ;; subset that have semantic meaning in ESIM-FAIG (mostly.  Actually, :n and
 ;; :full-s are unused and :s is unused in non-primitives.)
