@@ -6,15 +6,25 @@
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 
 
 ; stv-compile.lisp -- compile a symbolic test vector into a compiled-stv that
@@ -31,7 +41,7 @@
 (include-book "centaur/misc/tuplep" :dir :system)
 (include-book "std/util/defmvtypes" :dir :system)
 (include-book "std/util/defprojection" :dir :system)
-(include-book "str/natstr" :dir :system)
+(include-book "std/strings/decimal" :dir :system)
 (include-book "std/lists/final-cdr" :dir :system)
 (include-book "centaur/vl/util/defs" :dir :system)
 (local (include-book "../esim-sexpr-support-thms"))
@@ -181,10 +191,10 @@ turned into a list like @('(|opcode[0]| ... |opcode[n]|)').</li>
         (mv (bool-to-4v-sexpr-lst (int-to-v entry width)) gensyms usersyms))
 
        ((when (eq entry 'x))
-        (mv (repeat *4vx-sexpr* width) gensyms usersyms))
+        (mv (replicate width *4vx-sexpr*) gensyms usersyms))
 
        ((when (eq entry :ones))
-        (mv (repeat *4vt-sexpr* width) gensyms usersyms))
+        (mv (replicate width *4vt-sexpr*) gensyms usersyms))
 
        ((when (eq entry '~))
         (or (= width 1)
@@ -216,7 +226,7 @@ turned into a list like @('(|opcode[0]| ... |opcode[n]|)').</li>
         (raise "Phase ~x0 for ~x1: value ~x2 is not legal for input lines of ~
                 symbolic test vectors.  See :xdoc symbolic-test-vector-format ~
                 for help." pnum name entry)
-        (mv (repeat *4vx-sexpr* width) gensyms usersyms))
+        (mv (replicate width *4vx-sexpr*) gensyms usersyms))
 
        (my-syms (stv-gensyms (symbol-name entry) width))
        (look    (hons-get entry usersyms)))
@@ -671,36 +681,36 @@ and which bit of which simulation variable the name corresponds to.</p>"
             (raise "Phase ~x0 for ~x1: value ~x2 is too wide to fit in ~x3 ~
                     bits!" pnum name entry width))
         (mv (bool-to-4v-sexpr-lst (int-to-v entry width))
-            (repeat *4vt-sexpr* width)
+            (replicate width *4vt-sexpr*)
             '_
             in-usersyms
             out-usersyms))
 
        ((when (eq entry 'x))
-        (mv (repeat *4vx-sexpr* width)
-            (repeat *4vt-sexpr* width)
+        (mv (replicate width *4vx-sexpr*)
+            (replicate width *4vt-sexpr*)
             '_
             in-usersyms
             out-usersyms))
 
        ((when (eq entry :ones))
-        (mv (repeat *4vt-sexpr* width)
-            (repeat *4vt-sexpr* width)
+        (mv (replicate width *4vt-sexpr*)
+            (replicate width *4vt-sexpr*)
             '_
             in-usersyms
             out-usersyms))
 
        ((when (eq entry '~))
         (raise "Phase ~x0 for ~x1: value ~~ is not legal on overrides." pnum name)
-        (mv (repeat *4vx-sexpr* width)
-            (repeat *4vt-sexpr* width)
+        (mv (replicate width *4vx-sexpr*)
+            (replicate width *4vt-sexpr*)
             '_
             in-usersyms
             out-usersyms))
 
        ((when (eq entry '_))
-        (mv (repeat *4vx-sexpr* width)
-            (repeat *4vf-sexpr* width)
+        (mv (replicate width *4vx-sexpr*)
+            (replicate width *4vf-sexpr*)
             '_
             in-usersyms
             out-usersyms))
@@ -710,8 +720,8 @@ and which bit of which simulation variable the name corresponds to.</p>"
         (raise "Phase ~x0 for ~x1: value ~x2 is not legal for override lines of ~
                 symbolic test vectors.  See :xdoc symbolic-test-vector-format ~
                 for help." pnum name entry)
-        (mv (repeat *4vx-sexpr* width)
-            (repeat *4vt-sexpr* width)
+        (mv (replicate width *4vx-sexpr*)
+            (replicate width *4vt-sexpr*)
             '_
             in-usersyms
             out-usersyms))
@@ -732,7 +742,7 @@ and which bit of which simulation variable the name corresponds to.</p>"
                 ~x4-bit wire." pnum name entry width (len (cdr in-look))))
 
     (mv my-syms
-        (repeat *4vt-sexpr* width)
+        (replicate width *4vt-sexpr*)
         my-syms
         (if in-look in-usersyms (hons-acons entry my-syms in-usersyms))
         (hons-acons entry my-syms out-usersyms))))
@@ -1038,7 +1048,7 @@ collect and how we want to name them.</p>"
        ;;          variables (from :output and :internal lines).  Illegally ~
        ;;          reused variables: ~x0" illegally-reused-simvars))
 
-       (all-in-bits (vl::append-domains-exec in-usersyms gensyms))
+       (all-in-bits (vl::append-alist-vals-exec in-usersyms gensyms))
        ((unless (uniquep all-in-bits))
         ;; It's hard to imagine this happening, but if somehow the user gave an
         ;; input simulation variable name that clashed with a gensym, it'd be
