@@ -6,15 +6,25 @@
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 
 
 ; stv-doc.lisp -- documentation generation for STVs
@@ -24,10 +34,10 @@
 (in-package "ACL2")
 (include-book "stv-util")
 (include-book "stv-widen")
-(include-book "str/stringify" :dir :system)
+(include-book "std/strings/stringify" :dir :system)
 (include-book "centaur/vl/util/print-htmlencode" :dir :system)
 (local (include-book "std/typed-lists/character-listp" :dir :system))
-(local (include-book "str/explode-nonnegative-integer" :dir :system))
+(local (include-book "std/strings/explode-nonnegative-integer" :dir :system))
 
 (defsection stv-doc
   :parents (symbolic-test-vectors)
@@ -35,8 +45,7 @@
 
   :long "<p>Symbolic test vectors are integrated into @(see xdoc) so that you
 can generate attractive explanations of your setup.  This is often useful when
-communicating with logic designers.  For an example, see @(see counter-run) in
-the @(see esim-tutorial).</p>
+communicating with logic designers.</p>
 
 <p><b>NOTE</b>: the topics here cover how we generate this documentation.  If
 you just want to document your own STVs, you don't need to know about any of
@@ -51,6 +60,17 @@ generally just expect things to be well-formed at this point.</p>
 and is not supported by tools like @(':xdoc').  How these new tags get rendered
 into HTML is controlled by, e.g., @('xdoc/fancy/render.xsl').</p>")
 
+
+(local (defthm natp-of-vl-html-encode-string-aux
+         (natp (mv-nth 0 (vl::VL-HTML-ENCODE-STRING-AUX X N XL COL TABSIZE ACC)))
+         :rule-classes :type-prescription))
+
+(local (defthm character-listp-of-vl-html-encode-string-aux
+         (implies (character-listp acc)
+                  (character-listp (mv-nth 1 (vl::VL-HTML-ENCODE-STRING-AUX X N XL COL TABSIZE ACC))))))
+
+(local (in-theory (disable vl::vl-html-encode-string-aux)))
+
 (define stv-name-bits-to-xml ((bits true-listp)
                               (col  natp)
                               acc)
