@@ -6,15 +6,25 @@
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 
 
 ; stv-expand.lisp -- expansion of names and hierarchical names for STVs
@@ -37,7 +47,7 @@
                (lsb? vl::maybe-natp :rule-classes :type-prescription))
   :parents (stv-expand)
   :short "Match an expression with an optional bit- or part-select."
-  (b* (((unless (vl::vl-nonatom-p x))
+  (b* (((when (vl::vl-atom-p x))
         (mv nil x nil nil))
        (op   (vl::vl-nonatom->op x))
        (args (vl::vl-nonatom->args x))
@@ -54,7 +64,7 @@
                   x nil nil)))
           (mv nil from (vl::vl-resolved->val msb) (vl::vl-resolved->val lsb))))
 
-       ((when (eq op :vl-bitselect))
+       ((when (eq op :vl-index))
         (b* ((from (first args))
              (bit  (second args))
              ((unless (vl::vl-expr-resolved-p bit))
@@ -268,7 +278,7 @@ make sure that the name refers to valid input or output bits.</p>"
   :returns (mv (instnames true-listp :rule-classes :type-prescription)
                (wirename stringp :rule-classes :type-prescription))
 
-  (b* (((unless (vl::vl-hid-indicies-resolved-p hid))
+  (b* (((unless (vl::vl-hidexpr-resolved-p hid))
         (raise "HID has unresolved indices: ~s0~%" (vl::vl-pps-expr hid))
         (mv nil ""))
        (parts (vl::vl-explode-hid hid))
