@@ -6,15 +6,25 @@
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 
 ; Added 10/17/2013 by Matt K.: Making this hons-only, because the proof of
 ; RETURN-TYPE-OF-STV2C-OUTPUT-STUFF-AUX seems to bog down in ACL2 as opposed to
@@ -281,9 +291,9 @@
        (out-bit-names        (alist-keys relevant-sexpr-alist))
        (sexprs               (alist-vals relevant-sexpr-alist))
        (used-vars            (4v-sexpr-vars-1pass-list sexprs))  ;; a set
-       (bound-vars           (sets::mergesort (alist-keys onoff)))
-       (unbound-vars         (sets::difference used-vars bound-vars))
-       ((unless (sets::empty unbound-vars))
+       (bound-vars           (set::mergesort (alist-keys onoff)))
+       (unbound-vars         (set::difference used-vars bound-vars))
+       ((unless (set::empty unbound-vars))
         (raise "variables used in relevant sexprs, but not bound in onoff: ~x0."
                unbound-vars))
        ;; Else looks okay, go ahead and translate.  We'll at least do
@@ -601,4 +611,4 @@
 #||
 Example:
 (stv2c-main (alu16-test-vector))
-||#
\ No newline at end of file
+||#
