@@ -6,24 +6,34 @@
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 ;
 ; Original author: Jared Davis <jared@centtech.com>
 
 (in-package "GETOPT")
 (include-book "std/util/top" :dir :system)
-(include-book "str/case-conversion" :dir :system)
-(include-book "str/strprefixp" :dir :system)
-(include-book "str/subseq" :dir :system)
-(include-book "str/strpos" :dir :system)
+(include-book "std/strings/case-conversion" :dir :system)
+(include-book "std/strings/strprefixp" :dir :system)
+(include-book "std/strings/subseq" :dir :system)
+(include-book "std/strings/strpos" :dir :system)
 (include-book "parsers")
 (include-book "xdoc/word-wrap" :dir :system)
 (include-book "std/misc/two-nats-measure" :dir :system)
@@ -38,7 +48,7 @@
 <p><b>Getopt</b> is a tool for writing command-line programs in ACL2.  It is
 similar in spirit to <a
 href='http://perldoc.perl.org/Getopt/Long.html'>Getopt::Long</a> for Perl, <a
-href='http://trollop.rubyforge.org/'>Trollop</a> for Ruby, and so on.</p>
+href='https://rubygems.org/gems/trollop/'>Trollop</a> for Ruby, and so on.</p>
 
 <p>We basically extend @(see defaggregate) with a command-line parsing layer.
 This has some nice consequences:</p>
@@ -103,38 +113,7 @@ top book:</p>
 
 <p>After that, the main command is @(see defoptions).</p>
 
-<p>There is also a demo of how to use Getopt: @(see getopt-demo::demo-p).</p>
-
-
-<h3>Copyright Information</h3>
-
-<p>ACL2 Getopt Library<br/>
-Copyright (C) 2013
-<a href=\"http://www.centtech.com\">Centaur Technology</a>.</p>
-
-<p>Contact:</p>
-@({
-Centaur Technology Formal Verification Group
-7600-C N. Capital of Texas Highway, Suite 300
-Austin, TX 78731, USA.
-})
-
-<p>This program is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the Free
-Software Foundation; either version 2 of the License, or (at your option) any
-later version.</p>
-
-<p>This program is distributed in the hope that it will be useful but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
-details.</p>
-
-<p>You should have received a copy of the GNU General Public License along with
-this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
-Street, Suite 500, Boston, MA 02110-1335, USA.</p>
-
-<p>Original author: Jared Davis
-&lt;<a href='mailto:jared@centtech.com'>jared@centtech.com</a>&gt;</p>")
+<p>There is also a demo of how to use Getopt: @(see getopt-demo::demo-p).</p>")
 
 
 (defsection defoptions
