@@ -6,24 +6,34 @@
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 ;
 ; Original author: Sol Swords <sswords@centtech.com>
 
 (in-package "GL")
 (include-book "g-if")
 (include-book "g-primitives-help")
-(include-book "symbolic-arithmetic-fns")
+(include-book "symbolic-arithmetic")
 (include-book "eval-g-base")
-(local (include-book "symbolic-arithmetic"))
+
 (local (include-book "eval-g-base-help"))
 (local (include-book "hyp-fix"))
 (local (include-book "centaur/bitops/ihsext-basics" :dir :system))
@@ -88,7 +98,7 @@
     (bfr-ite-bss
      (car n)
      (bfr-logapp-nus (lnfix w) (s-take w x)
-                 (logapp-uss (ash (lnfix w) 1) (cdr n) (logtail-ns w x)
+                 (logapp-uss (ash (lnfix w) 1) (cdr n) (bfr-logtail-ns w x)
                              y))
      (logapp-uss (ash (lnfix w) 1) (cdr n) x y))))
 
@@ -213,7 +223,7 @@
        ((unless (and nintp-known xintp-known yintp-known))
         (g-apply 'logapp (gl-list n x y)))
        ;; nfix
-       (nbits (bfr-ite-bvv-fn (bfr-and (bfr-not (s-sign nrn)) nintp)
+       (nbits (bfr-ite-bvv-fn (bfr-and (bfr-not (bfr-sign-s nrn)) nintp)
                               nrn nil))
        ;; ifix
        (xbits (bfr-ite-bss-fn xintp xrn nil))
@@ -354,7 +364,7 @@
                                      eval-g-base-alt-def
                                      components-to-number-alt-def
                                      hons-assoc-equal
-                                     sets::double-containment
+                                     set::double-containment
                                      equal-of-booleans-rewrite
                                      bfr-eval-list
                                      acl2::cancel_times-equal-correct
@@ -477,7 +487,6 @@
                                       rationalp-implies-acl2-numberp
                                       hons-assoc-equal
                                       default-car default-cdr
-                                      bfr-eval-list-consts
                                       mv-nth-cons-meta
                                       possibilities-for-x-5
                                       possibilities-for-x-3
@@ -512,7 +521,7 @@
        ((unless iintp-known) (mv t nil)) ;; error
        ;; ifix
        (ibits (bfr-ite-bss-fn iintp irn nil)))
-    (mv nil (mk-g-number ibits))))
+    (mv nil (mk-g-number (rlist-fix ibits)))))
 
 (defthm deps-of-g-ifix-of-number
   (implies (and (not (gobj-depends-on k p i))
@@ -589,7 +598,8 @@
 (def-gobj-dependency-thm maybe-integer
   :hints `(("goal" :induct ,gcall
             :expand (,gcall)
-            :in-theory (disable (:d ,gfn)))))
+            :in-theory (disable (:d ,gfn)
+                                (g-ifix-of-number)))))
 
 
 (def-g-correct-thm maybe-integer eval-g-base
@@ -611,7 +621,6 @@
                                       rationalp-implies-acl2-numberp
                                       hons-assoc-equal
                                       default-car default-cdr
-                                      bfr-eval-list-consts
                                       mv-nth-cons-meta
                                       possibilities-for-x-5
                                       possibilities-for-x-3
