@@ -6,15 +6,25 @@
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 ;
 ; Original author: Sol Swords <sswords@centtech.com>
 
@@ -73,6 +83,7 @@
 (local (bfr-reasoning-mode t))
 
 (defun mk-xes (n i)
+  (declare (xargs :mode :program))
   (if (zp n)
       nil
     (cons (intern$ (str::cat "X" (str::natstr i)) "ACL2")
@@ -80,6 +91,7 @@
 
 (mutual-recursion
  (defun dumb-gify-body (x)
+   (declare (xargs :mode :program))
    (cond ((atom x) `(mv ,x hyp))
          ((eq (car x) 'quote) `(mv ,x hyp))
          ((eq (car x) 'if)
@@ -91,18 +103,18 @@
                (vars (cadr (car x)))
                (body (caddr (car x))))
             `(b* ,(pairlis$ 
-                   (pairlis$ (acl2::repeat 'mv (len vars))
+                   (pairlis$ (acl2::replicate (len vars) 'mv)
                              (pairlis$ vars
-                                       (pairlis$ (acl2::repeat 'hyp (len vars))
+                                       (pairlis$ (acl2::replicate (len vars) 'hyp)
                                                  nil)))
                    (pairlis$ result-exprs nil))
                ,(dumb-gify-body body))))
          (t (b* ((result-exprs (dumb-gify-body-lst (cdr x)))
                  (vars (mk-xes (len (cdr x)) 0)))
               `(b* ,(pairlis$ 
-                     (pairlis$ (acl2::repeat 'mv (len vars))
+                     (pairlis$ (acl2::replicate (len vars) 'mv)
                                (pairlis$ vars
-                                         (pairlis$ (acl2::repeat 'hyp (len vars))
+                                         (pairlis$ (acl2::replicate (len vars) 'hyp)
                                                    nil)))
                      (pairlis$ result-exprs nil))
                  (glr ,(car x)
@@ -194,7 +206,7 @@
                   :hints `(("Goal" :in-theory
                             (disable* ,gfn
                                       mv-nth-cons-meta
-                                      sets::double-containment
+                                      set::double-containment
                                       equal-of-booleans-rewrite
                                       bfr-assume-correct
                                       (:rules-of-class :forward-chaining :here)
