@@ -6,15 +6,25 @@
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 ;
 ; Original author: Sol Swords <sswords@centtech.com>
 
@@ -1055,7 +1065,7 @@
      (implies (gobj-listp gobj)
               (gobj-listp (acl2::take n gobj)))
      :hints(("Goal" :in-theory (enable gobj-listp acl2::take-redefinition
-                                       acl2::repeat))))
+                                       acl2::replicate))))
 
    (defun count-down2-cdr (n m l)
      (if (zp n)
@@ -1232,7 +1242,7 @@
                               n a b c))))
             (and stable-under-simplificationp
                  '(:in-theory (e/d (my-run-gified-ev-constraint-0
-                                    acl2::repeat))))))
+                                    acl2::replicate))))))
 
    (encapsulate nil
      (local (defthm nthcdr-is-nil
@@ -1318,29 +1328,6 @@
          (clauses (list* geval-thm eval-thm clauses)))
       (mv nil clauses))))
 
-#!ACL2
-(progn
-  (defun access-b*-bindings (recname var pairs)
-    (if (atom pairs)
-        nil
-      (cons
-       (if (atom (car pairs))
-           (list (car pairs) `(acl2::access ,recname ,var
-                                            ,(intern-in-package-of-symbol
-                                             (symbol-name (car pairs))
-                                             :keyword)))
-         (list (caar pairs) `(acl2::access ,recname ,var
-                                           ,(intern-in-package-of-symbol
-                                            (symbol-name (cadar pairs))
-                                            :keyword))))
-       (access-b*-bindings recname var (cdr pairs)))))
-
-  (def-b*-binder
-    access
-    `(b* ,(access-b*-bindings (car args) (car forms) (cdr args))
-       ,rest-expr)))
-
-
 (defun ev-constraint-for-search (lemmas hyp-terms ev-term)
   (if (atom lemmas)
       nil
@@ -1402,7 +1389,7 @@
                         (:META ACL2::CANCEL_TIMES-EQUAL-CORRECT)
                         (:META ACL2::CANCEL_PLUS-EQUAL-CORRECT)
                         ; (:REWRITE RUN-GIFIED-EV-CONSTRAINT-3)
-                        (:REWRITE ACL2::SYMBOLP-ASSOC-EQUAL)
+                        ;; (:REWRITE ACL2::SYMBOLP-ASSOC-EQUAL)
                         (:DEFINITION ACL2::LIST-FIX)
                         (:REWRITE GEVAL-LIST-DEF-THM-CORRECT)
                         (:DEFINITION SYMBOL-LISTP)
