@@ -5,15 +5,25 @@
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 ;
 ; Original author: Sol Swords <sswords@centtech.com>
 
@@ -30,7 +40,7 @@
 
 (local (in-theory (disable append-of-nil)))
 (local (in-theory (disable all-boundp
-                           sets::double-containment
+                           set::double-containment
                            all-keys-bound)))
 
 (defxdoc contextual-rewriting
@@ -190,6 +200,14 @@ when it becomes available.
            (symbol-listp (strip-cars x)))
   :hints(("Goal" :in-theory (enable symbol-alistp))))
 
+
+(local (defthm pseudo-term-substp-of-append
+         (implies (and (pseudo-term-substp a)
+                       (pseudo-term-substp b))
+                  (pseudo-term-substp (append a b)))
+         :hints(("Goal" :in-theory (enable pseudo-term-substp)))))
+
+
 (defsection mx-relieve-hyp
   (local (defthm symbol-alistp-impl-true-listp
            (implies (symbol-alistp x)
@@ -241,7 +259,7 @@ when it becomes available.
                                      (not (assoc-eq nil val))
                                      (not (intersectp-eq (strip-cars val)
                                                          (strip-cars alist)))
-                                     (pseudo-term-val-alistp val)))))
+                                     (pseudo-term-substp val)))))
           (mv nil alist)))
       (mv t (if (eq syntaxp-fn 'bind-free)
                 (append val alist)
@@ -291,7 +309,9 @@ when it becomes available.
         (mx-relieve-hyp hyp alist rune target n mfc state)
         (implies ok
                  (all-keys-bound (simple-term-vars hyp) new-alist)))
-      :hints(("Goal" :in-theory (enable all-keys-bound)))))
+      :hints(("Goal" :in-theory (enable all-keys-bound
+                                        simple-term-vars
+                                        simple-term-vars-lst)))))
 
 
   (defthm hons-assoc-equal-in-ctx-ev-alist
@@ -336,7 +356,9 @@ when it becomes available.
       :hints ((and stable-under-simplificationp
                    '(:in-theory (enable ctx-ev-of-fncall-args
                                         sub-alistp-hons-assoc-equal
-                                        all-keys-bound))))
+                                        simple-term-vars-lst
+                                        all-keys-bound)
+                     :expand ((simple-term-vars x)))))
       :flag substitute-into-term)
     (defthm ctx-ev-lst-extension-when-vars-bound
       (implies (and (all-keys-bound (simple-term-vars-lst x) a)
@@ -345,6 +367,7 @@ when it becomes available.
                (equal (equal (ctx-ev-lst x b)
                              (ctx-ev-lst x a))
                       t))
+      :hints ('(:expand ((simple-term-vars-lst x))))
       :flag substitute-into-list))
 
   (defthm-substitute-into-term-flag
@@ -355,17 +378,19 @@ when it becomes available.
                       (ctx-ev x a)))
       :hints ((and stable-under-simplificationp
                    '(:in-theory (enable ctx-ev-of-fncall-args
-                                        all-keys-bound))))
+                                        all-keys-bound)
+                     :expand ((simple-term-vars x)))))
       :flag substitute-into-term)
     (defthm ctx-ev-lst-extension-append-when-vars-bound
       (implies (and (all-keys-bound (simple-term-vars-lst x) a)
                     (pseudo-term-listp x))
                (equal (ctx-ev-lst x (append a b))
                       (ctx-ev-lst x a)))
+      :hints ('(:expand ((simple-term-vars-lst x))))
       :flag substitute-into-list))
 
   (local (in-theory (disable symbol-alistp magic-ev
-                             pseudo-term-val-alistp)))
+                             pseudo-term-substp)))
 
   (defthm ctx-ev-mx-relieve-hyp-correct
     (mv-let (ok new-alist)
@@ -402,9 +427,9 @@ when it becomes available.
              (not (assoc nil (mv-nth 1 (mx-relieve-hyp
                                         hyp alist rune target n mfc state))))))
 
-  (defthm pseudo-term-val-alistp-mx-relieve-hyp
-    (implies (pseudo-term-val-alistp alist)
-             (pseudo-term-val-alistp (mv-nth 1 (mx-relieve-hyp
+  (defthm pseudo-term-substp-mx-relieve-hyp
+    (implies (pseudo-term-substp alist)
+             (pseudo-term-substp (mv-nth 1 (mx-relieve-hyp
                                                 hyp alist rune target n mfc
                                                 state))))))
 
@@ -477,9 +502,9 @@ when it becomes available.
              (alistp (mv-nth 1 (mx-relieve-hyps
                                        hyps alist rune target n mfc state)))))
 
-  (defthm pseudo-term-val-alistp-mx-relieve-hyps
-    (implies (pseudo-term-val-alistp alist)
-             (pseudo-term-val-alistp (mv-nth 1 (mx-relieve-hyps
+  (defthm pseudo-term-substp-mx-relieve-hyps
+    (implies (pseudo-term-substp alist)
+             (pseudo-term-substp (mv-nth 1 (mx-relieve-hyps
                                                 hyps alist rune target n mfc
                                                 state)))))
 
@@ -495,9 +520,11 @@ when it becomes available.
   (defthm-substitute-into-term-flag
     (defthm nil-not-member-simple-term-vars
       (not (member nil (simple-term-vars x)))
+      :hints ('(:expand ((simple-term-vars x))))
       :flag substitute-into-term)
     (defthm nil-not-member-simple-term-vars-lst
       (not (member nil (simple-term-vars-lst x)))
+      :hints ('(:expand ((simple-term-vars-lst x))))
       :flag substitute-into-list))
 
   (defthm all-vars-bound-when-sub-alistp-mx-relieve-hyps
@@ -514,7 +541,8 @@ when it becomes available.
       (mx-relieve-hyps hyps alist rune target n mfc state)
       (implies ok
                (all-keys-bound (simple-term-vars-lst hyps) new-alist)))
-    :hints(("Goal" :in-theory (enable all-keys-bound))))
+    :hints(("Goal" :in-theory (enable all-keys-bound
+                                      simple-term-vars-lst))))
 
   (defthm hons-assoc-equal-ctx-ev-alist
     (implies (alistp x)
@@ -610,7 +638,9 @@ when it becomes available.
   (defthm all-keys-bound-of-conjoin
     (implies (all-keys-bound (simple-term-vars-lst hyps) a)
              (all-keys-bound (simple-term-vars (conjoin hyps)) a))
-    :hints(("Goal" :in-theory (enable conjoin all-keys-bound))))
+    :hints(("Goal" :in-theory (enable conjoin all-keys-bound
+                                      simple-term-vars-lst
+                                      simple-term-vars))))
 
   (defthm ctx-ev-mx-relieve-hyps-correct1
     (mv-let (ok new-alist)
@@ -734,10 +764,17 @@ when it becomes available.
 (local (defthm subsetp-of-union
          (subsetp-equal a (union-equal a b))))
 
+(defthm symbol-<-merge-under-set-equiv
+  (set-equiv (symbol-<-merge x y)
+             (append x y))
+  :hints((set-reasoning)))
+
 (defthm simple-term-vars-of-conjoin
   (subsetp-equal (simple-term-vars (conjoin x))
                  (simple-term-vars-lst x))
-  :hints(("Goal" :in-theory (enable conjoin))))
+  :hints(("Goal" :in-theory (enable conjoin
+                                    simple-term-vars-lst
+                                    simple-term-vars))))
 
 (defthm all-keys-bound-when-subsetp
   (implies (and (subsetp-equal a b)
@@ -911,8 +948,9 @@ when it becomes available.
 ;;                 (assoc-equal var alist))))
 
 (defthm pseudo-termp-hons-assoc
-  (implies (pseudo-term-val-alistp x)
-           (pseudo-termp (cdr (hons-assoc-equal k x)))))
+  (implies (pseudo-term-substp x)
+           (pseudo-termp (cdr (hons-assoc-equal k x))))
+  :hints(("Goal" :in-theory (enable pseudo-term-substp))))
 
 
 
@@ -1036,7 +1074,8 @@ when it becomes available.
                     (ctx-ev x (cons (cons var x1) a))))
     :hints ((and stable-under-simplificationp
                  '(:in-theory (enable ctx-ev-of-fncall-args
-                                      all-keys-bound))))
+                                      all-keys-bound)
+                   :expand ((simple-term-vars x)))))
     :flag substitute-into-term)
   (defthm ctx-ev-lst-when-all-identities-except-x
     (implies (and (pseudo-term-listp x)
@@ -1045,6 +1084,7 @@ when it becomes available.
              (equal (ctx-ev-lst x (cons (cons var x1)
                                         (ctx-ev-alist subst a)))
                     (ctx-ev-lst x (cons (cons var x1) a))))
+    :hints ('(:expand ((simple-term-vars-lst x))))
     :flag substitute-into-list))
 
 
@@ -1074,7 +1114,9 @@ when it becomes available.
     (implies (and ok
                   (all-keys-bound (simple-term-vars pat) alist))
              (equal subst alist)))
-  :hints(("Goal" :in-theory (enable unify-const all-keys-bound))))
+  :hints(("Goal" :in-theory (enable unify-const all-keys-bound
+                                    simple-term-vars-lst)
+          :expand ((simple-term-vars pat)))))
 
 
 (defthm-simple-one-way-unify-flag
@@ -1086,7 +1128,8 @@ when it becomes available.
                (equal subst alist)))
     :hints ((and stable-under-simplificationp
                  '(:expand ((:free (x) (simple-one-way-unify pat x alist))
-                            (:free (x) (simple-one-way-unify nil x alist)))
+                            (:free (x) (simple-one-way-unify nil x alist))
+                            (simple-term-vars pat))
                    :in-theory (enable all-keys-bound))))
     :flag simple-one-way-unify)
   (defthm one-way-unify-lst-reduce-when-all-keys-bound
@@ -1096,7 +1139,8 @@ when it becomes available.
                     (all-keys-bound (simple-term-vars-lst pat) alist))
                (equal subst alist)))
     :hints ((and stable-under-simplificationp
-                 '(:expand ((:free (x) (simple-one-way-unify-lst pat x alist))))))
+                 '(:expand ((:free (x) (simple-one-way-unify-lst pat x alist))
+                            (simple-term-vars-lst pat)))))
     :flag simple-one-way-unify-lst)
   :hints (("goal" :induct (simple-one-way-unify-flag flag pat x alist))))
 
@@ -1151,7 +1195,8 @@ when it becomes available.
                (set-equiv (alist-keys subst)
                            (append (simple-term-vars pat)
                                    (alist-keys subst0)))))
-    :hints(("Goal" :in-theory (enable unify-const)
+    :hints(("Goal" :in-theory (enable unify-const simple-term-vars
+                                      simple-term-vars-lst)
             :induct t))))
 
 (defthm-simple-one-way-unify-flag
@@ -1164,7 +1209,8 @@ when it becomes available.
                                    (alist-keys subst0)))))
     :hints ((and stable-under-simplificationp
                  '(:expand ((:free (x subst)
-                             (simple-one-way-unify pat x subst)))
+                             (simple-one-way-unify pat x subst))
+                            (simple-term-vars pat))
                    :in-theory (enable all-keys-bound))))
     :flag simple-one-way-unify)
   (defthm alist-keys-of-simple-one-way-unify-lst
@@ -1176,7 +1222,8 @@ when it becomes available.
                                    (alist-keys subst0)))))
     :hints ((and stable-under-simplificationp
                  '(:expand ((:free (x subst)
-                             (simple-one-way-unify-lst pat x subst))))))
+                             (simple-one-way-unify-lst pat x subst))
+                            (simple-term-vars-lst pat)))))
     :flag simple-one-way-unify-lst)
   :hints (("Goal" :induct (simple-one-way-unify-flag flag pat x subst0))))
 
@@ -1307,7 +1354,7 @@ when it becomes available.
            :in-theory (e/d ((:induction unify-const))
                            (pseudo-termp
                             unify-const-redef
-                            sets::double-containment
+                            set::double-containment
                             default-car default-cdr
                             append len hons-assoc-equal
                             alists-compatible-when-sub-alistp
@@ -1323,8 +1370,9 @@ when it becomes available.
                            (unify-const-redef nil x)))))
           (alist-reasoning)))
 
-(defthm pseudo-term-val-alistp-of-unify-const-redef
-  (pseudo-term-val-alistp (mv-nth 1 (unify-const-redef pat const))))
+(defthm pseudo-term-substp-of-unify-const-redef
+  (implies (pseudo-termp pat)
+           (pseudo-term-substp (mv-nth 1 (unify-const-redef pat const)))))
         
 (in-theory (disable unify-const-redef))
 
@@ -1434,10 +1482,6 @@ when it becomes available.
          (list (subst-ind t (car pat) (car term))
                (subst-ind nil (cdr pat) (cdr term)))))))
 
-  (defthm pseudo-term-listp-cdr
-    (implies (and (pseudo-termp x)
-                  (not (eq (car x) 'quote)))
-             (pseudo-term-listp (cdr x))))
 
   (encapsulate nil
     (local (defthm equal-of-len
@@ -1462,7 +1506,10 @@ when it becomes available.
                                         (ctx-ev-alist subst a)))
                              (ctx-ev-alist subst a)))
                       const))
-      :hints(("Goal" :in-theory (enable unify-const all-keys-bound)))))
+      :hints(("Goal" :in-theory (enable unify-const
+                                        simple-term-vars
+                                        simple-term-vars-lst
+                                        all-keys-bound)))))
 
 
   (defthm-simple-one-way-unify-flag
@@ -1480,7 +1527,8 @@ when it becomes available.
                              (ctx-ev-alist subst a)))
                       (ctx-ev term (ctx-ev-alist subst a))))
       :hints ('(:do-not-induct t
-                :expand ((:free (term) (simple-one-way-unify pat term alist))))
+                :expand ((:free (term) (simple-one-way-unify pat term alist))
+                         (simple-term-vars pat)))
               (and stable-under-simplificationp
                    '(:in-theory (enable ctx-ev-of-fncall-args all-keys-bound))))
       :flag simple-one-way-unify)
@@ -1500,7 +1548,8 @@ when it becomes available.
       :hints ('(:expand ((:free (term) (simple-one-way-unify-lst nil term
                                                                  alist))
                          (:free (term) (simple-one-way-unify-lst pat term
-                                                                 alist)))
+                                                                 alist))
+                         (simple-term-vars-lst pat))
                 :in-theory (enable ctx-ev-alist)))
       :flag simple-one-way-unify-lst)))
 
