@@ -6,21 +6,33 @@
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 ;
 ; Original author: Sol Swords <sswords@centtech.com>
 
 ; defapply.lisp -- introduces an executable apply function that recognizes a
 ; specified set of function symbols.
 
+; cert_param: (non-acl2r)
+
 (in-package "ACL2")
 
 (local
@@ -859,7 +871,7 @@ The function ~x0 is missing its ~x1 property; perhaps it is not defined.~%"
                                   ;;  subsetp-car-member
                                   ;;  member-equal
                                   ;;  _name_-ev-concrete
-                                  ;;  sets::double-containment
+                                  ;;  set::double-containment
                                   ;;  default-cdr
                                   ;;  (_name_-apply)
                                   ;;  (:rules-of-class :definition :here))
@@ -897,10 +909,14 @@ The function ~x0 is missing its ~x1 property; perhaps it is not defined.~%"
  (progn
    ;; (defapply/ev/concrete-ev mything2 (if len mv-list binary-append))
 
-
    (make-event
     `(defapply/ev/concrete-ev
-       everything ,(nthcdr 400 *all-logic-function-syms*)))))
+       everything
+
+       ;; Note: Earlier this was (nthcdr 400 *all-logic-function-syms*), but
+       ;; Allegro CL died with a large rewrite stack.
+
+       ,(take 100 *all-logic-function-syms*)))))
 
 ;;  100:    2.02      97M
 ;;  200:    4.02     204M
