@@ -1,26 +1,38 @@
-; Quicklisp setup for Centaur books
-; Copyright (C) 2008-2013 Centaur Technology
+; ACL2 Quicklisp Interface
+; Copyright (C) 2008-2014 Centaur Technology
 ;
 ; Contact:
 ;   Centaur Technology Formal Verification Group
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 ;
 ; Original author: Jared Davis <jared@centtech.com>
 
 (in-package "ACL2")
-(include-book "tools/include-raw" :dir :system)
-; (depends-on "setup.lisp")
+(include-book "base")
+(include-book "bordeaux")
+(include-book "hunchentoot")
+(include-book "osicat")
 
 (defsection quicklisp
   :parents (acl2::interfacing-tools)
@@ -59,7 +71,7 @@ documentation for all of the libraries in Quicklisp.</li>
 course requires a <see topic='@(url defttag)'>trust tag</see>:</p>
 
 @({
-    (include-book \"centaur/quicklisp/top\" :dir :system)
+    (include-book \"centaur/quicklisp/base\" :dir :system)
 })
 
 <p><b>NOTE:</b> this book isn't automatically certified when you just run
@@ -68,12 +80,16 @@ that you want to use Quicklisp&mdash;e.g.,</p>
 
 @({
     cd [...]/acl2-sources/books
-    make USE_QUICKLISP=1
+    make USE_QUICKLISP=1 ...
 })
 
-<p>The Quicklisp book really is just a way to get Quicklisp itself loaded into
+<p>The @('base') book really is just a way to get Quicklisp itself loaded into
 an ACL2 session.  It doesn't load anything libraries.</p>
 
+<p>Other files in the Quicklisp directory are named after the Common Lisp
+libraries they load, and the @('top') book loads all of the libraries that we
+have been currently using.</p>
+
 
 <h3>Practical Howto</h3>
 
@@ -87,9 +103,8 @@ href='http://common-lisp.net/project/cl-json/'>CL-JSON</a> library.</p>
     ; ** my-book.lisp
     (in-package \"MY-PKG\")
 
-    ; ** Load Quicklisp, tell cert.pl this book needs Quicklisp support
-    (include-book \"centaur/quicklisp/top\" :dir :system)
-    ;; cert_param: (uses-quicklisp)
+    ; ** Load Quicklisp
+    (include-book \"centaur/quicklisp/base\" :dir :system)
 
     ; ** [OPTIONAL] develop a logical story so you can use the
     ; ** library from proper ACL2 functions...
@@ -113,8 +128,9 @@ href='http://common-lisp.net/project/cl-json/'>CL-JSON</a> library.</p>
 
 <p>You usually need to use the @(':host-readtable') option because real Common
 Lisp libraries will use things (packages, floats, etc.) that ACL2's reader will
-reject.  You usually need to use @(':do-not-compile') because BOZO why?  I
-don't know, but it never *!@$! works if you try to compile it.</p>
+reject.  You usually need to use @(':do-not-compile') because otherwise you
+tend to not have the right packages around at compile time.  You may be able to
+work around that using @('eval-when').</p>
 
 <p>The corresponding raw file, then would look something like this:</p>
 
@@ -130,24 +146,3 @@ don't know, but it never *!@$! works if you try to compile it.</p>
     (defun foo (x y z)
       ...)
 })")
-
-(defttag :quicklisp)
-
-(value-triple (cw "~%~%~
-***********************************************************************~%~
-*****                                                             *****~%~
-*****                  IF YOUR BUILD FAILS                        *****~%~
-*****                                                             *****~%~
-***** The include-raw form here can fail if you try to certify    *****~%~
-***** this book without first getting quicklisp installed.  You   *****~%~
-***** need to run 'make' first.                                   *****~%~
-*****                                                             *****~%~
-***** See books/centaur/README.html for detailed directions for   *****~%~
-***** properly building the Centaur books.                        *****~%~
-*****                                                             *****~%~
-***********************************************************************~%~
-~%"))
-
-(include-raw "setup.lisp"
-             :do-not-compile t
-             :host-readtable t)
\ No newline at end of file
