@@ -10,8 +10,6 @@
 ; License along with this program; if not, write to the Free Software
 ; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 
-; cert_param: (uses-glucose)
-
 (in-package "ACL2")
 
 (include-book "common")
@@ -21,7 +19,8 @@
    :start-files (list "idiv.v")))
 
 (defconst *divide-translation*
-  (VL::VL-TRANSLATION->MODS *DIVIDE-MODULES*))
+  (vl::vl-design->mods
+   (vl::vl-translation->good *DIVIDE-MODULES*)))
 
 ; (vl::vl-modulelist-flat-warnings
 ;  (vl::vl-translation->failmods *divide-modules*))
