@@ -1,36 +1,34 @@
 ; VL Verilog Toolkit
-; Copyright (C) 2008-2013 Centaur Technology
+; Copyright (C) 2008-2014 Centaur Technology
 ;
 ; Contact:
 ;   Centaur Technology Formal Verification Group
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 ;
 ; Original author: Jared Davis <jared@centtech.com>
 
 (in-package "VL")
-
-(make-event
-
-; Disabling waterfall parallelism because this book allegedly uses memoization
-; while performing its proofs.
-
- (if (and (ACL2::hons-enabledp state)
-          (f-get-global 'ACL2::parallel-execution-enabled state))
-     (er-progn (set-waterfall-parallelism nil)
-               (value '(value-triple nil)))
-   (value '(value-triple nil))))
-
 (include-book "../loader/loader")
 
 (include-book "../lint/bit-use-set")
@@ -41,7 +39,6 @@
 (include-book "../lint/xf-drop-user-submodules")
 (include-book "../lint/xf-lint-stmt-rewrite")
 (include-book "../lint/xf-remove-toohard")
-(include-book "../lint/xf-undefined-names")
 (include-book "../lint/xf-suppress-warnings")
 
 (include-book "../checkers/condcheck")
@@ -58,10 +55,10 @@
 
 (include-book "../transforms/cn-hooks")
 (include-book "../transforms/xf-argresolve")
-(include-book "../transforms/xf-array-indexing")
 (include-book "../transforms/xf-assign-trunc")
 (include-book "../transforms/xf-blankargs")
 (include-book "../transforms/xf-clean-params")
+(include-book "../transforms/xf-clean-warnings")
 (include-book "../transforms/xf-drop-blankports")
 (include-book "../transforms/xf-expr-split")
 (include-book "../transforms/xf-expand-functions")
@@ -70,23 +67,26 @@
 (include-book "../transforms/xf-orig")
 (include-book "../transforms/xf-oprewrite")
 (include-book "../transforms/xf-portdecl-sign")
+(include-book "../transforms/xf-resolve-indexing")
 (include-book "../transforms/xf-resolve-ranges")
 (include-book "../transforms/xf-replicate-insts")
+(include-book "../transforms/xf-selresolve")
 (include-book "../transforms/xf-sizing")
 (include-book "../transforms/xf-unparameterize")
-(include-book "../transforms/xf-unused-reg")
+(include-book "../transforms/xf-unused-vars")
 
 (include-book "../../misc/sneaky-load")
 
 (include-book "../mlib/json")
-(local (include-book "../mlib/modname-sets"))
-(local (include-book "../util/arithmetic"))
-(local (include-book "../util/osets"))
 
 (include-book "centaur/getopt/top" :dir :system)
 (include-book "std/io/read-file-characters" :dir :system)
 (include-book "progutils")
 
+(local (include-book "../mlib/modname-sets"))
+(local (include-book "../util/arithmetic"))
+(local (include-book "../util/osets"))
+(local (non-parallel-book))
 
 (defsection lint
   :parents (vl)
@@ -139,6 +139,17 @@ Verilog, reusing much of @(see vl).</p>")
                 :merge acl2::rcons
                 :default '("v"))
 
+   (include-dirs string-listp
+                 :longname "incdir"
+                 :alias #\I
+                 :argname "DIR"
+                 "Control the list of directories for `include files.  You can
+                  give this switch multiple times.  By default, we look only in
+                  the current directory."
+                 :parser getopt::parse-string
+                 :merge acl2::rcons
+                 :default '("."))
+
    (topmods    string-listp
                :longname "topmod"
                :argname "MOD"
@@ -193,6 +204,18 @@ Verilog, reusing much of @(see vl).</p>")
                  sub-expressions."
                 :default 0)
 
+   (edition     vl-edition-p
+                :argname "EDITION"
+                "Which edition of the Verilog standard to implement?
+                 Default: \"SystemVerilog\" (IEEE 1800-2012).  You can
+                 alternately use \"Verilog\" for IEEE 1364-2005, i.e.,
+                 Verilog-2005."
+                :default :system-verilog-2012)
+
+   (strict      booleanp
+                :rule-classes :type-prescription
+                "Disable VL extensions to Verilog.")
+
    (mem         posp
                 :alias #\m
                 :argname "GB"
@@ -206,12 +229,13 @@ Verilog, reusing much of @(see vl).</p>")
                 "Print extra information for debugging."
                 :rule-classes :type-prescription)))
 
-(defsection *vl-lint-help*
+(defval *vl-lint-help*
   :parents (lint)
   :short "Usage message for vl lint."
   :long "@(def *vl-lint-help*)"
+  :showval t
 
-  (defconsts *vl-lint-help* (str::cat "
+  (str::cat "
 vl lint:  A linting tool for Verilog.  Scans your Verilog files for things
           that look like bugs (size mismatches, unused wires, etc.)
 
@@ -221,7 +245,7 @@ Example:  vl lint engine.v wrapper.v core.v \\
 
 Usage:    vl lint [OPTIONS] file.v [file2.v ...]
 
-Options:" *nls* *nls* *vl-lintconfig-usage* *nls*)))
+Options:" *nls* *nls* *vl-lintconfig-usage* *nls*))
 
 
 (define vl-filter-mods-with-good-paramdecls
@@ -298,25 +322,25 @@ shown.</p>"
   :parents (lint)
   :short "Results from running the linter."
   :tag :vl-lintresult
-  ((mods      vl-modulelist-p
+  ((design    vl-design-p
               "Final, transformed list of modules.  Typically this isn't very
                interesting or relevant to anything.")
 
-   (mods0     vl-modulelist-p
+   (design0   vl-design-p
               "Almost: the initial, pre-transformed modules.  The only twist is
                that we have already removed modules that are unnecessary or
                that we wanted to drop; see, e.g., the @('topmods') and
                @('ignore') options of @(see vl-lintconfig-p).  This is used for
                @(see skip-detection), for instance.")
 
-   (mwalist   vl-modwarningalist-p
-              "The main result: binds \"original\" (pre-unparameterization)
-               module names to their warnings.")
+   (reportcard vl-reportcard-p
+               "The main result: binds \"original\" (pre-unparameterization)
+                module names to their warnings.")
 
    (sd-probs  sd-problemlist-p
               "Possible problems noticed by @(see skip-detection).  These are
                in a different format than ordinary @(see warnings), so they
-               aren't included in the @('mwalist').")
+               aren't included in the @('reportcard').")
 
    (dalist    us-dbalist-p
               "Use-set database alist, mapping module names to use-set databases.
@@ -360,20 +384,80 @@ shown.</p>"
                    SUBSETP-EQUAL-OF-VL-MODINSTLIST->MODNAMESS-WHEN-SUBSETP-EQUAL
                    CDR-OF-VL-MODULELIST-DUPERHS-CHECK
                    CDR-OF-VL-MODULELIST-ORIGEXPRS
-                   CDR-OF-VL-MODULELIST-MAKE-ARRAY-INDEXING
                    vl-modulelist-p-of-append
                    NO-DUPLICATESP-EQUAL-OF-APPEND
                    acl2::no-duplicatesp-equal-append-iff
                    mergesort)))
 
-(define run-vl-lint-main ((mods     (and (vl-modulelist-p mods)
-                                         (uniquep (vl-modulelist->names mods))))
-                          (config   vl-lintconfig-p))
+(define vl-design-remove-unnecessary-modules ((keep string-listp)
+                                              (design vl-design-p))
+  :returns (new-design vl-design-p)
+  (b* ((design (vl-design-fix design))
+       (keep   (mbe :logic (if (string-listp keep) keep nil) :exec keep))
+       (keep   (mergesort keep))
+       ((unless keep)
+        ;; Special feature: if the user didn't specify any top modules,
+        ;; then we want to keep everything.
+        design)
+       (mods (mergesort (vl-design->mods design)))
+       ((unless (uniquep (vl-modulelist->names mods)))
+        (raise "Name clash: duplicate module names: ~&0."
+               (duplicated-members (vl-modulelist->names mods)))
+        design)
+       (new-mods (vl-remove-unnecessary-modules keep mods)))
+    (change-vl-design design :mods new-mods)))
+
+
+(define vl-lint-unparam ((good vl-design-p))
+  :returns (good vl-design-p)
+  (b* ((good (vl-design-fix good))
+
+       ;; Subtle thing.  Move any bad modules out of the way before doing
+       ;; unparameterization.
+       ((mv ok-mods bad-mods)
+        (vl-filter-mods-with-good-paramdecls (vl-design->mods good) nil nil))
+
+       (- (or (not bad-mods)
+              (progn$
+               (cw "~%~%Note: deleting ~x0 module~s1 because they include ~
+                    unsupported parameter declarations.~%~%~
+                    Module~s1 being deleted: ~&2~%~%~
+                    Details:~%~%"
+                   (len bad-mods)
+                   (if (vl-plural-p bad-mods) "s" "")
+                   (mergesort (vl-modulelist->names bad-mods)))
+               (vl-print-certain-warnings bad-mods
+                                          (list :vl-bad-paramdecl
+                                                :vl-bad-paramdecls)
+                                          nil))))
+
+       (good (change-vl-design good :mods ok-mods))
+       (good (cwtime (vl-design-drop-missing-submodules good)))
+       (good (vl-design-unparameterize good))
+       (good (change-vl-design good :mods (append-without-guard
+                                           bad-mods (vl-design->mods good)))))
+    (or (uniquep (vl-modulelist->names (vl-design->mods good)))
+        (raise "Programming error.  Expected modules to have unique names ~
+                after vl-unparameterize, but found duplicate modules named ~
+                ~x0.  Please tell Jared."
+               (duplicated-members (vl-modulelist->names (vl-design->mods good)))))
+    good))
+
+(define vl-lint-apply-quiet ((quiet string-listp)
+                             (design vl-design-p))
+  :returns (new-design vl-design-p)
+  (b* ((design (vl-design-fix design))
+       (mods   (vl-design->mods design))
+       (mods   (vl-delete-modules quiet mods)))
+    (change-vl-design design :mods mods)))
+
+(define run-vl-lint-main ((design vl-design-p)
+                          (config vl-lintconfig-p))
+  :guard-debug t
   :returns (result vl-lintresult-p :hyp :fguard)
 
   (b* (((vl-lintconfig config) config)
-
-       (mods (vl-modulelist-drop-user-submodules mods config.dropmods))
+       (design (cwtime (vl-design-drop-user-submodules design config.dropmods)))
 
        ;; You might expect that we'd immediately throw out modules that we
        ;; don't need for topmods.  Historically we did that.  But then we found
@@ -382,154 +466,98 @@ shown.</p>"
        ;; thrown away!  So now we deal with this stuff after HID resolution.
        ;; Except that for skip-detection, we also need to remove them from
        ;; mods0, so do that now:
-       (mods0 (if (not config.topmods)
-                  mods
-                (vl-remove-unnecessary-modules (mergesort config.topmods)
-                                               (mergesort mods))))
-
+       (design0 (vl-design-remove-unnecessary-modules config.topmods design))
 
        (- (cw "~%vl-lint: initial processing...~%"))
-       (mods (cwtime (vl-modulelist-portcheck mods)))
-       (mods (cwtime (vl-modulelist-check-case mods)))
-       (mods (cwtime (vl-modulelist-duperhs-check mods)))
-       (mods (cwtime (vl-modulelist-duplicate-detect mods)))
-       (mods (cwtime (vl-modulelist-condcheck mods)))
-       (mods (cwtime (vl-modulelist-leftright-check mods)))
-       (mods (cwtime (vl-modulelist-drop-missing-submodules mods)))
+       (design (cwtime (vl-design-portcheck design)))
+       (design (cwtime (vl-design-check-case design)))
+       (design (cwtime (vl-design-duperhs-check design)))
+       (design (cwtime (vl-design-duplicate-detect design)))
+       (design (cwtime (vl-design-condcheck design)))
+       (design (cwtime (vl-design-leftright-check design)))
+       (design (cwtime (vl-design-drop-missing-submodules design)))
        ;; BOZO reinstate this??
        ;; (mods (cwtime (vl-modulelist-add-undefined-names mods)))
-       (mods (cwtime (vl-modulelist-portdecl-sign mods)))
-       (mods (cwtime (vl-modulelist-make-array-indexing mods)))
-       (mods (cwtime (vl-modulelist-origexprs mods)))
-       (mods (cwtime (vl-modulelist-check-namespace mods)))
+       (design (cwtime (vl-design-resolve-indexing design)))
+       (design (cwtime (vl-design-portdecl-sign design)))
+       (design (cwtime (vl-design-origexprs design)))
+       (design (cwtime (vl-design-check-namespace design)))
 
        (- (cw "~%vl-lint: processing arguments, parameters...~%"))
-       (mods (cwtime (vl-modulelist-elim-unused-regs mods)))
-       (mods (cwtime (vl-modulelist-argresolve mods)))
-       (mods (cwtime (vl-modulelist-dupeinst-check mods)))
+       (design (cwtime (vl-design-elim-unused-vars design)))
+       (design (cwtime (vl-design-argresolve design)))
+       (design (cwtime (vl-design-dupeinst-check design)))
 
        ;; BOZO not exactly sure where this should go, maybe this will work.
-       (mods (cwtime (vl-modulelist-expand-functions mods)))
+       (design (cwtime (vl-design-expand-functions design)))
 
        ;; BOZO we need to do something to throw away instances with unresolved
        ;; arguments to avoid programming-errors in drop-blankports... and actually
        ;; we hit errors like that later, too.
-       (mods (cwtime (vl-modulelist-drop-blankports mods)))
-       (mods (cwtime (mp-verror-transform-hook mods)))
-       (mods (cwtime (vl-modulelist-follow-hids mods)))
-       (mods (cwtime (vl-modulelist-clean-params mods)))
-       (mods (cwtime (vl-modulelist-check-good-paramdecls mods)))
-
-       ((mv mods bad)
-        (vl-filter-mods-with-good-paramdecls mods nil nil))
-       (- (or (not bad)
-              (progn$
-               (cw "~%~%Note: deleting ~x0 module~s1 because they include ~
-                    unsupported parameter declarations.~%~%~
-                    Module~s1 being deleted: ~&2~%~%~
-                    Details:~%~%"
-                   (len bad)
-                   (if (= (len bad) 1) "" "s")
-                   (mergesort (vl-modulelist->names bad)))
-               (vl-print-certain-warnings bad
-                                          (list :vl-bad-paramdecl
-                                                :vl-bad-paramdecls)
-                                          nil))))
-       (mods (cwtime (vl-modulelist-drop-missing-submodules mods)))
-       (mods (if (and (uniquep (vl-modulelist->names mods))
-                      (vl-modulelist-complete-p mods mods)
-                      (vl-good-paramdecllist-list-p-of-vl-modulelist->paramdecls mods))
-                 mods
-               (er hard? 'vl-lint
-                   "Programming error.  Expected modules to be complete ~
-                    and to have good parameters, but this is not the case. ~
-                    Please tell Jared about this failure.")))
-       ((mv mods failmods) (cwtime (vl-unparameterize mods 30)))
-       (mods (append mods failmods))
+       (design (cwtime (vl-design-drop-blankports design)))
+       (design (cwtime (mp-verror-transform-hook design))) ;; BOZO really keep this???
+       (design (cwtime (vl-design-follow-hids design)))
+       (design (cwtime (vl-design-clean-params design)))
+       (design (cwtime (vl-design-check-good-paramdecls design)))
+       (design (cwtime (vl-lint-unparam design)))
        (- (vl-gc))
 
-       (mods (if (uniquep (vl-modulelist->names mods))
-                 mods
-               (progn$
-                (sneaky-save :bad-names mods)
-                (er hard? 'vl-lint
-                    "Programming error.  Expected modules to have unique ~
-                      names after vl-unparameterize, but found duplicate ~
-                      modules named ~x0.  Please tell Jared."
-                    (duplicated-members (vl-modulelist->names mods))))))
-
-
        (- (cw "~%vl-lint: processing ranges, statements...~%"))
-       (mods (cwtime (vl-modulelist-rangeresolve mods)))
-       (mods (cwtime (vl-modulelist-selresolve mods)))
-       (mods (cwtime (vl-modulelist-check-selfassigns mods)))
-       (mods (cwtime (vl-modulelist-lint-stmt-rewrite mods)))
-       (mods (cwtime (vl-modulelist-stmtrewrite mods 1000)))
-       (mods (cwtime (vl-modulelist-hid-elim mods)))
-
-       (mods (if (uniquep (vl-modulelist->names mods))
-                 mods
-               (progn$
-                (sneaky-save :bad-names mods)
-                (er hard? 'vl-lint
-                    "Programming error.  Expected modules to have unique ~
-                    names after vl-modulelist-hid-elim, but found duplicate ~
-                    modules named ~x0.  Please tell Jared."
-                    (duplicated-members (vl-modulelist->names mods))))))
+       (design (cwtime (vl-design-rangeresolve design)))
+       (design (cwtime (vl-design-selresolve design)))
+       (design (cwtime (vl-design-check-selfassigns design)))
+       (design (cwtime (vl-design-lint-stmt-rewrite design)))
+       (design (cwtime (vl-design-stmtrewrite design 1000)))
+       (design (cwtime (vl-design-hid-elim design)))
 
        ;; Now that HIDs are gone, we can throw away any modules we don't care
        ;; about, if we have been given any topmods.
-       (mods (if (not config.topmods)
-                 mods
-               (vl-remove-unnecessary-modules (mergesort config.topmods)
-                                              (mergesort mods))))
-
+       (design (cwtime (vl-design-drop-user-submodules design config.dropmods)))
 
        ;; BOZO it seems sort of legitimate to do this before sizing, which
        ;; might be nice.  Of course, a more rigorous use/set analysis will
        ;; need to be post-sizing.
        (- (cw "~%vl-lint: finding disconnected wires...~%"))
-       (mods (cwtime (vl-modulelist-remove-toohard mods)))
-       (mods (cwtime (vl-modulelist-find-disconnected mods)))
+       (design (cwtime (vl-design-remove-toohard design)))
+       (design (cwtime (vl-design-find-disconnected design)))
 
        (- (cw "~%vl-lint: processing expressions...~%"))
-       (mods (cwtime (vl-modulelist-oddexpr-check mods)))
-       (mods (cwtime (vl-modulelist-oprewrite mods)))
-       (mods (cwtime (vl-modulelist-exprsize mods)))
-       (mods (cwtime (vl-modulelist-constcheck-hook mods config.cclimit)))
-       (mods (cwtime (vl-modulelist-qmarksize-check mods)))
+       (design (cwtime (vl-design-oddexpr-check design)))
+       (design (cwtime (vl-design-oprewrite design)))
+       (design (cwtime (vl-design-exprsize design)))
+       (design (cwtime (vl-design-constcheck-hook design config.cclimit)))
+       (design (cwtime (vl-design-qmarksize-check design)))
 
        (- (cw "~%vl-lint: finding unused/unset wires...~%"))
        ;; BOZO this probably doesn't quite work here due to replicate not having been done
-       ((mv mods dalist) (cwtime (us-analyze-mods mods)))
+       ((mv design dalist) (cwtime (vl-design-bit-use-set design)))
        (- (vl-gc))
 
        (- (cw "~%vl-lint: processing assignments...~%"))
-       (mods (cwtime (vl-modulelist-split mods)))
-       (mods (cwtime (vl-modulelist-replicate mods)))
-       (mods (cwtime (vl-modulelist-blankargs mods)))
-       (mods (cwtime (vl-modulelist-trunc mods)))
+       (design (cwtime (vl-design-split design)))
+       (design (cwtime (vl-design-replicate design)))
+       (design (cwtime (vl-design-blankargs design)))
+       (design (cwtime (vl-design-trunc design)))
 
        (- (cw "~%vl-lint: finding skipped and multiply driven wires...~%"))
-       ;; NOTE: use mods0, not mods, if you ever want this to finish. :)
-       (sd-probs (cwtime (sd-analyze-modulelist mods0)))
-       (mods     (cwtime (vl-modulelist-multidrive-detect mods)))
+       ;; NOTE: use design0, not design, if you ever want this to finish. :)
+       (sd-probs (cwtime (sd-analyze-design design0)))
+       (design   (cwtime (vl-design-multidrive-detect design)))
 
        (- (cw "~%vl-lint: cleaning up...~%"))
-       (mods     (cwtime (vl-modulelist-clean-warnings mods)))
-       (mods     (cwtime (vl-modulelist-suppress-lint-warnings mods)))
-       (mods     (cwtime (vl-modulelist-lint-ignoreall mods config.ignore)))
-       (mods     (cwtime (vl-delete-modules config.quiet mods)))
+       (design   (cwtime (vl-design-clean-warnings design)))
+       (design   (cwtime (vl-design-suppress-lint-warnings design)))
+       (design   (cwtime (vl-design-lint-ignoreall design config.ignore)))
+       (design   (cwtime (vl-lint-apply-quiet config.quiet design)))
        (sd-probs (cwtime (vl-delete-sd-problems-for-modnames config.quiet sd-probs)))
-       (mwalist  (cwtime (vl-origname-modwarningalist mods))))
+       (reportcard  (cwtime (vl-design-origname-reportcard design))))
 
-    (make-vl-lintresult :mods mods
-                        :mods0 mods0
-                        :mwalist mwalist
+    (make-vl-lintresult :design design
+                        :design0 design0
+                        :reportcard reportcard
                         :sd-probs sd-probs
                         :dalist dalist)))
 
-
 (define run-vl-lint ((config vl-lintconfig-p) &key (state 'state))
   :returns (mv (res vl-lintresult-p :hyp :fguard)
                (state state-p1 :hyp (state-p1 state)))
@@ -539,9 +567,12 @@ shown.</p>"
               (cw "Lint configuration: ~x0~%" config)))
 
        (loadconfig (make-vl-loadconfig
+                    :edition       config.edition
+                    :strictp       config.strict
                     :start-files   config.start-files
                     :search-path   config.search-path
-                    :search-exts   config.search-exts))
+                    :search-exts   config.search-exts
+                    :include-dirs  config.include-dirs))
        (- (or (not config.debug)
               (cw "Load configuration: ~x0~%" loadconfig)))
 
@@ -549,40 +580,29 @@ shown.</p>"
        ((mv loadres state) (cwtime (vl-load loadconfig)))
 
        (lintres
-        (cwtime (run-vl-lint-main (vl-loadresult->mods loadres)
+        (cwtime (run-vl-lint-main (vl-loadresult->design loadres)
                                   config))))
     (mv lintres state)))
 
 
-(defund sd-problem-major-p (x)
-  (declare (xargs :guard (sd-problem-p x)))
+(define sd-problem-major-p ((x sd-problem-p))
   (b* (((sd-problem x) x))
     (or (>= x.priority 10)
         (and (>= x.priority 6) (>= x.groupsize 4))
         (>= (sd-problem-score x) 8))))
 
-(defsection sd-filter-problems
-
-  (defund sd-filter-problems (x major minor)
-    "Returns (MV MAJOR MINOR)"
-    (declare (xargs :guard (sd-problemlist-p x)))
-    (cond ((atom x)
-           (mv major minor))
-          ((sd-problem-major-p (car x))
-           (sd-filter-problems (cdr x) (cons (car x) major) minor))
-          (t
-           (sd-filter-problems (cdr x) major (cons (car x) minor)))))
-
-  (local (in-theory (enable sd-filter-problems)))
-
-  (defthm sd-problemlist-p-of-sd-filter-problems
-    (and (implies (and (sd-problemlist-p x)
-                       (sd-problemlist-p major))
-                  (sd-problemlist-p (mv-nth 0 (sd-filter-problems x major minor))))
-         (implies (and (sd-problemlist-p x)
-                       (sd-problemlist-p minor))
-                  (sd-problemlist-p (mv-nth 1 (sd-filter-problems x major minor))))))
-
+(define sd-filter-problems ((x     sd-problemlist-p)
+                            (major sd-problemlist-p)
+                            (minor sd-problemlist-p))
+  :returns (mv (major sd-problemlist-p :hyp :fguard)
+               (minor sd-problemlist-p :hyp :fguard))
+  (cond ((atom x)
+         (mv major minor))
+        ((sd-problem-major-p (car x))
+         (sd-filter-problems (cdr x) (cons (car x) major) minor))
+        (t
+         (sd-filter-problems (cdr x) major (cons (car x) minor))))
+  ///
   (defthm true-listp-sd-filter-problems
     (and (implies (true-listp major)
                   (true-listp (mv-nth 0 (sd-filter-problems x major minor))))
@@ -590,70 +610,30 @@ shown.</p>"
                   (true-listp (mv-nth 1 (sd-filter-problems x major minor)))))))
 
 
-
-(defthm symbol-listp-of-vl-warninglist->types
-  (implies (force (vl-warninglist-p x))
-           (symbol-listp (vl-warninglist->types x)))
-  :hints(("Goal" :induct (len x))))
-
-
-(define vl-modwarningalist-types ((x vl-modwarningalist-p))
-  (if (atom x)
-      nil
-    (append (vl-warninglist->types (cdar x))
-            (vl-modwarningalist-types (cdr x))))
-  ///
-  (defthm symbol-listp-of-vl-modwarningalist-types
-    (implies (force (vl-modwarningalist-p x))
-             (symbol-listp (vl-modwarningalist-types x)))))
-
-
-(defund vl-keep-from-modwarningalist (types x)
-  ;; Returns a new fast alist.
-  (declare (xargs :guard (and (symbol-listp types)
-                              (vl-modwarningalist-p x))))
-  (if (atom x)
-      nil
-    (b* ((name1     (caar x))
-         (warnings1 (cdar x))
-         (keep1     (vl-keep-warnings types warnings1))
-         (rest      (vl-keep-from-modwarningalist types (cdr x))))
-      (if keep1
-          (hons-acons name1 keep1 rest)
-        rest))))
-
-(defthm vl-modwarningalist-p-of-vl-keep-from-modwarningalist
-  (implies (and (force (symbol-listp types))
-                (force (vl-modwarningalist-p x)))
-           (vl-modwarningalist-p (vl-keep-from-modwarningalist types x)))
-  :hints(("Goal" :in-theory (enable vl-keep-from-modwarningalist))))
-
-
 (define vl-lint-print-warnings ((filename stringp)
                                 (label    stringp)
                                 (types    symbol-listp)
-                                (walist   vl-modwarningalist-p)
+                                (reportcard   vl-reportcard-p)
                                 &key (ps 'ps))
-  (b* ((walist (vl-keep-from-modwarningalist types walist))
-       (walist (vl-clean-modwarningalist walist))
-       (count  (length (append-domains walist)))
-       (-      (cond ((int= count 0)
+  (b* ((reportcard (vl-keep-from-reportcard types reportcard))
+       (reportcard (vl-clean-reportcard reportcard))
+       (count  (length (append-alist-vals reportcard)))
+       (-      (cond ((eql count 0)
                       (cw "~s0: No ~s1 Warnings.~%" filename label))
-                     ((int= count 1)
+                     ((eql count 1)
                       (cw "~s0: One ~s1 Warning.~%" filename label))
                      (t
                       (cw "~s0: ~x1 ~s2 Warnings.~%" filename count label)))))
     (vl-ps-seq
-     (cond ((int= count 0)
+     (cond ((eql count 0)
             (vl-cw "No ~s0 Warnings.~%~%" label))
-           ((int= count 1)
+           ((eql count 1)
             (vl-cw "One ~s0 Warning:~%~%" label))
            (t
             (vl-cw "~x0 ~s1 Warnings:~%~%" count label)))
-     (vl-print-modwarningalist walist))))
+     (vl-print-reportcard reportcard))))
 
-
-(define vl-jp-modwarningalist-aux ((x vl-modwarningalist-p) &key (ps 'ps))
+(define vl-jp-reportcard-aux ((x vl-reportcard-p) &key (ps 'ps))
   (b* (((when (atom x))
         ps)
        ((cons modname warnings) (car x)))
@@ -664,11 +644,11 @@ shown.</p>"
                (if (atom (cdr x))
                    ps
                  (vl-println ","))
-               (vl-jp-modwarningalist-aux (cdr x)))))
+               (vl-jp-reportcard-aux (cdr x)))))
 
-(define vl-jp-modwarningalist ((x vl-modwarningalist-p) &key (ps 'ps))
+(define vl-jp-reportcard ((x vl-reportcard-p) &key (ps 'ps))
   (vl-ps-seq (vl-print "{")
-             (vl-jp-modwarningalist-aux x)
+             (vl-jp-reportcard-aux x)
              (vl-println "}")))
 
 (defconst *use-set-warnings*
@@ -699,7 +679,7 @@ shown.</p>"
         :vl-warn-duplicates
         :vl-bad-instance
         :vl-unresolved-hid
-        :vl-warn-unused-reg
+        :vl-warn-unused-var
         :vl-warn-blank
         :vl-undefined-names
         :vl-port-mismatch))
@@ -810,15 +790,15 @@ shown.</p>"
 
    ))
 
-(local (in-theory (disable sets::in sets::in-tail
-                           sets::difference sets::mergesort)))
+(local (in-theory (disable set::in set::in-tail
+                           set::difference set::mergesort)))
 
 (defun vl-lint-report (lintresult state)
   (declare (xargs :guard (vl-lintresult-p lintresult)
                   :stobjs state))
 
   (b* (((vl-lintresult lintresult) lintresult)
-       (walist   lintresult.mwalist)
+       (reportcard   lintresult.reportcard)
        (sd-probs lintresult.sd-probs)
 
        ((mv major minor)
@@ -828,7 +808,7 @@ shown.</p>"
 
        (- (cw "~%vl-lint: saving results...~%~%"))
 
-       (othertypes  (difference (mergesort (vl-modwarningalist-types walist))
+       (othertypes  (difference (mergesort (vl-reportcard-types reportcard))
                                 (mergesort (append *warnings-covered*
                                                    *warnings-ignored*))))
 
@@ -836,7 +816,7 @@ shown.</p>"
         (with-ps-file
          "vl-basic.txt"
          (vl-ps-update-autowrap-col 68)
-         (vl-lint-print-warnings "vl-basic.txt" "Basic" *basic-warnings* walist)))
+         (vl-lint-print-warnings "vl-basic.txt" "Basic" *basic-warnings* reportcard)))
 
        (state
         (with-ps-file
@@ -848,7 +828,7 @@ mean and how to avoid them.
 
 ")
 
-         (vl-lint-print-warnings "vl-trunc.txt" "Truncation/Extension" *trunc-warnings* walist)
+         (vl-lint-print-warnings "vl-trunc.txt" "Truncation/Extension" *trunc-warnings* reportcard)
 
          (vl-print "
 
@@ -893,25 +873,25 @@ you can see \"vl-trunc-minor.txt\" to review them.")))
         (with-ps-file
          "vl-fussy.txt"
          (vl-ps-update-autowrap-col 68)
-         (vl-lint-print-warnings "vl-fussy.txt" "Fussy Size Warnings" *fussy-size-warnings* walist)))
+         (vl-lint-print-warnings "vl-fussy.txt" "Fussy Size Warnings" *fussy-size-warnings* reportcard)))
 
        (state
         (with-ps-file
          "vl-fussy-minor.txt"
          (vl-ps-update-autowrap-col 68)
-         (vl-lint-print-warnings "vl-fussy-minor.txt" "Minor Fussy Size Warnings" *fussy-size-minor-warnings* walist)))
+         (vl-lint-print-warnings "vl-fussy-minor.txt" "Minor Fussy Size Warnings" *fussy-size-minor-warnings* reportcard)))
 
        (state
         (with-ps-file
          "vl-disconnected.txt"
          (vl-ps-update-autowrap-col 68)
-         (vl-lint-print-warnings "vl-disconnected.txt" "Disconnected Wire" *disconnected-warnings* walist)))
+         (vl-lint-print-warnings "vl-disconnected.txt" "Disconnected Wire" *disconnected-warnings* reportcard)))
 
        (state
         (with-ps-file
          "vl-multi.txt"
          (vl-ps-update-autowrap-col 68)
-         (vl-lint-print-warnings "vl-multi.txt" "Multidrive" *multidrive-warnings* walist)))
+         (vl-lint-print-warnings "vl-multi.txt" "Multidrive" *multidrive-warnings* reportcard)))
 
        (state
         (if (not major)
@@ -934,7 +914,7 @@ NOTE: see the bottom of this file for an explanation of what these warnings
 mean and how to avoid them.
 
 ")
-         (vl-lint-print-warnings "vl-trunc-minor.txt" "Minor Truncation/Extension" *trunc-minor-warnings* walist)
+         (vl-lint-print-warnings "vl-trunc-minor.txt" "Minor Truncation/Extension" *trunc-minor-warnings* reportcard)
          (vl-print "
 
 UNDERSTANDING THESE WARNINGS.
@@ -986,7 +966,7 @@ wide addition instead of a 10-bit wide addition.")))
         (with-ps-file
          "vl-multi-minor.txt"
          (vl-ps-update-autowrap-col 68)
-         (vl-lint-print-warnings "vl-multi-minor.txt" "Minor Multidrive" *multidrive-minor-warnings* walist)))
+         (vl-lint-print-warnings "vl-multi-minor.txt" "Minor Multidrive" *multidrive-minor-warnings* reportcard)))
 
        (state
         (if (not minor)
@@ -1008,20 +988,20 @@ wide addition instead of a 10-bit wide addition.")))
          (vl-lint-print-warnings "vl-use-set.txt"
                                  "Unused/Unset Wire Warnings"
                                  *use-set-warnings*
-                                 walist)))
+                                 reportcard)))
 
 
        (state
         (with-ps-file
          "vl-smells.txt"
          (vl-ps-update-autowrap-col 68)
-         (vl-lint-print-warnings "vl-smells.txt" "Code-Smell Warnings" *smell-warnings* walist)))
+         (vl-lint-print-warnings "vl-smells.txt" "Code-Smell Warnings" *smell-warnings* reportcard)))
 
        (state
         (with-ps-file
          "vl-smells-minor.txt"
          (vl-ps-update-autowrap-col 68)
-         (vl-lint-print-warnings "vl-smells-minor.txt" "Minor Code-Smell Warnings" *smell-minor-warnings* walist)))
+         (vl-lint-print-warnings "vl-smells-minor.txt" "Minor Code-Smell Warnings" *smell-minor-warnings* reportcard)))
 
 
       (state
@@ -1030,20 +1010,20 @@ wide addition instead of a 10-bit wide addition.")))
                      (vl-lint-print-warnings "vl-same-ports.txt"
                                              "Same-ports Warnings"
                                              *same-ports-warnings*
-                                             walist)))
+                                             reportcard)))
       (state
        (with-ps-file "vl-same-ports-minor.txt"
                      (vl-ps-update-autowrap-col 68)
                      (vl-lint-print-warnings "vl-same-ports-minor.txt"
                                              "Minor same-ports Warnings"
                                              *same-ports-minor-warnings*
-                                             walist)))
+                                             reportcard)))
 
       (state
        (with-ps-file
         "vl-other.txt"
         (vl-ps-update-autowrap-col 68)
-        (vl-lint-print-warnings "vl-other.txt" "Other/Unclassified" othertypes walist)))
+        (vl-lint-print-warnings "vl-other.txt" "Other/Unclassified" othertypes reportcard)))
 
       (- (cw "~%"))
 
@@ -1051,7 +1031,7 @@ wide addition instead of a 10-bit wide addition.")))
        (cwtime
         (with-ps-file "vl-warnings.json"
                       (vl-print "{\"warnings\":")
-                      (vl-jp-modwarningalist walist)
+                      (vl-jp-reportcard reportcard)
                       (vl-println "}"))
         :name write-warnings-json)))
 
@@ -1103,6 +1083,7 @@ wide addition instead of a 10-bit wide addition.")))
 
        (state (must-be-regular-files! config.start-files))
        (state (must-be-directories! config.search-path))
+       (state (must-be-directories! config.include-dirs))
 
        ((mv result state)
         (cwtime (run-vl-lint config)
@@ -1112,9 +1093,3 @@ wide addition instead of a 10-bit wide addition.")))
     (exit-ok)
     state))
 
-
-#||
-
-(vl-lint (list "
-
-||#
\ No newline at end of file
