@@ -1,20 +1,30 @@
 ; VL Verilog Toolkit
-; Copyright (C) 2008-2013 Centaur Technology
+; Copyright (C) 2008-2014 Centaur Technology
 ;
 ; Contact:
 ;   Centaur Technology Formal Verification Group
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 ;
 ; Original author: Jared Davis <jared@centtech.com>
 
@@ -23,10 +33,14 @@
 (include-book "lint")
 (include-book "model")
 (include-book "shell")
+(include-book "pp")
+(include-book "gather")
 (include-book "oslib/argv" :dir :system)
 (include-book "centaur/esim/stv/stv2c/top" :dir :system)
 (include-book "centaur/misc/intern-debugging" :dir :system)
 (include-book "centaur/misc/memory-mgmt" :dir :system)
+(include-book "centaur/misc/tshell" :dir :system)
+(include-book "centaur/nrev/fast" :dir :system)
 (local (include-book "../util/arithmetic"))
 (local (include-book "../util/osets"))
 
@@ -43,6 +57,8 @@ Commands:
   lint    Find potential bugs in a Verilog design
   model   Translate Verilog designs for the ACL2 theorem prover
   stv2c   Translate symbolic runs of Verilog designs into C++
+  pp      Preprocess Verilog designs
+  gather  Collect Verilog files into a single file
   shell   Interactive VL shell (for experts)
 
 Use 'vl help <command>' for help on a specific command.
@@ -78,12 +94,14 @@ commands.</p>
 @(def vl-toolkit-help-message)"
 
   (defconst *vl-help-messages*
-    (list (cons "help"  *vl-generic-help*)
-          (cons "json"  *vl-json-help*)
-          (cons "lint"  *vl-lint-help*)
-          (cons "model" *vl-model-help*)
-          (cons "stv2c" acl2::*stv2c-help*)
-          (cons "shell" *vl-shell-help*)))
+    (list (cons "help"   *vl-generic-help*)
+          (cons "json"   *vl-json-help*)
+          (cons "lint"   *vl-lint-help*)
+          (cons "model"  *vl-model-help*)
+          (cons "stv2c"  acl2::*stv2c-help*)
+          (cons "pp"     *vl-pp-help*)
+          (cons "gather" *vl-gather-help*)
+          (cons "shell"  *vl-shell-help*)))
 
   (encapsulate
     (((vl-toolkit-help-message *) => *
@@ -165,7 +183,13 @@ toolkit with their own commands.</p>
   :parents (kit)
   :short "The top-level @('vl') meta-command."
 
-  (b* (((mv argv state) (oslib::argv))
+  (b* ((state
+        ;; Since the VL executable is a non-interactive program, there's no
+        ;; chance to enter a break loop if something crashes.  Printing a
+        ;; backtrace before aborting, then, can be extremely useful.
+        (set-debugger-enable :bt))
+       (- (acl2::tshell-ensure))
+       ((mv argv state) (oslib::argv))
 
        ((unless (consp argv))
         (b* ((state (vl-help '("help"))))
@@ -196,6 +220,16 @@ toolkit with their own commands.</p>
           (exit-ok)
           state))
 
+       ((when (equal cmd "pp"))
+        (b* ((state (vl-pp args)))
+          (exit-ok)
+          state))
+
+       ((when (equal cmd "gather"))
+        (b* ((state (vl-gather args)))
+          (exit-ok)
+          state))
+
        ((when (equal cmd "stv2c"))
         (b* ((state (acl2::stv2c args)))
           (exit-ok)
