@@ -1,27 +1,38 @@
 ; VL Verilog Toolkit
-; Copyright (C) 2008-2011 Centaur Technology
+; Copyright (C) 2008-2014 Centaur Technology
 ;
 ; Contact:
 ;   Centaur Technology Formal Verification Group
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 ;
 ; Original author: Jared Davis <jared@centtech.com>
 
 (in-package "VL")
-(include-book "warnings")
+(include-book "reportcard")
 (include-book "fmt")
 (local (include-book "../util/arithmetic"))
+(local (std::add-default-post-define-hook :fix))
 
 (define vl-print-warning-text-mode ((x vl-warning-p) &key (ps 'ps))
   :parents (vl-print-warning)
@@ -156,7 +167,7 @@ warnings\".</p>"
         (if (atom x)
             ps
           (vl-ps-seq (vl-println "")
-                     (vl-print modname)
+                     (vl-print-str modname)
                      (vl-print " -- ")
                      (vl-println msg)
                      (vl-print-warnings-aux x)))
@@ -180,26 +191,29 @@ warnings\".</p>"
 
        (vl-println-markup "</div>")))))
 
-(define vl-print-modwarningalist-aux ((x vl-modwarningalist-p) &key (ps 'ps))
-  :parents (warnings)
-  (if (atom x)
-      ps
+(define vl-print-reportcard-aux ((x vl-reportcard-p) &key (ps 'ps))
+  :parents (vl-reportcard-p)
+  :measure (vl-reportcard-count x)
+  (b* ((x (vl-reportcard-fix x))
+       ((when (atom x))
+        ps))
     (vl-ps-seq (vl-print-warnings-with-named-header (caar x) (cdar x))
                (vl-println "")
-               (vl-print-modwarningalist-aux (cdr x)))))
+               (vl-print-reportcard-aux (cdr x)))))
 
-(define vl-print-modwarningalist ((x vl-modwarningalist-p) &key (ps 'ps))
-  :parents (warnings)
-  :short "Pretty-print a @(see vl-modwarningalist-p)."
-  :long "<p>See also @(see vl-modwarningalist-to-string).</p>"
-  (b* ((x-shrink (hons-shrink-alist x nil))
+(define vl-print-reportcard ((x vl-reportcard-p) &key (ps 'ps))
+  :parents (vl-reportcard-p)
+  :short "Pretty-print a @(see vl-reportcard-p)."
+  :long "<p>See also @(see vl-reportcard-to-string).</p>"
+  (b* ((x        (vl-reportcard-fix x))
+       (x-shrink (hons-shrink-alist x nil))
        (-        (fast-alist-free x-shrink))
        (x-sorted (mergesort x-shrink)))
-      (vl-print-modwarningalist-aux x-sorted)))
+      (vl-print-reportcard-aux x-sorted)))
 
-(define vl-modwarningalist-to-string ((x vl-modwarningalist-p))
+(define vl-reportcard-to-string ((x vl-reportcard-p))
   :returns (str stringp :rule-classes :type-prescription)
-  :parents (warnings)
-  :short "Pretty-print a @(see vl-modwarningalist-p) into a string."
-  :long "<p>See also @(see vl-print-modwarningalist).</p>"
-  (with-local-ps (vl-print-modwarningalist x)))
+  :parents (vl-reportcard-p)
+  :short "Pretty-print a @(see vl-reportcard-p) into a string."
+  :long "<p>See also @(see vl-print-reportcard).</p>"
+  (with-local-ps (vl-print-reportcard x)))
