@@ -1,20 +1,30 @@
 ; VL Verilog Toolkit
-; Copyright (C) 2008-2011 Centaur Technology
+; Copyright (C) 2008-2014 Centaur Technology
 ;
 ; Contact:
 ;   Centaur Technology Formal Verification Group
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 ;
 ; Original author: Jared Davis <jared@centtech.com>
 
@@ -24,19 +34,9 @@
 (include-book "centaur/esim/esim-primitives" :dir :system)
 (local (include-book "../util/arithmetic"))
 (local (include-book "../util/esim-lemmas"))
+(local (non-parallel-book))
 
-(make-event
-
-; Disabling waterfall parallelism because this book allegedly uses memoization
-; while performing its proofs.  
-
- (if (and (ACL2::hons-enabledp state) 
-          (f-get-global 'ACL2::parallel-execution-enabled state)) 
-     (er-progn (set-waterfall-parallelism nil)
-               (value '(value-triple nil)))
-   (value '(value-triple nil))))
-
-(defsection resolving-multiple-drivers
+(defxdoc resolving-multiple-drivers
   :parents (e-conversion)
   :short "How we replace multiply driven wires with explicit resolution
 modules."
@@ -49,7 +49,7 @@ wires by inserting explicit resolution modules.</p>
 
 <p><b>Note:</b> In this transform we assume that the module's primary inputs
 are not driven by any occurrence.  This is something we explicitly check for in
-@(see bozo-say-where-we-check-for-it).</p>
+@(see vl-module-make-esim); see the warning about @(':vl-backflow') there.</p>
 
 <p>Given this assumption, we basically need to do three things:</p>
 
@@ -77,6 +77,8 @@ This is done with @(see vl-make-res-occs).</li>
 <p>The top-level function is @(see vl-add-res-modules), and it just stitches
 these steps together.</p>")
 
+(local (xdoc::set-default-parents resolving-multiple-drivers))
+
 
 (defalist vl-res-sigma-p (x)
   :key (vl-emodwire-p x)
@@ -301,7 +303,7 @@ straightforward, right-associative manner.</p>"
            ;; atoms.  The 4v-sexpr-vars function could get confused if NIL was
            ;; permitted, for instance.
            (implies (vl-emodwirelist-p args)
-                    (equal (sets::in a (acl2::4v-sexpr-vars (vl-make-res-sexpr args)))
+                    (equal (set::in a (acl2::4v-sexpr-vars (vl-make-res-sexpr args)))
                            (if (member-equal a args)
                                t
                              nil)))
@@ -313,7 +315,7 @@ straightforward, right-associative manner.</p>"
   (defthm 4v-sexpr-vars-of-vl-make-res-sexpr
     (implies (force (vl-emodwirelist-p args))
              (equal (acl2::4v-sexpr-vars (vl-make-res-sexpr args))
-                    (sets::mergesort args)))
+                    (set::mergesort args)))
     :hints((set-reasoning))))
 
 
