@@ -1,20 +1,30 @@
 ; VL Verilog Toolkit
-; Copyright (C) 2008-2011 Centaur Technology
+; Copyright (C) 2008-2014 Centaur Technology
 ;
 ; Contact:
 ;   Centaur Technology Formal Verification Group
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 ;
 ; Original author: Jared Davis <jared@centtech.com>
 
@@ -397,7 +407,7 @@ versa.  We extend @('X') with a fatal warning if this doesn't hold.</p>"
      (local (defthm insert-under-iff
               (iff (insert a x)
                    t)
-              :hints(("Goal" :in-theory (enable (:ruleset sets::primitive-rules))))))
+              :hints(("Goal" :in-theory (enable (:ruleset set::primitive-rules))))))
 
      (local (defthm demote-in-to-member-equal
               (implies (setp x)
@@ -405,22 +415,22 @@ versa.  We extend @('X') with a fatal warning if this doesn't hold.</p>"
                               (if (member-equal a x)
                                   t
                                 nil)))
-              :hints(("Goal" :in-theory (enable (:ruleset sets::primitive-rules))))))
+              :hints(("Goal" :in-theory (enable (:ruleset set::primitive-rules))))))
 
      (defthm empty-intersect-to-intersectp-equal
        (implies (and (setp x)
                      (setp y))
-                (equal (empty (sets::intersect x y))
+                (equal (empty (set::intersect x y))
                        (not (intersectp-equal x y))))
        :hints(("Goal"
-               :induct (sets::intersect x y)
-               :in-theory (e/d ((:ruleset sets::primitive-rules))
+               :induct (set::intersect x y)
+               :in-theory (e/d ((:ruleset set::primitive-rules))
                                ;; speed hint
                                (promote-member-equal-to-membership
-                                sets::insert-identity
-                                sets::intersect-symmetric
-                                sets::double-containment
-                                sets::setp)))))))
+                                set::insert-identity
+                                set::intersect-symmetric
+                                set::double-containment
+                                set::setp)))))))
 
   ;; Speed hint
   (local (in-theory (disable no-duplicatesp-equal
@@ -432,7 +442,7 @@ versa.  We extend @('X') with a fatal warning if this doesn't hold.</p>"
                              hons-assoc-equal
                              acl2::consp-under-iff-when-true-listp
                              subsetp-equal-when-first-two-same-yada-yada
-                             sets::double-containment)))
+                             set::double-containment)))
 
   (defund vl-module-check-port-bits (x)
     (declare (xargs :guard (vl-module-p x)))
@@ -473,7 +483,7 @@ versa.  We extend @('X') with a fatal warning if this doesn't hold.</p>"
                               (mbe :logic (uniquep flat-outs)
                                    :exec  (same-lengthp flat-outs-s flat-outs))
                               (mbe :logic (not (intersectp-equal flat-ins flat-outs))
-                                   :exec  (not (sets::intersectp flat-ins-s flat-outs-s)))))
+                                   :exec  (not (set::intersectp flat-ins-s flat-outs-s)))))
               warnings
             ;; Else, there are duplicated port names!
             (b* ((dupe-names (duplicated-members (vl-portdecllist->names x.portdecls)))
@@ -902,7 +912,7 @@ modinsts-to-eoccs).</p>"
                    :fn 'vl-modinst-to-eocc)))
             (mv nil (cons w warnings) nil)))
 
-         ((when (vl-arguments->namedp x.portargs))
+         ((when (eq (vl-arguments-kind x.portargs) :named))
           (b* ((w (make-vl-warning
                    :type :vl-bad-instance
                    :msg "~a0: expected only resolved module instances, but ~
@@ -972,7 +982,7 @@ modinsts-to-eoccs).</p>"
 
          ;; Build the alist binding formals to actuals...
 
-         (actuals (vl-arguments->args x.portargs))
+         (actuals (vl-arguments-plain->args x.portargs))
          ((unless (same-lengthp actuals portpat))
           (b* ((w (make-vl-warning
                    :type :vl-bad-instance
