@@ -1,20 +1,30 @@
 ; VL Verilog Toolkit
-; Copyright (C) 2008-2011 Centaur Technology
+; Copyright (C) 2008-2014 Centaur Technology
 ;
 ; Contact:
 ;   Centaur Technology Formal Verification Group
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 ;
 ; Original author: Jared Davis <jared@centtech.com>
 
@@ -22,9 +32,12 @@
 (include-book "add")
 (local (include-book "../../util/arithmetic"))
 (local (include-book "../../util/osets"))
+(local (include-book "centaur/misc/arith-equivs" :dir :system))
+(local (std::add-default-post-define-hook :fix))
 (local (in-theory (disable vl-maybe-module-p-when-vl-module-p)))
+(local (in-theory (enable acl2::arith-equiv-forwarding)))
 
-(defsection *vl-1-bit-mult*
+(defval *vl-1-bit-mult*
   :parents (occform)
   :short "One-bit multiplier."
 
@@ -51,41 +64,43 @@ module VL_1_BIT_MULT (o, a, b);
 endmodule
 })"
 
-  (defconst *vl-1-bit-mult*
-    (b* ((name (hons-copy "VL_1_BIT_MULT"))
+  (b* ((name (hons-copy "VL_1_BIT_MULT"))
 
-         ((mv o-expr o-port o-portdecl o-netdecl) (vl-primitive-mkport "o" :vl-output))
-         ((mv a-expr a-port a-portdecl a-netdecl) (vl-primitive-mkport "a" :vl-input))
-         ((mv b-expr b-port b-portdecl b-netdecl) (vl-primitive-mkport "b" :vl-input))
+       ((mv o-expr o-port o-portdecl o-netdecl) (vl-primitive-mkport "o" :vl-output))
+       ((mv a-expr a-port a-portdecl a-netdecl) (vl-primitive-mkport "a" :vl-input))
+       ((mv b-expr b-port b-portdecl b-netdecl) (vl-primitive-mkport "b" :vl-input))
 
-         ((mv p0-expr p0-netdecl) (vl-primitive-mkwire "p0"))
-         ((mv x0-expr x0-netdecl) (vl-primitive-mkwire "x0"))
-         ((mv x1-expr x1-netdecl) (vl-primitive-mkwire "x1"))
+       ((mv p0-expr p0-netdecl) (vl-primitive-mkwire "p0"))
+       ((mv x0-expr x0-netdecl) (vl-primitive-mkwire "x0"))
+       ((mv x1-expr x1-netdecl) (vl-primitive-mkwire "x1"))
 
-         (p0-inst (vl-simple-inst *vl-1-bit-and* "mk_p0" p0-expr a-expr  b-expr))
-         (x0-inst (vl-simple-inst *vl-1-bit-xor* "mk_x0" x0-expr a-expr  b-expr))
-         (x1-inst (vl-simple-inst *vl-1-bit-xor* "mk_x1" x1-expr x0-expr x0-expr))
-         (o-inst  (vl-simple-inst *vl-1-bit-xor* "mk_o"  o-expr  p0-expr x1-expr)))
+       (p0-inst (vl-simple-inst *vl-1-bit-and* "mk_p0" p0-expr a-expr  b-expr))
+       (x0-inst (vl-simple-inst *vl-1-bit-xor* "mk_x0" x0-expr a-expr  b-expr))
+       (x1-inst (vl-simple-inst *vl-1-bit-xor* "mk_x1" x1-expr x0-expr x0-expr))
+       (o-inst  (vl-simple-inst *vl-1-bit-xor* "mk_o"  o-expr  p0-expr x1-expr)))
 
-      (make-vl-module :name      name
-                      :origname  name
-                      :ports     (list o-port a-port b-port)
-                      :portdecls (list o-portdecl a-portdecl b-portdecl)
-                      :netdecls  (list o-netdecl a-netdecl b-netdecl p0-netdecl x0-netdecl x1-netdecl)
-                      :modinsts  (list p0-inst x0-inst x1-inst o-inst)
-                      :minloc    *vl-fakeloc*
-                      :maxloc    *vl-fakeloc*))))
+    (hons-copy
+     (make-vl-module :name      name
+                     :origname  name
+                     :ports     (list o-port a-port b-port)
+                     :portdecls (list o-portdecl a-portdecl b-portdecl)
+                     :netdecls  (list o-netdecl a-netdecl b-netdecl p0-netdecl x0-netdecl x1-netdecl)
+                     :modinsts  (list p0-inst x0-inst x1-inst o-inst)
+                     :minloc    *vl-fakeloc*
+                     :maxloc    *vl-fakeloc*))))
 
 
 ;; bozo this all needs to be documented.
 
 (define vl-partprod-insts-aux ((i natp)
                                (n natp))
-  :returns (insts vl-modinstlist-p :hyp :fguard)
+  :returns (insts vl-modinstlist-p)
   :guard (< i n)
   ;; Create instances that drive pi, the i-th, shifted partial product for an
   ;; n-bit multiplier
-  (b* ((p-name (hons-copy (cat "p" (natstr i))))
+  (b* ((i      (lnfix i))
+       (n      (lnfix n))
+       (p-name (hons-copy (cat "p" (natstr i))))
        (p-expr (vl-idexpr p-name n :vl-unsigned))
        (p-high (vl-make-list-of-bitselects p-expr i (- n 1)))
        (p-low  (if (zp i)
@@ -95,13 +110,13 @@ endmodule
        (a-name (hons-copy "a"))
        (a-expr (vl-idexpr a-name n :vl-unsigned))
        (a[i]   (vl-make-bitselect a-expr i))
-       (a-high (repeat a[i] (len p-high)))
+       (a-high (replicate (len p-high) a[i]))
 
        (b-name (hons-copy "b"))
        (b-expr (vl-idexpr b-name n :vl-unsigned))
        (b-bits (vl-make-list-of-bitselects b-expr 0 (- n 1)))
        (b-high (take (len p-high) b-bits))
-       (b-low  (repeat |*sized-1'b0*| (len p-low)))
+       (b-low  (replicate (len p-low) |*sized-1'b0*|))
 
        (ands (vl-simple-inst-list *vl-1-bit-and* (cat "mk_" p-name "_high")
                                   p-high a-high b-high))
@@ -111,15 +126,11 @@ endmodule
 
 (define vl-partprod-insts ((i natp)
                            (n natp))
-  :returns (insts vl-modinstlist-p :hyp :fguard)
+  :returns (insts vl-modinstlist-p)
   :guard (<= i n)
   :measure (nfix (- (nfix n) (nfix i)))
   ;; Create instances that drive all all pi (0 <= i < n), all partial products
   ;; for an n-bit multiplier
-  (declare (xargs :guard (and (natp i)
-                              (natp n)
-                              (<= i n))
-                  :measure (nfix (- (nfix n) (nfix i)))))
   (if (zp (- (nfix n) (nfix i)))
       nil
     (append (vl-partprod-insts-aux i n)
@@ -131,7 +142,7 @@ endmodule
                        (consp x)))
          :hints(("Goal" :in-theory (enable last)))))
 
-(def-vl-modgen vl-make-n-bit-mult (n)
+(def-vl-modgen vl-make-n-bit-mult ((n posp))
   :short "Generate an multiplier module."
 
   :long "<p>We produce @('VL_N_BIT_MULT') for the given @('n'), which is
@@ -154,9 +165,9 @@ with @('n-1') instances of an N-bit wide adder circuit.</p>
 @('X') or @('Z'), then every bit of the output is @('X').  We implement this
 explicitly, which adds a layer of X-detection around the core circuitry.</p>"
 
-  :guard (posp n)
   :body
-  (b* (((when (= n 1))
+  (b* ((n (lposfix n))
+       ((when (eql n 1))
         (list *vl-1-bit-mult* *vl-1-bit-and* *vl-1-bit-xor*))
        (name  (hons-copy (cat "VL_" (natstr n) "_BIT_MULT")))
 
@@ -210,7 +221,7 @@ explicitly, which adds a layer of X-detection around the core circuitry.</p>"
 
                                         (cons (car (last p-exprs)) (butlast s-exprs 1))
                                         (cdr (rev p-exprs))
-                                        (repeat |*sized-1'b0*| (- n 1))))
+                                        (replicate (- n 1) |*sized-1'b0*|)))
 
        ;; this is the answer except for x-detection
        (ans (car (last s-exprs)))
