@@ -1,20 +1,30 @@
 ; VL Verilog Toolkit
-; Copyright (C) 2008-2011 Centaur Technology
+; Copyright (C) 2008-2014 Centaur Technology
 ;
 ; Contact:
 ;   Centaur Technology Formal Verification Group
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 ;
 ; Original author: Jared Davis <jared@centtech.com>
 
@@ -22,6 +32,8 @@
 (include-book "util")
 (local (include-book "../../util/arithmetic"))
 (local (include-book "../../util/osets"))
+(local (std::add-default-post-define-hook :fix))
+(local (non-parallel-book))
 
 ; This file has generators for:
 ;
@@ -34,20 +46,13 @@
 ;  - N-bit case equality modules (===)
 ;  - N-bit pure-X module
 
-(make-event
+(def-vl-modgen vl-make-n-bit-binary-op
+  ((type (member type '(:vl-and :vl-or :vl-xor :vl-xnor)))
+   (n    posp))
 
-; Disabling waterfall parallelism because this book allegedly uses memoization
-; while performing its proofs.  
-
- (if (and (ACL2::hons-enabledp state) 
-          (f-get-global 'ACL2::parallel-execution-enabled state)) 
-     (er-progn (set-waterfall-parallelism nil)
-               (value '(value-triple nil)))
-   (value '(value-triple nil))))
-
-(def-vl-modgen vl-make-n-bit-binary-op (type n)
   :short "Generate a wide, pointwise AND, OR, XOR, or XNOR module."
 
+
   :long "<p>The @('type') must be either @(':VL-AND'), @(':VL-OR'),
 @(':VL-XOR'), or @(':VL-XNOR').  Depending on the type, we generate a module
 that is written using @(see primitives) but is semantically equivalent to:</p>
@@ -77,14 +82,14 @@ endmodule
   VL_1_BIT_OR bit0 (out[0], a[0], b[0]);
 })"
 
-  :guard (and (member type '(:vl-and :vl-or :vl-xor :vl-xnor))
-              (posp n))
   :body
-  (b* ((prim (case type
-               (:vl-and  *vl-1-bit-and*)
-               (:vl-or   *vl-1-bit-or*)
-               (:vl-xor  *vl-1-bit-xor*)
-               (:vl-xnor *vl-1-bit-xnor*)))
+  (b* ((n (lposfix n))
+       (prim (case type
+               (:vl-and   *vl-1-bit-and*)
+               (:vl-or    *vl-1-bit-or*)
+               (:vl-xor   *vl-1-bit-xor*)
+               (otherwise *vl-1-bit-xnor*)))
+
 
 ; [Jared and Sol]: It's tempting here to just return (list prim) in the special
 ; case that n = 1.  But after discussing this, we decided it seems nicer to put
@@ -97,11 +102,11 @@ endmodule
 ; consistency we go ahead and try to keep the wrapper in all cases.
 
        (name (hons-copy (cat "VL_" (natstr n) "_BIT_POINTWISE_"
-                                  (case type
-                                    (:vl-and "AND")
-                                    (:vl-or "OR")
-                                    (:vl-xor "XOR")
-                                    (:vl-xnor "XNOR")))))
+                             (case type
+                               (:vl-and "AND")
+                               (:vl-or "OR")
+                               (:vl-xor "XOR")
+                               (:vl-xnor "XNOR")))))
 
        ((mv out-expr out-port out-portdecl out-netdecl) (vl-occform-mkport "out" :vl-output n))
        ((mv a-expr a-port a-portdecl a-netdecl)         (vl-occform-mkport "a" :vl-input n))
@@ -129,7 +134,7 @@ endmodule
 
 
 
-(def-vl-modgen vl-make-n-bit-assign (n)
+(def-vl-modgen vl-make-n-bit-assign ((n posp))
   :short "Generate a wide assignment module."
 
   :long "<p>We generate a module that is semantically equal to:</p>
@@ -157,10 +162,9 @@ module VL_4_BIT_ASSIGN (out, in);
 endmodule
 })"
 
-  :guard (posp n)
-
   :body
-  (b* (((when (= n 1))
+  (b* ((n (lposfix n))
+       ((when (eql n 1))
         (list *vl-1-bit-assign*))
 
        (name (hons-copy (cat "VL_" (natstr n) "_BIT_ASSIGN")))
@@ -189,11 +193,7 @@ endmodule
 ||#
 
 
-
-
-
-
-(def-vl-modgen vl-make-n-bit-not (n)
+(def-vl-modgen vl-make-n-bit-not ((n posp))
   :short "Generate a wide negation module."
 
   :long "<p>We generate a module that is written using gates and which is
@@ -217,9 +217,9 @@ above we would have:</p>
   VL_1_BIT_NOT bit3 (out[3], in[3]) ;
 })"
 
-  :guard (posp n)
   :body
-  (b* (((when (= n 1))
+  (b* ((n (lposfix n))
+       ((when (eql n 1))
         (list *vl-1-bit-not*))
 
        (name (hons-copy (cat "VL_" (natstr n) "_BIT_NOT")))
@@ -248,7 +248,9 @@ above we would have:</p>
 
 
 
-(def-vl-modgen vl-make-n-bit-reduction-op (type n)
+(def-vl-modgen vl-make-n-bit-reduction-op
+  ((type (member type (list :vl-unary-bitand :vl-unary-bitor :vl-unary-xor)))
+   (n    posp))
   :short "Generate a wide reduction AND, OR, or XOR module."
 
   :long "<p>The @('type') must be either @(':VL-UNARY-BITAND'),
@@ -281,15 +283,13 @@ endmodule
   xor(out,   in4, temp2);
 })"
 
-  :guard (and (member type (list :vl-unary-bitand :vl-unary-bitor :vl-unary-xor))
-              (posp n))
-
   :body
-  (b* ((name (hons-copy (cat "VL_" (natstr n) "_BIT_REDUCTION_"
-                                  (case type
-                                    (:vl-unary-bitand "AND")
-                                    (:vl-unary-bitor "OR")
-                                    (:vl-unary-xor "XOR")))))
+  (b* ((n (lposfix n))
+       (name (hons-copy (cat "VL_" (natstr n) "_BIT_REDUCTION_"
+                             (case type
+                               (:vl-unary-bitand "AND")
+                               (:vl-unary-bitor  "OR")
+                               (otherwise        "XOR")))))
 
        ((mv out-expr out-port out-portdecl out-netdecl) (vl-occform-mkport "out" :vl-output 1))
        ((mv in-expr in-port in-portdecl in-netdecl)     (vl-occform-mkport "in" :vl-input n))
@@ -298,12 +298,12 @@ endmodule
        (prim      (case type
                     (:vl-unary-bitand *vl-1-bit-and*)
                     (:vl-unary-bitor  *vl-1-bit-or*)
-                    (:vl-unary-xor    *vl-1-bit-xor*)))
+                    (otherwise        *vl-1-bit-xor*)))
 
        ((when (< n 3))
         ;; Special cases.  See test-redux.v.
         (b* (((mv inst prim)
-              (if (= n 1)
+              (if (eql n 1)
                   (mv (vl-simple-inst *vl-1-bit-buf* "ans" out-expr in-expr)
                       *vl-1-bit-buf*)
                 ;; Any two-bit op is just the op applied to the argument bits.
@@ -351,7 +351,8 @@ endmodule
 ||#
 
 
-(def-vl-modgen vl-make-n-bit-mux (n approxp)
+(def-vl-modgen vl-make-n-bit-mux ((n       posp)
+                                  (approxp booleanp))
   :short "Generate a wide multiplexor module."
 
   :long "<p>We generate a module that is written using gates and which is a
@@ -410,13 +411,11 @@ behavior that is closest to Verilog.  But the more conservative version may
 generally produce smaller AIGs since the output doesn't depend upon the inputs
 when the select is X.  So, we generally set @('approxp') to T.</p>"
 
-  :guard (and (posp n)
-              (booleanp approxp))
-
   :body
-  (b* ((onebitmux (if approxp *vl-1-bit-approx-mux* *vl-1-bit-mux*))
+  (b* ((n         (lposfix n))
+       (onebitmux (if approxp *vl-1-bit-approx-mux* *vl-1-bit-mux*))
 
-       ((when (= n 1))
+       ((when (eql n 1))
         (list onebitmux))
 
        (name (cat "VL_" (natstr n) "_BIT_" (if approxp "APPROX_MUX" "MUX")))
@@ -430,7 +429,7 @@ when the select is X.  So, we generally set @('approxp') to T.</p>"
        (a-wires   (vl-make-list-of-bitselects a-expr 0 (- n 1)))
        (b-wires   (vl-make-list-of-bitselects b-expr 0 (- n 1)))
 
-       (insts     (vl-simple-inst-list onebitmux "bit" out-wires (repeat sel-expr n) a-wires b-wires))
+       (insts     (vl-simple-inst-list onebitmux "bit" out-wires (replicate n sel-expr) a-wires b-wires))
 
        (mod  (make-vl-module :name      name
                              :origname  name
@@ -448,7 +447,7 @@ when the select is X.  So, we generally set @('approxp') to T.</p>"
 ||#
 
 
-(def-vl-modgen vl-make-n-bit-zmux (n)
+(def-vl-modgen vl-make-n-bit-zmux ((n posp))
   :short "Generate a wide tri-state buffer module."
 
   :long "<p>We generate a module using @(see *vl-1-bit-zmux*) primitives that
@@ -471,10 +470,9 @@ ternary) operators whose last argument is @('Z').  Note that in @(see
 oprewrite), we canonicalize @('sel ? Z : a') to @('~sel ? a : Z'), so this
 actually handles both cases.</p>"
 
-  :guard (posp n)
-
   :body
-  (b* (((when (= n 1))
+  (b* ((n (lposfix n))
+       ((when (eql n 1))
         (list *vl-1-bit-zmux*))
 
        (name (hons-copy (cat "VL_" (natstr n) "_BIT_ZMUX")))
@@ -485,7 +483,7 @@ actually handles both cases.</p>"
 
        (out-wires        (vl-make-list-of-bitselects out-expr 0 (- n 1)))
        (a-wires          (vl-make-list-of-bitselects a-expr 0 (- n 1)))
-       (insts            (vl-simple-inst-list *vl-1-bit-zmux* "bit" out-wires (repeat sel-expr n) a-wires))
+       (insts            (vl-simple-inst-list *vl-1-bit-zmux* "bit" out-wires (replicate n sel-expr) a-wires))
 
        (mod (make-vl-module :name      name
                             :origname  name
@@ -505,14 +503,7 @@ actually handles both cases.</p>"
 
 
 
-
-(local (defthm crock
-         ;; BOZO sholdn't need this, but DO NEED IT.  >=\
-         (first (vl-make-n-bit-reduction-op type n))
-         :hints(("Goal" :in-theory (enable vl-make-n-bit-reduction-op)))))
-
-
-(def-vl-modgen vl-make-n-bit-ceq (n)
+(def-vl-modgen vl-make-n-bit-ceq ((n posp))
   :short "Generate a wide case-equality module."
 
   :long "<p>We generate a module that is written using gates and which is
@@ -530,9 +521,9 @@ endmodule
 <p>We basically just instantiate @(see *vl-1-bit-ceq*) N times and then
 reduction-and the results.</p> "
 
-  :guard (posp n)
   :body
-  (b* (((when (= n 1))
+  (b* ((n (lposfix n))
+       ((when (eql n 1))
         (list *vl-1-bit-ceq*))
 
        (name (hons-copy (cat "VL_" (natstr n) "_BIT_CEQ")))
@@ -569,7 +560,7 @@ reduction-and the results.</p> "
 ||#
 
 
-(def-vl-modgen vl-make-n-bit-x (n)
+(def-vl-modgen vl-make-n-bit-x ((n posp))
   :short "Generate a wide X module."
 
   :long "<p>We generate a module that is semantically equal to:</p>
@@ -601,10 +592,9 @@ module VL_4_BIT_X (out, in);
 endmodule
 })"
 
-  :guard (posp n)
-
   :body
-  (b* (((when (int= n 1))
+  (b* ((n (lposfix n))
+       ((when (eql n 1))
         (list *vl-1-bit-x*))
 
        (name (hons-copy (cat "VL_" (natstr n) "_BIT_X")))
@@ -615,7 +605,7 @@ endmodule
        (x-inst (vl-simple-inst *vl-1-bit-x* "xdriver" x-expr))
 
        (out-wires (vl-make-list-of-bitselects out-expr 0 (- n 1)))
-       (in-wires  (repeat x-expr n))
+       (in-wires  (replicate n x-expr))
        (out-insts (vl-simple-inst-list *vl-1-bit-assign* "bit" out-wires in-wires)))
 
     (list (make-vl-module :name      name
