@@ -1,46 +1,55 @@
 ; VL Verilog Toolkit
-; Copyright (C) 2008-2011 Centaur Technology
+; Copyright (C) 2008-2014 Centaur Technology
 ;
 ; Contact:
 ;   Centaur Technology Formal Verification Group
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 ;
 ; Original author: Jared Davis <jared@centtech.com>
 
 (in-package "VL")
-(include-book "xf-subst")
+(include-book "../mlib/subst")
 (include-book "../mlib/namemangle")
 (include-book "../mlib/relocate")
 (include-book "../mlib/namefactory")
 (include-book "../mlib/port-tools")
 (include-book "../mlib/writer")
-(include-book "../mlib/print-warnings")
 (local (include-book "../util/arithmetic"))
 (local (include-book "../util/osets"))
 
+(defalist vl-renaming-alist-p (x)
+  :key (stringp x)
+  :val (stringp x)
+  :keyp-of-nil nil
+  :valp-of-nil nil)
 
-(local (defthm vl-string-values-p-of-pairlis$
-         (implies (and (force (same-lengthp x y))
-                       (string-listp y))
-                  (vl-string-values-p (pairlis$ x y)))
-         :hints(("Goal" :in-theory (enable pairlis$)))))
-
-(local (defthm vl-string-keys-p-of-pairlis$
-         (implies (string-listp x)
-                  (vl-string-keys-p (pairlis$ x y)))
-         :hints(("Goal" :in-theory (enable pairlis$)))))
-
+(defthm vl-renaming-alist-p-of-pairlis$
+  (implies (and (string-listp x)
+                (string-listp y)
+                (same-lengthp x y))
+           (vl-renaming-alist-p (pairlis$ x y)))
+  :hints(("Goal" :in-theory (enable pairlis$))))
 
 (defxdoc inline-mods
   :parents (transforms)
@@ -98,12 +107,8 @@ clever.</p>")
              (cw "no: inout ports~%"))
          (or (not x.alwayses)
              (cw "no: always blocks~%"))
-         (or (not x.regdecls)
-             (cw "no: reg declarations~%"))
          (or (not x.vardecls)
              (cw "no: var declarations~%"))
-         (or (not x.eventdecls)
-             (cw "no: event declarations~%"))
          (or (not x.paramdecls)
              (cw "no: parameter declarations~%"))
          (or (not x.fundecls)
@@ -129,11 +134,11 @@ clever.</p>")
    (warnings  vl-warninglist-p))
   :returns
   (mv (successp booleanp :rule-classes :type-prescription)
-      (warnings vl-warninglist-p :hyp (force (vl-warninglist-p warnings)))
+      (warnings vl-warninglist-p)
       (assigns  vl-assignlist-p :hyp :fguard))
 
   (b* (((when (atom ports))
-        (mv t warnings nil))
+        (mv t (ok) nil))
        (port1   (car ports))
        (inside  (vl-port->expr port1))
        (outside (vl-plainarg->expr (car plainargs)))
@@ -145,12 +150,11 @@ clever.</p>")
         (mv nil warnings nil))
 
        ((when (eq dir :vl-inout))
-        (b* ((w (make-vl-warning :type :vl-inline-fail
-                                 :msg "Inout ports aren't supported for inlining."
-                                 :args (list (car ports))
-                                 :fn 'vl-make-inlining-assigns
-                                 :fatalp nil)))
-          (mv nil (cons w warnings) nil)))
+        (mv nil
+            (warn :type :vl-inline-fail
+                  :msg "Inout ports aren't supported for inlining."
+                  :args (list (car ports)))
+            nil))
 
        (assigns1
         ;; If the port's expression (inside) is blank (nil), we don't need
@@ -194,8 +198,7 @@ clever.</p>")
 
 (define vl-inline-rename-portdecl
   ((x     vl-portdecl-p)
-   (alist (and (vl-string-keys-p alist)
-               (vl-string-values-p alist))))
+   (alist vl-renaming-alist-p))
   :returns (new-x vl-portdecl-p :hyp :guard)
   (b* ((new-name (or (cdr (hons-get (vl-portdecl->name x) alist))
                      (raise "all portdecls should be bound")
@@ -205,8 +208,7 @@ clever.</p>")
 (defprojection vl-inline-rename-portdecls (x alist)
   (vl-inline-rename-portdecl x alist)
   :guard (and (vl-portdecllist-p x)
-              (vl-string-keys-p alist)
-              (vl-string-values-p alist))
+              (vl-renaming-alist-p alist))
   :result-type vl-portdecllist-p
   :short "Rename portdecls using the renaming alist (which binds old names to
           their new, mangled names).")
@@ -227,46 +229,47 @@ clever.</p>")
       (gateinsts vl-gateinstlist-p :hyp :fguard)
       (assigns   vl-assignlist-p   :hyp :fguard)
       (netdecls  vl-netdecllist-p  :hyp :fguard)
-      (warnings  vl-warninglist-p  :hyp (force (vl-warninglist-p warnings))))
+      (warnings  vl-warninglist-p))
 
   (b* (((vl-modinst x) x)
        ((vl-module sub) sub)
 
        ((unless (equal x.modname sub.name))
         ;; Not an instance of the desired module, do nothing to this instance.
-        (mv nf (list x) nil nil nil warnings))
+        (mv nf (list x) nil nil nil (ok)))
 
-       ((when (vl-arguments->namedp x.portargs))
-        (b* ((w (make-vl-warning :type :vl-inline-fail
-                                 :msg "~a0: can't inline because args aren't resolved."
-                                 :args (list x)
-                                 :fn 'vl-inline-mod-in-modinst
-                                 :fatalp t)))
-          (mv nf (list x) nil nil nil (cons w warnings))))
-       (plainargs (vl-arguments->args x.portargs))
+       ((unless (eq (vl-arguments-kind x.portargs) :plain))
+        (mv nf (list x) nil nil nil
+            (fatal :type :vl-inline-fail
+                   :msg "~a0: can't inline because args aren't resolved."
+                   :args (list x))))
 
+       (plainargs (vl-arguments-plain->args x.portargs))
        ((when (vl-arguments->args x.paramargs))
-        (b* ((w (make-vl-warning :type :vl-inline-fail
-                                 :msg "~a0: can't inline because of parameters."
-                                 :args (list x)
-                                 :fn 'vl-inline-mod-in-modinst
-                                 :fatalp t)))
-          (mv nf (list x) nil nil nil (cons w warnings))))
+        (mv nf (list x) nil nil nil
+            (fatal :type :vl-inline-fail
+                   :msg "~a0: can't inline because of parameters."
+                   :args (list x))))
 
        ((unless (same-lengthp sub.ports plainargs))
-        (b* ((w (make-vl-warning :type :vl-inline-fail
-                                 :msg "~a0: can't inline due to improper arity."
-                                 :args (list x)
-                                 :fn 'vl-inline-mod-in-modinst
-                                 :fatalp t)))
-          (mv nf (list x) nil nil nil (cons w warnings))))
+        (mv nf (list x) nil nil nil
+            (fatal :type :vl-inline-fail
+                   :msg "~a0: can't inline due to improper arity."
+                   :args (list x))))
 
        ;; Mangle and relocate the submodule's guts so that they are all at the
        ;; location of the instance, all the names are fresh, and all the
        ;; expressions have been updated to the new names.
        (prefix           (or x.instname "inst"))
        ((mv netdecls nf) (vl-namemangle-netdecls prefix sub.netdecls nf))
-       (netdecls         (vl-relocate-netdecls x.loc netdecls))
+       (netdecls         (vl-relocate-netdecls
+                          ;; Dumb hack: try to make sure that newly introduced net
+                          ;; declarations come BEFORE any uses of them.
+                          (change-vl-location
+                           x.loc
+                           :line (max 1 (- (vl-location->line x.loc) 1))
+                           :col 0)
+                          netdecls))
        (old-names        (vl-netdecllist->names sub.netdecls))
        (new-names        (vl-netdecllist->names netdecls))
        (new-exprs        (vl-make-idexpr-list new-names nil nil))
@@ -299,12 +302,10 @@ clever.</p>")
         (vl-make-inlining-assigns ports plainargs portdecls palist x.loc warnings))
        (- (fast-alist-free palist))
        ((unless okp)
-        (b* ((w (make-vl-warning :type :vl-inline-fail
-                                 :msg "~a0: problem with inlining port connections."
-                                 :args (list x)
-                                 :fn 'vl-inline-mod-in-modinst
-                                 :fatalp t)))
-          (mv nf (list x) nil nil nil (cons w warnings))))
+        (mv nf (list x) nil nil nil
+            (fatal :type :vl-inline-fail
+                   :msg "~a0: problem with inlining port connections."
+                   :args (list x))))
 
        ;; If we get this far, then the port-assigns are already set and everything
        ;; else is looking good, too.
@@ -328,9 +329,9 @@ clever.</p>")
       (gateinsts vl-gateinstlist-p :hyp :fguard)
       (assigns   vl-assignlist-p   :hyp :fguard)
       (netdecls  vl-netdecllist-p  :hyp :fguard)
-      (warnings  vl-warninglist-p  :hyp (force (vl-warninglist-p warnings))))
+      (warnings  vl-warninglist-p))
   (b* (((when (atom x))
-        (mv nf nil nil nil nil warnings))
+        (mv nf nil nil nil nil (ok)))
        ((mv nf modinsts1 gateinsts1 assigns1 netdecls1 warnings)
         (vl-inline-mod-in-modinst sub (car x) nf warnings))
        ((mv nf modinsts2 gateinsts2 assigns2 netdecls2 warnings)
@@ -349,13 +350,11 @@ clever.</p>")
 
 (define vl-inline-mod-in-mod
   :short "Expand any instances of a submodule into its inlined body, throughout
-a module."
-
+          a module."
   ((sub (and (vl-module-p sub)
              (vl-ok-to-inline-p sub)))
    (x   vl-module-p))
   :returns (new-mod vl-module-p :hyp :guard)
-
   (b* (((vl-module x) x)
        ((when (vl-module->hands-offp x))
         x)
@@ -364,33 +363,22 @@ a module."
         (vl-inline-mod-in-modinsts sub x.modinsts nf x.warnings)))
     (vl-free-namefactory nf)
     (change-vl-module x
-                        :modinsts  modinsts
-                        :gateinsts (append gateinsts x.gateinsts)
-                        :assigns   (append assigns x.assigns)
-                        :netdecls  (append netdecls x.netdecls)
-                        :warnings  warnings))
-  ///
-  (defthm vl-module->name-of-vl-inline-mod-in-mod
-    (equal (vl-module->name (vl-inline-mod-in-mod sub x))
-           (vl-module->name x))))
-
+                      :modinsts  modinsts
+                      :gateinsts (append gateinsts x.gateinsts)
+                      :assigns   (append assigns x.assigns)
+                      :netdecls  (append netdecls x.netdecls)
+                      :warnings  warnings)))
 
 (defprojection vl-inline-mod-in-mods-aux (sub x)
   (vl-inline-mod-in-mod sub x)
   :guard (and (vl-module-p sub)
               (vl-ok-to-inline-p sub)
               (vl-modulelist-p x))
-  :result-type vl-modulelist-p
-  :rest
-  ((defthm vl-modulelist->names-of-vl-inline-mod-in-mods-aux
-     (equal (vl-modulelist->names (vl-inline-mod-in-mods-aux sub x))
-            (vl-modulelist->names x)))))
-
-
+  :result-type vl-modulelist-p)
 
 (define vl-inline-mods
   :short "@(call vl-inline-mods) inlines all modules in @('x') throughout
-@('all-mods')."
+          @('all-mods')."
   ((x        (and (vl-modulelist-p x)
                   (vl-ok-to-inline-list-p x)))
    (all-mods vl-modulelist-p))
@@ -399,10 +387,15 @@ a module."
   (if (atom x)
       all-mods
     (b* ((all-mods (vl-inline-mod-in-mods-aux (car x) all-mods)))
-      (vl-inline-mods (cdr x) all-mods)))
-  ///
-  (defthm vl-modulelist->names-of-vl-inline-mods
-    (equal (vl-modulelist->names (vl-inline-mods x all-mods))
-           (vl-modulelist->names all-mods))))
+      (vl-inline-mods (cdr x) all-mods))))
 
 
+(define vl-design-inline-mods
+  ((mods-to-inline (and (vl-modulelist-p mods-to-inline)
+                        (vl-ok-to-inline-list-p mods-to-inline)))
+   (design vl-design-p))
+  (b* ((design (vl-design-fix design))
+       ((vl-design design) design)
+       (new-mods (vl-inline-mods mods-to-inline design.mods)))
+    (change-vl-design design :mods new-mods)))
+
