@@ -1,20 +1,30 @@
 ; VL Verilog Toolkit
-; Copyright (C) 2008-2013 Centaur Technology
+; Copyright (C) 2008-2014 Centaur Technology
 ;
 ; Contact:
 ;   Centaur Technology Formal Verification Group
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 ;
 ; Original author: Jared Davis <jared@centtech.com>
 
@@ -30,43 +40,22 @@
 ; other, conflicting theorems about.
 
 (include-book "std/util/top" :dir :system)
-(include-book "std/util/defconsts" :dir :system)
 (include-book "std/misc/two-nats-measure" :dir :system)
 (include-book "std/lists/list-defuns" :dir :system)
-(include-book "centaur/bitops/integer-length" :dir :system)
 (include-book "centaur/misc/alist-equiv" :dir :system)
 (include-book "centaur/misc/hons-extra" :dir :system)
-(include-book "str/top" :dir :system)
-(include-book "str/fast-cat" :dir :system)
+(include-book "std/strings/top" :dir :system)
+(include-book "std/strings/fast-cat" :dir :system)
+(include-book "misc/assert" :dir :system)
+(include-book "misc/definline" :dir :system) ;; bozo
+(include-book "std/system/non-parallel-book" :dir :system)
 (local (include-book "arithmetic/top-with-meta" :dir :system))
 (local (include-book "data-structures/list-defthms" :dir :system))
 
-
-(defxdoc utilities
-  :parents (vl)
-  :short "Generic utilities that are unrelated to Verilog processing, but which
-are provided by the VL books.")
-
-(defxdoc transforms
-  :parents (vl)
-  :short "High-level transformations for rewriting and simplifying Verilog
-modules.")
-
-(defxdoc mlib
-  :parents (vl)
-  :short "<b>M</b>odule <b>Lib</b>rary -- A collection of various functions for
-working with Verilog modules.")
-
-
-
-(defsection *nls*
+(defval *nls*
   :parents (utilities)
   :short "A string consisting of a newline character."
-
-  (defconst *nls*
-    (implode (list #\Newline))))
-
-
+  (implode (list #\Newline)))
 
 (define redundant-mergesort
   (x &key
@@ -111,13 +100,20 @@ typical example is:</p>
                  nil))
           (mergesort x)))))
 
-
 (deflist nat-listp (x)
   (natp x)
   :elementp-of-nil nil
   :parents (utilities))
 
 
+(define lposfix ((x posp))
+  :enabled t
+  :inline t
+  :hooks nil
+  (mbe :logic (pos-fix x)
+       :exec x))
+
+
 (deflist vl-maybe-nat-listp (x)
   (maybe-natp x)
   :elementp-of-nil t
@@ -245,9 +241,9 @@ alist, and hence is a hash table lookup.</p>"
 @('a')."
 
   :long "<p>In the logic, we define @('all-equalp') in terms of
-@('repeat').</p>
+@('replicate').</p>
 
-<p>We usually leave this enabled and prefer to reason about @('repeat')
+<p>We usually leave this enabled and prefer to reason about @('replicate')
 instead.  On the other hand, we also sometimes disable it and reason about it
 recursively, so we do prove a few theorems about it.</p>
 
@@ -256,14 +252,14 @@ consing.</p>"
   :enabled t
 
   (mbe :logic
-       (equal (list-fix x) (repeat a (len x)))
+       (equal (list-fix x) (replicate (len x) a))
        :exec
        (if (consp x)
            (and (equal a (car x))
                 (all-equalp a (cdr x)))
          t))
 
-  :guard-hints(("Goal" :in-theory (enable all-equalp repeat)))
+  :guard-hints(("Goal" :in-theory (enable all-equalp replicate)))
 
   ///
   (defrule all-equalp-when-atom
@@ -274,7 +270,7 @@ consing.</p>"
     (equal (all-equalp a (cons b x))
            (and (equal a b)
                 (all-equalp a x)))
-    :enable repeat)
+    :enable replicate)
 
   (local (in-theory (disable all-equalp)))
 
@@ -298,11 +294,11 @@ consing.</p>"
            (all-equalp a x))
     :induct (len x))
 
-  (defrule all-equalp-of-repeat
-    (equal (all-equalp a (repeat b n))
+  (defrule all-equalp-of-replicate
+    (equal (all-equalp a (replicate n b))
            (or (equal a b)
                (zp n)))
-    :enable repeat)
+    :enable replicate)
 
   (defrule all-equalp-of-take
     (implies (all-equalp a x)
@@ -591,10 +587,8 @@ such that @('p') is a prefix of every list in @('x')."
   :long "<p>Note that we leave this function enabled.</p>"
   :enabled t
   :inline t
-
-  (mbe :logic (if (stringp x) x "")
+  (mbe :logic (str-fix x)
        :exec x)
-
   ///
   (defrule stringp-of-string-fix
     (stringp (string-fix x))
@@ -765,79 +759,6 @@ instead.</p>"
 
 
 
-(defsection append-domains
-  :parents (utilities)
-  :short "@(call append-domains) appends all of the values from the alist
-@('x') into a single list."
-
-  :long "<p>Our guard is merely @('t'), but typically @('x') is an alist of
-@('(key . value)') pairs where every @('value') is a list of elements.  We walk
-through the alist, appending together all of the elements of each @('value')
-into a single, flat list.</p>
-
-<p>Note that we do not really treat @('x') as an association list.  That is, we
-will include the values from any \"shadowed pairs\" in the list.</p>"
-
-  (defund append-domains-exec (x acc)
-    (declare (xargs :guard t))
-    (mbe :logic
-         (if (atom x)
-             acc
-           (append-domains-exec (cdr x)
-                                (revappend (cdar x) acc)))
-         :exec
-         (cond ((atom x)
-                acc)
-               ((atom (car x))
-                (append-domains-exec (cdr x) acc))
-               (t
-                (append-domains-exec (cdr x)
-                                     (revappend-without-guard (cdar x) acc))))))
-
-  (local (in-theory (enable append-domains-exec)))
-
-  (local (defrule true-listp-of-append-domains-exec
-           (implies (true-listp acc)
-                    (equal (true-listp (append-domains-exec x acc))
-                           t))
-           :rule-classes :type-prescription))
-
-  (defun append-domains (x)
-    (declare (xargs :guard t :verify-guards nil))
-    (mbe :logic
-         (if (atom x)
-             nil
-           (append (cdar x) (append-domains (cdr x))))
-         :exec (reverse (append-domains-exec x nil))))
-
-  (local (in-theory (enable append-domains)))
-
-  (defrule append-domains-exec-removal
-    (equal (append-domains-exec x acc)
-           (revappend (append-domains x) acc))
-    :induct (append-domains-exec x acc))
-
-  (verify-guards append-domains)
-
-  (defrule append-domains-when-atom
-    (implies (atom x)
-             (equal (append-domains x)
-                    nil)))
-
-  (defrule append-domains-of-cons
-    (equal (append-domains (cons a x))
-           (append (cdr a) (append-domains x))))
-
-  (defrule true-listp-of-append-domains
-    (true-listp (append-domains x))
-    :rule-classes :type-prescription)
-
-  (defrule append-domains-of-append
-    (equal (append-domains (append x y))
-           (append (append-domains x) (append-domains y)))))
-
-
-
 (define fast-alist-free-each-alist-val (x)
   :parents (utilities)
   :short "Applies @(see fast-alist-free) to every value bound in the alist x."
@@ -1115,3 +1036,19 @@ occasionally useful, e.g., when defining structures using macros that expect
 constraints for certain fields.</p>"
   (declare (ignore x))
   t)
+
+
+(defenum vl-edition-p
+  (:verilog-2005 :system-verilog-2012)
+  :parents (utilities)
+  :short "Editions of the Verilog or SystemVerilog standards that VL attempts
+to implement."
+
+  :long "<p>Certain parts of VL are configurable and can try follow different
+versions of the standard.  We currently have some support for:</p>
+
+<ul>
+<li>@(':verilog-2005') corresponds to IEEE Std 1364-2005.</li>
+<li>@(':system-verilog-2012') corresponds to IEEE Std 1800-2012.</li>
+</ul>")
+
