@@ -6,6 +6,9 @@
 
 (in-package "ACL2")
 
+;; Jared added this because this book doesn't certify in ACL2(r).
+;; cert_param: (non-acl2r)
+
 ;Data has separate package namespace 'defdata' and which implements
 ;custom data definitions, type constructors(product and union)
 (include-book "defdata")
@@ -15,6 +18,12 @@
 (include-book "library-support")
 (include-book "random-state")
 
+(make-event 
+ (er-progn
+  (defdata::set-acl2s-defdata-verbose t)
+  (value '(value-triple :invisible)))
+ :check-expansion t)
+
 ;TODO.NOTE: constructors are now stored in globals and it 
 ;seems that include-book does not carry globals?? is it true?
 
@@ -41,26 +50,27 @@
 ;;rational number constructor
 ;;pair constructor
 (register-data-constructor (consp cons)
-                           ((allp car) (allp cdr))
-                           :proper t)
+                           ((allp car) (allp cdr)))
+                
 
 ;;jared's oset implementation
-(defun sets::non-empty-setp (x)
+(defun set::non-empty-setp (x)
   (declare (xargs :guard t))
-  (and (sets::setp x)
-       (not (sets::empty x))))
+  (and (set::setp x)
+       (not (set::empty x))))
 
-(register-data-constructor (SETS::non-empty-setp SETS::insert)
-                           ((allp SETS::head) (sets::setp SETS::tail))
+(register-data-constructor (SET::non-empty-setp SET::insert)
+                           ((allp SET::head) (set::setp SET::tail))
                            :proper nil)
 
 
 
 
-
 ;;symbols
 (register-data-constructor (symbolp intern$)
-                           ((stringp symbol-name) (stringp symbol-package-name)))
+                           ((stringp symbol-name) (stringp symbol-package-name))
+                           :proper nil) ;package name destructor fails
+
 
 
 (register-data-constructor (rationalp /)
@@ -76,14 +86,13 @@
 
 (register-data-constructor (aconsp acons)
                            (caar cdar cdr)
-                           :proper t
-                           :rule-classes nil;(:rewrite)
-                           :forcep t) 
+                           :rule-classes nil);(:rewrite)
+
 
 ;;complex number type
 (register-data-constructor (acl2-numberp complex)
-                           ((rationalp realpart) (rationalp imagpart))
-                           :proper t)
+                           ((rationalp realpart) (rationalp imagpart)))
+
 
 #||
 ;;natural numbers
@@ -164,7 +173,7 @@
 (defthm get-character-list-from-positions--character-listp
   (implies (naturals-listp l)
            (and (character-listp (get-character-list-from-positions l))
-                (standard-char-listp (get-standard-char-list-from-positions l))))
+                (character-listp (get-standard-char-list-from-positions l))))
   :hints (("goal" :in-theory (enable standard-char-listp))))
 
 )
@@ -201,6 +210,12 @@
          n)
        :exec (1+ n)))
 
+(defthm nth-pos-is-posp
+  (implies (natp x)
+           (posp (nth-pos x)))
+  :hints (("goal" :in-theory (enable nth-pos)))
+  :rule-classes (:rewrite :type-prescription))
+
 (defexec pos-index (i)
   (declare (xargs :guard (posp i)))
   (mbe :logic
@@ -416,6 +431,13 @@
          (den (nth-pos (cadr two-n-list))))
     (/ num den)))
 
+
+
+(defthm nth-rat-is-ratp
+  (implies (natp x)
+           (rationalp (nth-rational x)))
+  :rule-classes (:rewrite :type-prescription))
+
  ;lo included, hi included
     
   
@@ -604,6 +626,29 @@
 ;(register-custom-type boolean 2 *boolean-values*  booleanp );taken care of by define-enumeration-type
 (register-custom-type symbol t nth-symbol  symbolp)
 
+(defun proper-symbolp (x)
+  (declare (xargs :guard t))
+  (and (symbolp x)
+       (not (or (keywordp x);a keyword
+                (booleanp x);t or nil
+                (legal-constantp x)))))
+
+(in-theory (disable legal-constantp))
+
+(defconst *nice-symbol-names*
+  '(x y z a b c i j k p q r s u v w l d e f g h m n))
+
+(defun nth-proper-symbol (n)
+  (declare (xargs :guard (natp n)))
+  (let ((psym (nth-symbol n)))
+    (if (proper-symbolp psym)
+        psym
+      (nth (mod n (len *nice-symbol-names*)) *nice-symbol-names*))))
+
+(register-custom-type proper-symbol t nth-proper-symbol proper-symbolp)
+
+
+
 ;MAKE SURE THIS IS ALWAYS SYNCED, if you change character-values then change here too!
 ;(len *character-values*) = 62
 (defun nth-character (n)
@@ -661,6 +706,7 @@
 (defdata-subtype rational acl2-number)
 (defdata-subtype acl2-number atom)
 (defdata-subtype boolean symbol)
+(defdata-subtype proper-symbol symbol)
 (defdata-subtype character atom)
 (defdata-subtype string atom)
 (defdata-subtype symbol atom)
@@ -688,6 +734,7 @@
 (defdata-disjoint character string)
 (defdata-disjoint character symbol)
 (defdata-disjoint string symbol)
+(defdata-disjoint boolean proper-symbol)
 
 ;(assign make-event-debug t)
 ;lists of atoms
@@ -932,42 +979,23 @@
   (declare (xargs :guard (natp n)))
   (nth-all-but-zero-nil-t n))
 
-(register-custom-type wf-key t nth-wf-key  wf-keyp)
+(register-custom-type wf-key t nth-wf-key wf-keyp)
+
+;; Same problem as in sets. A nil is also a good-map!
+;; 3 April 2014
+(defun non-empty-good-map (x)
+  (declare (xargs :guard t))
+  (and (consp x)
+       (good-map x)))
+
+(defun all-but-nilp (x)
+  (declare (xargs :guard t))
+  (not (equal x 'nil)))
+; TODO: this is a major hiccup of our map and record implementation, disallowing nil explicitly!!
+(register-data-constructor (non-empty-good-map mset)
+                           ((wf-keyp caar) (all-but-nilp cdar) (good-map cdr))
+                           :proper nil)
 
-;; (register-data-constructor (good-map mset)
-;;                            ((wf-keyp caar) (allp cdar) (good-map cdr))
-;;                            :proper nil)
-(PROGN (DEFTHM MSET-CONSTRUCTOR-PRED
-                (IMPLIES (AND (WF-KEYP CAAR)
-                              (ALLP CDAR)
-                              (GOOD-MAP CDR))
-                         (GOOD-MAP (MSET CAAR CDAR CDR)))
-                :HINTS NIL
-                :RULE-CLASSES NIL)
-        ;; (DEFTHM MSET-CONSTRUCTOR-ELIM-RULE
-        ;;         (IMPLIES (GOOD-MAP X)
-        ;;                  (EQUAL (MSET (CAAR X) (CDAR X) (CDR X))
-        ;;                         X))
-        ;;         :HINTS NIL
-        ;;         :RULE-CLASSES NIL)
-        (DEFTHM MSET-CONSTRUCTOR-DESTRUCTORS
-          (IMPLIES (GOOD-MAP X)
-                         (AND (WF-KEYP (CAAR X))
-                              (ALLP (CDAR X))
-                              (GOOD-MAP (CDR X))))
-                :RULE-CLASSES NIL
-                :HINTS NIL)
-        (TABLE DATA-CONSTRUCTORS 'MSET
-               '(NIL GOOD-MAP
-                     ((CAAR . WF-KEYP)
-                      (CDAR . ALLP)
-                      (CDR . GOOD-MAP))
-                     . DEFDATA::NONE)
-               :PUT)
-        (VALUE-TRIPLE (LIST '(GOOD-MAP MSET)
-                            '((WF-KEYP CAAR)
-                              (ALLP CDAR)
-                              (GOOD-MAP CDR)))))
 
 (defdata-subtype all-but-zero-nil-t all)
 
@@ -1090,34 +1118,34 @@
 ;  `(is-disjoint ',T1 ',T2 R$ types-ht$))
 
 
-(defmacro show-acl2s-defdata-all-types ()
+(defmacro show-all-defdata-types ()
   `(table-alist 'defdata::types-info-table (w state)))
 
-(defmacro subtype-p (T1 T2)
-   ":Doc-Section DATA-DEFINITIONS
-  top-level query wether two types are disjoint~/
-  ~c[(subtype-p T1 T2)] asks the question
-  is T1 a subtype of T2? This call makes a quick
-  lookup into the internal data type graph where
-  subtype relation information provided by the user
-  in the past is stored and used to compute the
-  subtype relation closure. If T1 is indeed a subtype
-  of T2 (according to the computed information)
-  then we get back an affirmative , i.e ~c[t]. otherwise
-  it returns ~c[nil].
+;; (defmacro subtype-p (T1 T2)
+;;    ":Doc-Section DATA-DEFINITIONS
+;;   top-level query wether two types are disjoint~/
+;;   ~c[(subtype-p T1 T2)] asks the question
+;;   is T1 a subtype of T2? This call makes a quick
+;;   lookup into the internal data type graph where
+;;   subtype relation information provided by the user
+;;   in the past is stored and used to compute the
+;;   subtype relation closure. If T1 is indeed a subtype
+;;   of T2 (according to the computed information)
+;;   then we get back an affirmative , i.e ~c[t]. otherwise
+;;   it returns ~c[nil].
   
-  ~bv[]
-  Examples:
-  (subtype-p boolean atom)
-  (subtype-p character string)
-  (subtype-p list cons)
-  ~ev[]                      
-  ~bv[]
-  Usage:
-  (subtype-p <Type-name1> <Type-name2>)
-  ~ev[]~/
-  "
-   `(trans-eval '(defdata::is-subtype$$ ',t1 ',t2 defdata::R$ defdata::types-ht$) 'subtype-p state nil))
+;;   ~bv[]
+;;   Examples:
+;;   (subtype-p boolean atom)
+;;   (subtype-p character string)
+;;   (subtype-p list cons)
+;;   ~ev[]                      
+;;   ~bv[]
+;;   Usage:
+;;   (subtype-p <Type-name1> <Type-name2>)
+;;   ~ev[]~/
+;;   "
+;;    `(trans-eval '(defdata::is-subtype$$ ',t1 ',t2 defdata::R$ defdata::types-ht$) 'subtype-p state nil))
   ;`(is-subtype$$ ',T1 ',T2 R$ types-ht$))
 
 ;; (defun is-subtype (t1 t2 state)
@@ -1137,4 +1165,30 @@
 
 (defun map-identity (x) 
   "for map elim rules -- dummy destructor"
-  x)
\ No newline at end of file
+  x)
+
+
+; TODO 29 March 2014
+; - add oddp and evenp (but do this consistently, these definitions are only valid when we additionally know that v is a integer.
+(defun nth-even (n) 
+  (declare (xargs :guard (natp n)))
+  (* 2 (nth-integer n)))
+
+(register-type even 
+               :predicate evenp 
+               :enumerator nth-even 
+               :type-class basic)
+
+(defun nth-odd (n) 
+  (declare (xargs :guard (natp n)))
+  (if (evenp n)
+      (1+ n)
+    (- 0 n)))
+
+;(defun nth-odd (n) (1+ (* 2 (nth-integer))))
+(register-type odd 
+               :predicate oddp 
+               :enumerator nth-odd 
+               :type-class basic)
+
+
