@@ -89,17 +89,34 @@
 
 ;This might be needed for termination arguments for SETS
 (defthm non-nil-=>-not-empty
-  (implies (and (sets::setp v)
+  (implies (and (set::setp v)
                 (not (equal v nil)))
-           (not (sets::empty v)))
-  :hints (("Goal" :in-theory (enable sets::empty)))
+           (not (set::empty v)))
+  :hints (("Goal" :in-theory (enable set::empty)))
   :rule-classes :forward-chaining)
 
+;; (defthm good-map-implies-not-ifmp TODO CHECK - while testing TAU, we disabled this theorem because it was too expensive.
+;;   (implies (good-map x) 
+;;            (and (not (ifmp x))
+;;                 (well-formed-map x)))
+;;   :rule-classes ((:rewrite :backchain-limit-lst 1)
+;;                  (:forward-chaining)))
+
+;The following 3 rewrite rules are important for map modifier lemmas generated by defdata
 (defthm good-map-implies-not-ifmp
-  (implies (good-map x) (and (not (ifmp x))
-                             (well-formed-map x)))
-  :rule-classes ((:rewrite :backchain-limit-lst 1)
-                 (:forward-chaining)))
+  (implies (good-map x) (not (ifmp x)))
+  :rule-classes ((:rewrite :backchain-limit-lst 1)))
+                 ;(:forward-chaining)))
+  
+(defthm acl2->map-reduction
+  (implies (good-map x)
+           (equal (ACL2->MAP x) x))
+  :hints (("goal" :in-theory (e/d (acl2->map) (ifmp)))))
+
+(defthm map->acl2-reduction
+  (implies (good-map x)
+           (equal (MAP->ACL2 x) x))
+  :hints (("goal" :in-theory (e/d (MAP->ACL2) (ifmp)))))
 
 
 (local
@@ -116,3 +133,10 @@
                (wf-keyp a))
           (consp (MSET A V x)))
  :hints (("goal" :in-theory (enable map->acl2 acl2->map extensible-records))))
+
+(defthm mset-wf-key-non-boolean-val-is-consp
+ (IMPLIES (AND (not (booleanp v))
+               (wf-keyp a))
+          (consp (MSET A V x)))
+ :hints (("goal"  :use ((:instance mset-wf-key-non-nil-val-is-consp
+                                   (a a) (v v) (x x))))))
\ No newline at end of file
