@@ -6,15 +6,25 @@
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 ;
 ; Original author: Jared Davis <jared@centtech.com>
 
@@ -58,8 +68,8 @@
 
 
 ;; We can define an arbitrary sort using <<.  This is almost the same as
-;; SETS::mergesort in the ordered sets library, except that defsorts are
-;; always duplicate-preserving while SETS::mergesort throws away identical
+;; SET::mergesort in the ordered sets library, except that defsorts are
+;; always duplicate-preserving while SET::mergesort throws away identical
 ;; elements.
 
 (defsort :compare< <<
@@ -183,7 +193,7 @@
 (progn (ccl::gc)
        (time (loop for i fixnum from 1 to 1000
                    do
-                   (let ((result (SETS::mergesort (cons i *integers*))))
+                   (let ((result (SET::mergesort (cons i *integers*))))
                      (declare (ignore result))
                      nil))))
 
@@ -218,7 +228,7 @@
 (progn (ccl::gc)
        (time (loop for i fixnum from 1 to 1000
                    do
-                   (let ((result (SETS::mergesort (cons "foo" *strings*))))
+                   (let ((result (SET::mergesort (cons "foo" *strings*))))
                      (declare (ignore result))
                      nil))))
 
@@ -254,4 +264,4 @@
 
 
 
-||#
\ No newline at end of file
+||#
