@@ -6,15 +6,25 @@
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 ;
 ; Original author: Jared Davis <jared@centtech.com>
 
@@ -24,11 +34,11 @@
 (include-book "tools/include-raw" :dir :system)
 (local (include-book "std/typed-lists/string-listp" :dir :system))
 
+(local (xdoc::set-default-parents oslib))
 
 (define argv (&optional (state 'state))
   :returns (mv (arguments string-listp)
                (state state-p1 :hyp (force (state-p1 state))))
-  :parents (oslib)
   :short "Get the \"application level\" command line arguments passed to ACL2."
 
   :long "<p>Typically, @('(argv)') is useful for writing command-line programs
@@ -119,7 +129,6 @@ any better, so that's just how it is.</p>"
 (define date (&optional (state 'state))
   :returns (mv (datestamp stringp :rule-classes :type-prescription)
                (state state-p1 :hyp (force (state-p1 state))))
-  :parents (oslib)
   :short "Get the current datestamp, like \"November 17, 2010 10:25:33\"."
 
   :long "<p>In the logic this function reads from the ACL2 oracle.  In the
@@ -142,7 +151,6 @@ Common Lisp system.</p>"
                         (not pid))
                     :rule-classes :type-prescription)
                (state state-p1 :hyp (force (state-p1 state))))
-  :parents (oslib)
   :short "Get the current process identification (PID) number."
 
   :long "<p>This will just fail if called on an unsupported Lisp.</p>"
@@ -179,7 +187,6 @@ Common Lisp system.</p>"
                            (equal (string-listp val) (not error))))
                (state state-p1
                       :hyp (force (state-p1 state))))
-  :parents (oslib)
   :short "Get a subdirectory listing."
 
   :long "<p>In the logic this function reads from the ACL2 oracle.  In the
@@ -214,7 +221,6 @@ not full paths like @('\"/home/users/jared/foo\"').</p>"
                          (equal (string-listp val) (not error))))
                (state state-p1
                       :hyp (force (state-p1 state))))
-  :parents (oslib)
   :short "Get a file listing."
 
   :long "<p>In the logic this function reads from the ACL2 oracle.  In the
@@ -245,7 +251,6 @@ subdirectories) of the given @('path').</p>
                           permissions, illegal file names, etc.")
                (state state-p1
                       :hyp (force (state-p1 state))))
-  :parents (oslib)
   :short "Make new directories if they don't already exist, like @('mkdir -p'),
 and return a success indicator so you can recover from errors."
 
@@ -268,7 +273,6 @@ errors.</p>"
                &optional
                (state 'state))
   :returns (state state-p1 :hyp (force (state-p1 state)))
-  :parents (oslib)
   :short "Make new directories if they don't already exist, like @('mkdir -p'),
 or cause a hard error if there is any problem."
 
@@ -293,7 +297,6 @@ failure.</p>"
                           permissions, illegal file names, etc.")
                (state state-p1
                       :hyp (force (state-p1 state))))
-  :parents (oslib)
   :short "Recursively delete files, like the shell command @('rm -rf'), and
 return a success indicator so you can recover from errors."
 
@@ -309,3 +312,46 @@ detect errors.</p>"
                  val)))
     (mv okp state)))
 
+(define lisp-type (&optional (state 'state))
+  :returns (mv (description stringp :rule-classes :type-prescription
+                            "E.g., @('\"Clozure Common Lisp\"').")
+               (state state-p1 :hyp (force (state-p1 state))))
+  :short "Get a host-Lisp specific string describing what kind of Common Lisp
+implementation this is."
+
+  :long "<p>In the logic this function reads from the ACL2 oracle.  In the
+execution, we call the Common Lisp function @('lisp-implementation-type'), and
+return whatever string it produces.</p>
+
+<p>Note that the Common Lisp @('lisp-implementation-type') function is
+technically allowed to return @('nil'); in this case we return the empty
+string.</p>"
+
+  (b* (((mv err val state) (read-acl2-oracle state))
+       (description (if (and (not err)
+                             (stringp val))
+                        val
+                      "")))
+    (mv description state)))
+
+(define lisp-version (&optional (state 'state))
+  :returns (mv (description stringp :rule-classes :type-prescription
+                            "E.g., @('\"Version 1.9-r15996  (LinuxX8664)\"').")
+               (state state-p1 :hyp (force (state-p1 state))))
+  :short "Get a host-Lisp specific string describing the version number for
+this Common Lisp implementation."
+
+  :long "<p>In the logic this function reads from the ACL2 oracle.  In the
+execution, we call the Common Lisp function @('lisp-implementation-version'),
+and return the string it produces.</p>
+
+<p>Note that the Common Lisp @('lisp-implementation-type') function is
+technically allowed to return @('nil'); in this case we return the empty
+string.</p>"
+
+  (b* (((mv err val state) (read-acl2-oracle state))
+       (description (if (and (not err)
+                             (stringp val))
+                        val
+                      "")))
+    (mv description state)))
