@@ -18,14 +18,14 @@
 
 (in-package "ACL2")
 
-(include-book "str/top" :dir :system)
+(include-book "std/strings/top" :dir :system)
 (include-book "std/util/defaggregate" :dir :system)
 (include-book "std/util/deflist" :dir :system)
 (include-book "std/lists/repeat" :dir :system) ; redundant
 (include-book "tools/bstar" :dir :system)
 (include-book "std/util/defconsts" :dir :system)
 (include-book "xdoc/top" :dir :system)
-
+(include-book "misc/assert" :dir :system)
 (include-book "object-representations")
 (include-book "grammar-reader" :skip-proofs-okp t)
 (include-book "strings")
@@ -563,7 +563,7 @@
   (b* ((words (remove-equal "" (str::strtok sentence '(#\Space))))
        ;; Initialize charts, one chart per word in the sentence (plus an extra
        ;; chart that we won't use, in slot 0)
-       (chart-list (repeat (make-chart) (1+ (length words))))
+       (chart-list (replicate (1+ (length words)) (make-chart)))
        ;; Start off by enqueuing a dummy state in the first chart
        (chart-list (update-nth 0 (enqueue (make-pstate :source "G"
                                                        :targets '("S")
