@@ -21,10 +21,10 @@
 ;(include-book "std/util/defaggregate" :dir :system)
 (include-book "std/util/deflist" :dir :system)
 ;(include-book "tools/bstar" :dir :system)
-(include-book "str/top" :dir :system)
+(include-book "std/strings/top" :dir :system)
 (include-book "std/io/base" :dir :system)
 (include-book "std/util/define" :dir :system)
-(include-book "parallel/with-waterfall-parallelism" :dir :system)
+(include-book "misc/with-waterfall-parallelism" :dir :system)
 
 (include-book "object-representations")
 (include-book "strings")
