@@ -18,7 +18,7 @@
 
 (in-package "ACL2")
 
-(include-book "str/top" :dir :system)
+(include-book "std/strings/top" :dir :system)
 (include-book "std/util/define" :dir :system)
 
 (defconst *token-separators*
