@@ -6,15 +6,25 @@
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 ;
 ; Original author: Jared Davis <jared@centtech.com>
 
@@ -124,12 +134,31 @@
 ;
 ;   (2) we extend it with the :typed-record field, etc.
 
+(defmacro access-defstobj-field-template (x field)
+  `(acl2::access acl2::defstobj-field-template ,x ,field))
+
+(defun destructure-defstobj-field-template (x)
+
+  ;; BOZO this has to be kept in sync with defstobj-field-template.
+
+  (let ((fieldp-name (access-defstobj-field-template x :fieldp-name))
+        (type (access-defstobj-field-template x :type))
+        (init (access-defstobj-field-template x :init))
+        (accessor-name (access-defstobj-field-template x :accessor-name))
+        (updater-name (access-defstobj-field-template x :updater-name))
+        (length-name (access-defstobj-field-template x :length-name))
+        (resize-name (access-defstobj-field-template x :resize-name))
+        (resizable (access-defstobj-field-template x :resizable))
+        (other (access-defstobj-field-template x :other)))
+    (list fieldp-name type init accessor-name updater-name length-name
+          resize-name resizable other)))
+
 (defun make-fta (field-name defstobj-fld-template tr-alist mksym-pkg)
 
   (b* (((list recog-name type init accessor-name updater-name length-name
-              resize-name resizable)
-        ;; BOZO this has to be kept in sync with defstobj-template
-        defstobj-fld-template)
+              resize-name resizable other)
+        ;; BOZO this has to be kept in sync with defstobj-field-template
+        (destructure-defstobj-field-template defstobj-fld-template))
 
 ; For array fields, we'll add an :arr-rec-name field that gives the name of the
 ; additional record part of the array+record pair we'll use to implement this
@@ -158,6 +187,7 @@
       (:length-name   . ,length-name)
       (:resize-name   . ,resize-name)
       (:resizable     . ,resizable)
+      (:other         . ,other)
       (:offset        . ,(acl2::defconst-name accessor-name))
       (:arr-rec-name  . ,arr-rec-name)
       (:typed-record  . ,typed-record))))
@@ -813,6 +843,27 @@
        (equal-when-weak-stp-rhs (cdr ftas))))))
 
 
+(defmacro access-defstobj-template (x field)
+  `(acl2::access acl2::defstobj-template ,x ,field))
+
+(defun destructure-defstobj-template (x)
+
+  ;; BOZO this has to be kept in sync with defstobj-template.
+
+  (let ((recognizer (access-defstobj-template x :recognizer))
+        (creator (access-defstobj-template x :creator))
+        (field-templates (access-defstobj-template x :field-templates))
+        (doc (access-defstobj-template x :doc))
+        (inline (access-defstobj-template x :inline))
+        (congruent-to (access-defstobj-template x :congruent-to))
+        (non-memoizable (access-defstobj-template x :non-memoizable)))
+    (list recognizer
+          creator
+          field-templates
+          doc
+          inline
+          congruent-to
+          non-memoizable)))
 
 (defun defrstobj-fn (name args wrld)
   (declare (ignorable wrld)
@@ -830,9 +881,10 @@
        (st-fields     (eat-typed-records rsfs))
        (st-kw-part    (alist-to-keyword-alist st-kw-alist nil))
        (st-template   (defstobj-template name (append st-fields st-kw-part) wrld))
-       ((list namep create-name st-fld-templates ?doc ?inline)
+       ((list namep create-name st-fld-templates ?doc ?inline ?congruent-to
+              ?non-memoizable)
         ;; BOZO this has to be kept in sync with defstobj-template.
-        st-template)
+        (destructure-defstobj-template st-template))
 
        (ftas (make-ftas (strip-cars tr-alist) st-fld-templates tr-alist mksym-pkg))
 
@@ -866,7 +918,8 @@
        ;; Regenerate the templates that the extended form will use, so that
        ;; we'll have FTAs for the array records and the other extra fields.
        (ext-st-template (defstobj-template name args wrld))
-       ((list & & ext-st-fld-templates & &) ext-st-template)
+       ((list & & ext-st-fld-templates & &)
+        (destructure-defstobj-template ext-st-template))
        (ext-ftas (make-ftas (strip-cars st-fields+) ext-st-fld-templates tr-alist mksym-pkg))
 
        (misc-fta           (find-fta misc-name ext-ftas))
