@@ -37,11 +37,12 @@
   (declare (xargs :guard (real/rationalp x)))
   (floor x 1))
 
-
 ;;;**********************************************************************
 ;;;				BVECP
 ;;;**********************************************************************
 
+(defsection-rtl |Recognizing Bit Vectors| |Bit Vectors|
+
 (defund bvecp (x k)
   (declare (xargs :guard (integerp k)))
   (and (integerp x)
@@ -98,12 +99,14 @@
 		(not (equal x 0)))
 	   (equal x 1))
   :rule-classes :forward-chaining)
-
+)
 
 ;;;**********************************************************************
 ;;;			    BITS
 ;;;**********************************************************************
 
+(defsection-rtl |Bit Slices| |Bit Vectors|
+
 (defund bits (x i j)
   (declare (xargs :guard (and (integerp x)
                               (integerp i)
@@ -163,7 +166,7 @@
 	   (equal (bits x i 0)
 		  (mod x (expt 2 (1+ i))))))
 
-(defthmd bits-bits-sum
+(defthm bits-bits-sum
   (implies (and (integerp x)
                 (integerp y)
                 (integerp i)
@@ -174,7 +177,7 @@
 	   (equal (bits (+ (bits x k 0) y) i j)
 		  (bits (+ x y) i j))))
 
-(defthmd bits-bits-sum-alt
+(defthm bits-bits-sum-alt
   (implies (and (integerp x)
                 (integerp y)
                 (integerp i)
@@ -399,12 +402,14 @@
 
 (defun bitvec (x n)
   (if (bvecp x n) x 0))
-
+)
 
 ;;;**********************************************************************
 ;;;				BITN
 ;;;**********************************************************************
 
+(defsection-rtl |Bit Extraction| |Bit Vectors|
+
 (defund bitn (x n)
   (declare (xargs :guard (and (integerp x)
                               (integerp n))))
@@ -501,7 +506,10 @@
 	     (equal (bitn x n) 0)))
 
 
-(defthm neg-bitn-2
+;; The rewrite rule neg-bitn-2 has the hypotheses (integerp n) and (< x (- (expt 2 k) (expt 2 n))), 
+;; where n does not occur in the lhs.  When n is bound to a large integer, (expt 2 n) blows up. 
+
+(defthmd neg-bitn-2
     (implies (and (integerp x)
 		  (integerp n)
 		  (integerp k)
@@ -643,12 +651,14 @@
 		  (equal (mod c (expt 2 (1+ n))) 0))
 	     (equal (bitn (+ c x) n)
 		    (bitn x n))))
-
+)
 
 ;;;**********************************************************************
 ;;;			     CAT
 ;;;**********************************************************************
 
+(defsection-rtl |Concatenation| |Bit Vectors|
+
 (defund binary-cat (x m y n)
   (declare (xargs :guard (and (integerp x)
                               (integerp y)
@@ -939,11 +949,14 @@
 		(case-split (integerp n)))
 	   (equal (bitn (mulcat 1 n x) m)
 		  x)))
+)
 
 ;;;**********************************************************************
 ;;;		      Signed Integer Encodings
 ;;;**********************************************************************
 
+; This section is not documented in Russinoff's online manual.
+
 (defun intval (w x)
   (if (= (bitn x (1- w)) 1)
       (- x (expt 2 w))
