@@ -1,5 +1,5 @@
-; RTL - A Formal Theory of Register-Transfer Logic and Computer Arithmetic 
-; Copyright (C) 1995-2013 Advanced Mirco Devices, Inc. 
+; RTL - A Formal Theory of Register-Transfer Logic and Computer Arithmetic
+; Copyright (C) 1995-2013 Advanced Mirco Devices, Inc.
 ;
 ; Contact:
 ;   David Russinoff
@@ -115,6 +115,8 @@
 ;;;                         Truncation
 ;;;**********************************************************************
 
+(defsection-rtl |Truncation| |Rounding|
+
 (defund trunc (x n)
   (declare (xargs :guard (integerp n)))
   (* (sgn x) 
@@ -327,6 +329,19 @@
                  (bits x (1- n) (- n k)))))
   :rule-classes ())
 
+(defthm bits-trunc-bits
+  (implies (and (rationalp x)
+                (>= x 0)
+                (integerp k)
+                (integerp i)
+                (integerp j)
+                (> k 0)
+                (>= (expo x) i)
+                (>= j (- (1+ (expo x)) k)))
+           (= (bits (trunc x k) i j)
+              (bits x i j)))
+  :rule-classes ())
+
 (defthm trunc-logand
   (implies (and (>= x (expt 2 (1- n)))
                 (< x (expt 2 n))
@@ -349,11 +364,14 @@
                   (+ (trunc x k)
                      (* (expt 2 (- n m))
                         (bits x (1- (- n k)) (- n m)))))))
+)
 
 ;;;**********************************************************************
 ;;;                    Rounding Away from Zero
 ;;;**********************************************************************
 
+(defsection-rtl |Rounding Away from Zero| |Rounding|
+
 (defund away (x n)
   (* (sgn x) 
      (cg (* (expt 2 (1- n)) (sig x))) 
@@ -613,11 +631,14 @@
 			  (- (expt 2 (- (1+ (expo x)) m))))
 		       n)))
   :rule-classes ())
+)
 
 ;;;**********************************************************************
 ;;;                    Unbiased Rounding
 ;;;**********************************************************************
 
+(defsection-rtl |Unbiased Rounding| |Rounding|
+
 (defun re (x)
   (- x (fl x)))
 
@@ -1074,11 +1095,14 @@
 	     (= (near+ (trunc x n) m)
 		(near+ x m)))
   :rule-classes ())
+)
 
 ;;;**********************************************************************
 ;;;                          Sticky Rounding
 ;;;**********************************************************************
 
+(defsection-rtl |Sticky Rounding| |Rounding|
+
 (defund sticky (x n)
   (cond ((exactp x (1- n)) x)
 	(t (+ (trunc x (1- n))
@@ -1218,12 +1242,14 @@
 	     (= (+ x (sticky y k))
 		(sticky (+ x y) k2)))
   :rule-classes ())
-
+)
 
 ;;;**********************************************************************
 ;;;                    IEEE Rounding
 ;;;**********************************************************************
 
+(defsection-rtl |IEEE Rounding| |Rounding|
+
 (defun inf (x n)
   (if (>= x 0)
       (away x n)
@@ -1469,33 +1495,40 @@
 		  (trunc (+ x (rnd-const (expo x) mode n)) n))))
   :rule-classes ())
 
-(defun roundup (x mode n)
+(defun round-up (x sticky mode n)
   (case mode
     (near+ (= (bitn x (- (expo x) n)) 1))
     (near (and (= (bitn x (- (expo x) n)) 1)
-               (or (not (exactp x (1+ n)))
+               (or (not (= (bits x (- (expo x) (1+ n)) 0) 0))
+                   (= sticky 1)
                    (= (bitn x (- (1+ (expo x)) n)) 1))))
-    ((inf away) (not (exactp x n)))
+    ((inf away) (or (not (= (bits x (- (expo x) n) 0) 0))
+                    (= sticky 1)))
     (otherwise ())))
 
-(defthm roundup-thm
-    (implies (and (common-rounding-mode-p mode)
-		  (integerp n)
-		  (> n 1)
-		  (integerp x)
-		  (> x 0)
-		  (>= (expo x) n))
-	     (= (rnd x mode n)
-                (if (roundup x mode n)
-                    (fp+ (trunc x n) n)
-                  (trunc x n))))
+(defthm round-up-thm
+  (implies (and (common-rounding-mode-p mode)
+                (rationalp r)
+                (> r 0)
+                (not (zp n))
+                (natp k)
+                (< n k)
+                (<= k (1+ (expo r))))
+           (let* ((x (trunc r k))
+                  (sticky (if (< x r) 1 0)))
+	     (= (rnd r mode n)
+                (if (round-up x sticky mode n)
+                    (fp+ (trunc r n) n)
+                  (trunc r n)))))
   :rule-classes ())
-  
+)
 
 ;;;**********************************************************************
 ;;;                         Denormal Rounding 
 ;;;**********************************************************************
 
+(defsection-rtl |Denormal Rounding| |Rounding|
+
 (defund drnd (x mode p q)
   (rnd x mode (+ p (expo x) (- (expo (spn q))))))
 
@@ -1664,3 +1697,4 @@
              (equal (drnd x mode p q)
                     (drnd y mode p q)))
     :rule-classes nil)
+)
