@@ -6,28 +6,45 @@
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 ;
 ; Original authors: Jared Davis <jared@centtech.com>
 ;                   Sol Swords <sswords@centtech.com>
 
 (in-package "ACL2")
+(include-book "xdoc/top" :dir :system)
 (include-book "../lists/list-defuns")
-(local (include-book "../lists/sets"))
 (local (include-book "../lists/take"))
-(local (include-book "../lists/repeat"))
-(local (include-book "../lists/nthcdr"))
 
 (local (in-theory (enable strip-cars)))
 
+(local (defthm take-of-nil
+         (equal (take n nil)
+                (replicate n nil))
+         :hints(("Goal" :in-theory (enable replicate)))))
+
+(local (defthm nthcdr-of-nil
+         (equal (nthcdr n nil)
+                nil)))
+
 (defsection std/alists/strip-cars
   :parents (std/alists strip-cars)
   :short "Lemmas about @(see strip-cars) available in the @(see std/alists)
@@ -114,21 +131,23 @@ for some reason that's what you want to do.</p>"
 
   (defthm strip-cars-of-rev
     (equal (strip-cars (rev x))
-           (rev (strip-cars x))))
+           (rev (strip-cars x)))
+    :hints(("Goal" :in-theory (enable rev))))
 
   (defthm strip-cars-of-revappend
     (equal (strip-cars (revappend x y))
            (revappend (strip-cars x)
                       (strip-cars y))))
 
-  (defthm strip-cars-of-repeat
-    (equal (strip-cars (repeat x n))
-           (repeat (car x) n))
-    :hints(("Goal" :in-theory (enable repeat))))
+  (defthm strip-cars-of-replicate
+    (equal (strip-cars (replicate n x))
+           (replicate n (car x)))
+    :hints(("Goal" :in-theory (enable replicate))))
 
   (defthm strip-cars-of-take
     (equal (strip-cars (take n x))
-           (take n (strip-cars x))))
+           (take n (strip-cars x)))
+    :hints(("Goal" :in-theory (enable replicate))))
 
   (defthm strip-cars-of-nthcdr
     (equal (strip-cars (nthcdr n x))
