@@ -6,15 +6,25 @@
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 ;
 ; Original author: Jared Davis <jared@centtech.com>
 
@@ -217,9 +227,9 @@ bitset-members).</p>"
     (local (defthm l0
              (implies (and (natp a)
                            (natp b))
-                      (equal (sets::<< a b)
+                      (equal (set::<< a b)
                              (< a b)))
-             :hints(("Goal" :in-theory (enable sets::<< lexorder alphorder)))))
+             :hints(("Goal" :in-theory (enable set::<< lexorder alphorder)))))
 
     (local (defthm l1
              (equal (natp (car (bits-between n m x)))
@@ -233,20 +243,20 @@ bitset-members).</p>"
              :rule-classes ((:linear))))
 
     (defthm setp-of-bits-between
-      (sets::setp (bits-between n m x))
-      :hints(("Goal" :in-theory (enable* (:ruleset sets::primitive-rules))))))
+      (set::setp (bits-between n m x))
+      :hints(("Goal" :in-theory (enable* (:ruleset set::primitive-rules))))))
 
   (encapsulate
     ()
     (local (defthmd sets-membership-hack
-             (implies (and (sets::setp x)
+             (implies (and (set::setp x)
                            (syntaxp (not (quotep x))))
                       (iff (member-equal a x)
-                           (sets::in a x)))
-             :hints(("Goal" :in-theory (enable sets::in-to-member)))))
+                           (set::in a x)))
+             :hints(("Goal" :in-theory (enable set::in-to-member)))))
 
     (defthm in-of-bits-between
-      (equal (sets::in a (bits-between n m x))
+      (equal (set::in a (bits-between n m x))
              (and (natp a)
                   (<= (nfix n) a)
                   (< a (nfix m))
@@ -279,7 +289,7 @@ bitset-members).</p>"
            ; integerp-minus-x
            append
            not
-           sets::double-containment
+           set::double-containment
            ; normalize-terms-such-as-a/a+b-+-b/a+b
            ; normalize-addends
            acl2::default-+-1
