@@ -1,15 +1,34 @@
 ; Rev function and lemmas
-; Copyright (C) 2005-2013 by Jared Davis <jared@cs.utexas.edu>
+; Copyright (C) 2005-2013 Kookamara LLC
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+; Contact:
+;
+;   Kookamara LLC
+;   11410 Windermere Meadows
+;   Austin, TX 78759, USA
+;   http://www.kookamara.com/
+;
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
+;
+; Original author: Jared Davis <jared@kookamara.com>
 ;
 ; rev.lisp
 ; This file was originally part of the Unicode library.
@@ -18,6 +37,7 @@
 (include-book "list-fix")
 (local (include-book "revappend"))
 (local (include-book "append"))
+(include-book "equiv")
 
 (defun revappend-without-guard (x y)
   (declare (xargs :guard t))
@@ -150,6 +170,22 @@ perform quite well thanks to @(see mbe).</p>"
       ;; rev just to get lemmas about make-character-list.
       (equal (make-character-list (rev x))
              (rev (make-character-list x)))
-      :hints(("Goal" :in-theory (enable make-character-list))))))
+      :hints(("Goal" :in-theory (enable make-character-list)))))
 
 
+  (defthm list-equiv-of-rev-and-rev
+    (equal (list-equiv (rev x) (rev y))
+           (list-equiv x y))
+    :hints(("Goal" :in-theory (enable list-equiv))))
+
+  (def-listp-rule element-list-p-of-rev
+    (implies (element-list-p x)
+             (element-list-p (rev x))))
+
+  (def-listfix-rule element-list-fix-of-rev
+    (equal (element-list-fix (rev x))
+           (rev (element-list-fix x))))
+
+  (def-projection-rule elementlist-projection-of-rev
+    (equal (elementlist-projection (rev x))
+           (rev (elementlist-projection x)))))
