@@ -1,39 +1,49 @@
-; CUTIL - Centaur Basic Utilities
-; Copyright (C) 2008-2011 Centaur Technology
+; Standard Utilities Library
+; Copyright (C) 2008-2014 Centaur Technology
 ;
 ; Contact:
 ;   Centaur Technology Formal Verification Group
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 ;
 ; Original author: Jared Davis <jared@centtech.com>
 ;
-; Additional copyright notice:
+; Additional Copyright Notice.
 ;
-; This file is adapted from Milawa, which is also released under the GPL.
+; This file is adapted from the Milawa Theorem Prover, Copyright (C) 2005-2009
+; Kookamara LLC, which is also available under an MIT/X11 style license.
 
 (in-package "STD")
 (include-book "da-base")
 (include-book "formals")
-(include-book "xdoc/fmt-to-str" :dir :system)
 (include-book "tools/rulesets" :dir :system)
 (include-book "xdoc/names" :dir :system)
-(include-book "str/cat" :dir :system)
+(include-book "xdoc/fmt-to-str-orig" :dir :system)
 (set-state-ok t)
 
 (program)
 
-(def-ruleset tag-reasoning nil)
+(def-ruleset! tag-reasoning nil)
 
 (defxdoc defaggregate
   :parents (std/util)
@@ -86,16 +96,13 @@ have.  The example above shows only the very simplest syntax, but fields can
 also have well-formedness requirements, documentation, etc.; see below.</p>
 
 <p>The @('Option')s control various settings on the structure, and many options
-are described below.</p>
+are described below.  Options can actually come before or after the fields (or
+both).</p>
 
 <p>The @('other-events') is just a place for arbitrary other events to be put,
 as in @(see define).  This is mainly intended as a book structuring device, to
 allow you to keep related theorems near your aggregate.</p>
 
-<p>Note that the general form is not really quite expressive enough.  We
-actually allow the options and fields to come in any order.  That is, you can
-put the :tag and other options before the fields, etc.</p>
-
 
 <h3>Structure Tags</h3>
 
@@ -166,17 +173,22 @@ predicate that the field must satisfy.  Here is an example:</p>
 
 @({
  (defaggregate employee
+
    ((name   \"Should be in Lastname, Firstname format\"
             stringp :rule-classes :type-prescription)
+
     (salary \"Annual salary in dollars, nil for hourly employees\"
             (or (not salary) (natp salary))
             :rule-classes :type-prescription)
+
     (rank \"Employee rank.  Can be empty.\"
           (implies rank (and (characterp rank)
                              (alpha-char-p rank))))
+
     (position (and (position-p position)
                    (salary-in-range-for-position-p salary position))
               :default :peon))
+
    :tag :employee)
 })
 
@@ -276,9 +288,10 @@ its fields.  The very advanced user can then inspect these fields after
 submitting the aggregate, and perhaps use them to generate additional
 events.</dd>
 
-<dt>:require</dt>
+<dt>:verbosep</dt>
 
-<dd>This option is deprecated.  Please use the new fields syntax, instead.</dd>
+<dd>You can use @(':verbosep t') to turn off output hiding.  This option is
+generally meant for debugging failures in defaggregate.</dd>
 
 <dt>:rest</dt>
 
@@ -287,6 +300,82 @@ events.</dd>
 </dl>
 
 
+<h3>Dependent Requirements</h3>
+
+<p>The embedded \"guard\" in each extended formal allows you to naturally
+express simple requirements, e.g., @('arity') should be a natural and @('args')
+should be an @('true-listp').  But what if you need something more like <a
+href='https://en.wikipedia.org/wiki/Dependent_type'>dependent types</a>, e.g.,
+say that @('arity') is supposed to always match the length of args?</p>
+
+<p>It's valid to refer to other fields within the guards of an embedded formal,
+so one way we could write this would be, e.g.,:</p>
+
+@({
+    (defaggregate mytype
+       ((arity natp :rule-classes :type-prescription)
+        (args  (and (true-listp args)
+                    (equal (len args) arity)))))
+})
+
+<p>This is perfectly valid, but you may sometimes prefer not to embed these
+dependent requirements directly in the fields.  For instance, in the example
+above, the result-type theorem about @('args') becomes two @(see acl2::rewrite)
+rules.  It would probably be better for the @('true-listp') part to be a @(see
+acl2::type-prescription) rule.  But the @('len') requirement doesn't make sense
+as a @(':type-prescription').</p>
+
+<p>To work around this, you could use an explicit, compound @(':rule-classes')
+form with separate @(':corollary') theorems.  This gets very ugly, because you
+have to write out each corollary in full, e.g.,:</p>
+
+@({
+    (defaggregate mytype
+      ((arity natp :rule-classes :type-prescription)
+       (args  (and (true-listp args)
+                   (equal (len args) arity))
+              :rule-classes
+              ((:type-prescription :corollary
+                                   (implies (force (mytype-p x))
+                                            (true-listp (mytype->args x))))
+               (:rewrite :corollary
+                         (implies (force (mytype-p x))
+                                  (equal (len (mytype->args x))
+                                         (mytype->arity x))))))))
+})
+
+<p>So you may instead prefer to use the alternate @(':require') syntax.
+In this case, we would have:</p>
+
+@({
+     (defaggregate mytype
+       ((arity natp :rule-classes :type-prescription)
+        (args  true-listp :rule-classes :type-prescription))
+       :require
+       ((len-of-mytype->args (equal (len args) arity))))
+})
+
+<p>This would result in an ordinary @(see acl2::type-prescription) return-type
+theorems for both @('arity') and @('args'), and a separate rewrite rule to deal
+with the length dependency:</p>
+
+@({
+     (defthm len-of-mytype->args
+       (implies (force (mytype-p x))
+                (equal (len (mytype->args x))
+                       (mytype->arity x))))
+})
+
+<p>The general form of each @(':require') form is:</p>
+
+@({   (theorem-name conclusion [:rule-classes ...])  })
+
+<p>Where @('conclusion') may mention any of the fields of the aggregate.  Each
+requirement becomes a guard for the constructor, a well-formedness requirement
+in the recognizer, and a theorem about the accessors of your structure, exactly
+like the simple requirements on each field.</p>
+
+
 <h3>Make and Change Macros</h3>
 
 <p>Direct use of the constructor is discouraged.  Instead, we introduce two
@@ -322,7 +411,7 @@ fields</li>
 <li>The person reading the code can see what values are being given to which
 fields.</li>
 
-<li>Fields whose value should be @('nil') (or some other @(':default') may be
+<li>Fields whose value should be @('nil') (or some other @(':default')) may be
 omitted from the <i>make</i> macro.</li>
 
 <li>Fields whose value should be left alone can be omitted from the
@@ -465,7 +554,11 @@ optimization altogether.</p>")
                (and (= (len x) 4)
                     (equal (third x) :rule-classes)))
            (consp (second x))
-           (pseudo-termp (second x)))
+           ;; [Sol] Removed this.  It would be nice to check that this is a
+           ;; well-formed "term", but it won't be a pseudo-term if e.g. it
+           ;; contains something like an implicitly quoted constant, e.g., 1.
+           ;; (pseudo-termp (second x))
+           )
       (er hard? 'da-require-p "Ill-formed requirement: ~x0.~%" x)))
 
 (defun da-requirelist-p (x)
@@ -607,12 +700,6 @@ optimization altogether.</p>")
             (da-fields-recognizer-map basename (cdr fields)))
     nil))
 
-(defun da-accessor-names (basename fields)
-  (if (consp fields)
-      (cons (da-accessor-name basename (car fields))
-            (da-accessor-names basename (cdr fields)))
-    nil))
-
 (defun da-make-requirement-of-recognizer (name require map accnames)
   (let ((rule-classes (if (eq (third require) :rule-classes)
                           (fourth require)
@@ -644,7 +731,7 @@ optimization altogether.</p>")
   (b* (((formal x) x)
 
        (acc (str::revappend-chars "<li>" acc))
-       ((mv name-str state) (xdoc::fmt-to-str x.name base-pkg state))
+       ((mv name-str state) (xdoc::fmt-to-str-orig x.name base-pkg state))
        (acc (str::revappend-chars "<tt>" acc))
        (acc (xdoc::simple-html-encode-str name-str 0 (length name-str) acc))
        (acc (str::revappend-chars "</tt>" acc))
@@ -675,7 +762,7 @@ optimization altogether.</p>")
                 acc
               (str::revappend-chars "<br/>&nbsp;&nbsp;&nbsp;&nbsp;" acc)))
        (acc (str::revappend-chars "<color rgb='#606060'>" acc))
-       ((mv guard-str state) (xdoc::fmt-to-str x.guard base-pkg state))
+       ((mv guard-str state) (xdoc::fmt-to-str-orig x.guard base-pkg state))
        ;; Using @('...') here isn't necessarily correct.  If the sexpr has
        ;; something in it that can lead to '), we are hosed.  BOZO eventually
        ;; check for this and make sure we use <code> tags instead, if it
@@ -714,7 +801,7 @@ optimization altogether.</p>")
        (acc  (str::revappend-chars ") is a @(see std::defaggregate) of the following fields.</p>" acc))
        ((mv acc state) (da-fields-doc fields acc base-pkg state))
        (acc  (str::revappend-chars "<p>Source link: @(srclink " acc))
-       (acc  (str::revappend-chars (string-downcase (symbol-name name)) acc))
+       (acc  (str::revappend-chars (acl2::string-downcase (symbol-name name)) acc))
        (acc  (str::revappend-chars ")</p>" acc))
        (acc  (str::revappend-chars (or long "") acc))
        (long (str::rchars-to-string acc)))
@@ -730,9 +817,8 @@ optimization altogether.</p>")
        (foop     (da-recognizer-name name))
        (accessor (da-accessor-name name field.name))
        ;; bozo escaping issues...
-       (short    (str::cat "Access the <tt>" (string-downcase (symbol-name field.name))
-                           "</tt> field of a @(see "
-                           (symbol-package-name foop) "::" (symbol-name foop)
+       (short    (str::cat "Access the <tt>" (acl2::string-downcase (symbol-name field.name))
+                           "</tt> field of a @(see " (xdoc::full-escape-symbol foop)
                            ") structure.")))
     `(defxdoc ,accessor
        :parents (,foop)
@@ -770,7 +856,8 @@ optimization altogether.</p>")
   (b* ((ctor-name (xdoc::name-low (symbol-name name)))
        ;; +2 to account for the leading paren and trailing space after ctor-name 
        (len       (+ 2 (length ctor-name)))
-       (pad       (str::implode (cons #\Newline (repeat #\Space len))))
+       (pad       (str::implode (cons #\Newline
+                                      (make-list len :initial-element #\Space))))
        (args      (str::join field-strs pad)))
     (str::cat "<code>" *nl* "(" ctor-name " " args ")" *nl* "</code>")))
 
@@ -820,14 +907,14 @@ optimization altogether.</p>")
        (call-make-honsed-foo (da-ctor-optional-call make-honsed-foo opt-fields))
        (call-change-foo      (da-ctor-optional-call change-foo (cons "x" opt-fields)))
 
-       (def-foo           (str::cat "@(def " pkg "::" (symbol-name foo) ")"))
-       (def-honsed-foo    (str::cat "@(def " pkg "::" (symbol-name honsed-foo) ")"))
-       (def-make-foo-fn   (str::cat "@(def " pkg "::" (symbol-name make-foo-fn) ")"))
-       (def-make-foo      (str::cat "@(def " pkg "::" (symbol-name make-foo) ")"))
-       (def-make-honsed-foo-fn (str::cat "@(def " pkg "::" (symbol-name make-honsed-foo-fn) ")"))
-       (def-make-honsed-foo    (str::cat "@(def " pkg "::" (symbol-name make-honsed-foo) ")"))
-       (def-change-foo-fn (str::cat "@(def " pkg "::" (symbol-name change-foo-fn) ")"))
-       (def-change-foo    (str::cat "@(def " pkg "::" (symbol-name change-foo) ")")))
+       (def-foo                (str::cat "@(def " (xdoc::full-escape-symbol foo) ")"))
+       (def-honsed-foo         (str::cat "@(def " (xdoc::full-escape-symbol honsed-foo) ")"))
+       (def-make-foo-fn        (str::cat "@(def " (xdoc::full-escape-symbol make-foo-fn) ")"))
+       (def-make-foo           (str::cat "@(def " (xdoc::full-escape-symbol make-foo) ")"))
+       (def-make-honsed-foo-fn (str::cat "@(def " (xdoc::full-escape-symbol make-honsed-foo-fn) ")"))
+       (def-make-honsed-foo    (str::cat "@(def " (xdoc::full-escape-symbol make-honsed-foo) ")"))
+       (def-change-foo-fn      (str::cat "@(def " (xdoc::full-escape-symbol change-foo-fn) ")"))
+       (def-change-foo         (str::cat "@(def " (xdoc::full-escape-symbol change-foo) ")")))
 
     (list
      `(defxdoc ,foo
@@ -976,6 +1063,7 @@ optimization altogether.</p>")
     :long
     :already-definedp
     :extra-field-keywords
+    :verbosep
     ;; deprecated options
     :require
     :rest))
@@ -1013,7 +1101,8 @@ optimization altogether.</p>")
         (mv (raise "~x0: Too many field specifiers: ~x1" name field-specs) state))
        (efields     (parse-formals ctx (car field-specs)
                                    (append '(:rule-classes :default)
-                                           extra-field-keywords)))
+                                           extra-field-keywords)
+                                   (w state)))
        (field-names (formallist->names efields))
        (field-defaults (formallist->defaults efields))
        ((unless (no-duplicatesp field-names))
@@ -1066,6 +1155,10 @@ optimization altogether.</p>")
        ((unless (booleanp honsp))
         (mv (raise "~x0: :hons should be a boolean." name) state))
 
+       (verbosep (cdr (assoc :verbosep kwd-alist)))
+       ((unless (booleanp verbosep))
+        (mv (raise "~x0: :verbosep should be a boolean." name) state))
+
        ;; Expand requirements to include stuff from the field specifiers.
        (old-reqs   (cdr (assoc :require kwd-alist)))
        (field-reqs (da-formals-to-requires name efields))
@@ -1245,9 +1338,11 @@ optimization altogether.</p>")
            (value-triple '(defaggregate ,name)))))
     (mv `(with-output
            :stack :push
-           :gag-mode t
-           :off (acl2::summary acl2::observation acl2::prove acl2::proof-tree
-                               acl2::event)
+           ,@(if verbosep
+                 nil
+               '(:gag-mode t
+                 :off (acl2::summary acl2::observation acl2::prove acl2::proof-tree
+                                     acl2::event)))
            ,event)
         state)))
 
