@@ -1,25 +1,36 @@
-; CUTIL - Centaur Basic Utilities
-; Copyright (C) 2008-2011 Centaur Technology
+; Standard Utilities Library
+; Copyright (C) 2008-2014 Centaur Technology
 ;
 ; Contact:
 ;   Centaur Technology Formal Verification Group
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 ;
 ; Original author: Jared Davis <jared@centtech.com>
 
 (in-package "STD")
 (include-book "deflist")
+(include-book "std/strings/defs-program" :dir :system)
 (include-book "misc/hons-help" :dir :system)
 
 (defxdoc defalist
@@ -70,6 +81,7 @@ traditional functions with no such requirement, e.g.,:</p>
          valp-of-nil       ; :unknown by default
          true-listp        ; nil by default
          mode              ; current defun-mode by default
+         already-definedp  ; nil by default
          parents           ; nil by default
          short             ; nil by default
          long              ; nil by default
@@ -121,6 +133,11 @@ used when @('(key-recognizer nil ...)') (similarly @('(val-recognzier nil
 ...)')) is always known to be @('t') or @('nil').  When it is provided,
 @('defalist') can generate slightly better theorems.</p>
 
+<p>The optional @(':already-definedp') keyword can be set if you have already
+defined the function.  This can be used to generate all of the ordinary
+@('defalist') theorems without generating a @('defund') event, and is useful
+when you are dealing with mutually recursive recognizers.</p>
+
 <p>The optional @(':mode') keyword can be set to @(':program') to introduce the
 recognizer in program mode.  In this case, no theorems are introduced.</p>
 
@@ -134,6 +151,7 @@ to override it.</p>")
 (defun defalist-fn (name formals key val
                          guard verify-guards
                          keyp-of-nil valp-of-nil
+                         already-definedp
                          mode parents short long true-listp)
   (declare (xargs :mode :program))
   (b* (((unless (symbolp name))
@@ -198,48 +216,53 @@ to override it.</p>")
         (er hard 'defalist
             ":mode must be one of :logic or :program, but is ~x0." mode))
 
+       ((unless (or (eq mode :logic)
+                    (not already-definedp)))
+        (er hard 'defalist
+            ":mode :program and already-definedp cannot be used together."))
+
        (short (or short
                   (and parents
-                       (str::cat "@(call " (symbol-package-name
+                       (concatenate 'string "@(call " (symbol-package-name
 name) "::" (symbol-name name) ") recognizes association lists where every key
-satisfies @(see " (symbol-package-name keyp) "::" (symbol-name keyp) ") and
-each value satisfies @(see " (symbol-package-name valp) "::" (symbol-name
-valp) ")."))))
+satisfies @(see? " (xdoc::full-escape-symbol keyp) ") and
+each value satisfies @(see? " (xdoc::full-escape-symbol valp) ")."))))
 
        (long (or long
                  (and parents
-                      (str::cat "<p>This is an ordinary @(see std::defalist).</p>"
-                                "@(def " (symbol-package-name
-                                name) "::" (symbol-name name) ")"))))
+                      (concatenate 'string "<p>This is an ordinary @(see std::defalist).</p>"
+                                   "@(def " (xdoc::full-escape-symbol name) ")"))))
 
        (doc (if (or parents short long)
                 `((defxdoc ,name :parents ,parents :short ,short :long ,long))
               nil))
 
-       (def `(defund ,name (,@formals)
-               (declare (xargs :guard ,guard
-                               :verify-guards ,verify-guards
-                               :mode ,mode
-                               ;; We tell ACL2 not to normalize because
-                               ;; otherwise type reasoning can rewrite the
-                               ;; definition, and ruin some of our theorems
-                               ;; below, e.g., when KEYP is known to always be
-                               ;; true.
-                               :normalize nil
-                               ))
-               (if (consp ,x)
-                   (and (consp (car ,x))
-                        (,keyp ,@(subst `(caar ,x) x key-formals))
-                        (,valp ,@(subst `(cdar ,x) x val-formals))
-                        (,name ,@(subst `(cdr ,x) x formals)))
-                 ,(if true-listp
+       (def (if already-definedp
+                nil
+              `((defund ,name (,@formals)
+                  (declare (xargs :guard ,guard
+                                  :verify-guards ,verify-guards
+                                  :mode ,mode
+                                  ;; We tell ACL2 not to normalize because
+                                  ;; otherwise type reasoning can rewrite the
+                                  ;; definition, and ruin some of our theorems
+                                  ;; below, e.g., when KEYP is known to always be
+                                  ;; true.
+                                  :normalize nil
+                                  ))
+                  (if (consp ,x)
+                      (and (consp (car ,x))
+                           (,keyp ,@(subst `(caar ,x) x key-formals))
+                           (,valp ,@(subst `(cdar ,x) x val-formals))
+                           (,name ,@(subst `(cdr ,x) x formals)))
+                    ,(if true-listp
                          `(null ,x)
-                       t))))
+                       t))))))
 
        ((when (eq mode :program))
         `(progn
            ,@doc
-           ,def)))
+           ,@def)))
 
     `(encapsulate
        ()
@@ -320,7 +343,7 @@ valp) ")."))))
                                                name '-val-lemma))))))
           ))
 
-       ,def
+       ,@def
 
        (local (in-theory (theory 'minimal-theory)))
        (local (in-theory (disable (:executable-counterpart tau-system))))
@@ -457,7 +480,9 @@ valp) ")."))))
          :hints(("Goal"
                  :induct (len ,x)
                  :in-theory (enable subsetp-equal)
-                 :expand ((,name ,@formals)
+                 :expand (;; No, don't expand name directly, it doesn't work if we're
+                          ;; doing an already-definedp defalist that has a different
+                          ;; definition. (,name ,@formals)
                           (true-listp ,x)))))
 
        ,@(and (not true-listp)
@@ -522,7 +547,9 @@ valp) ")."))))
                                 (t ;; keyp-of-nil is :unknown
                                  `(or (consp ,x)
                                       (,valp ,@(subst nil x val-formals)))))))
-         :hints(("Goal" :expand (,name ,@formals))))
+         ;; No, don't expand, it doesn't work in the already-definedp case.
+         ;; :hints(("Goal" :expand (,name ,@formals)))
+         )
 
        (defthm ,(mksym name '-of-cdr-when- name)
          (implies (,name ,@formals)
@@ -544,15 +571,15 @@ valp) ")."))))
                  :in-theory (enable acl2::take-redefinition)
                  :induct (take ,n ,x))))
 
-       (defthm ,(mksym name '-of-repeat)
-         (equal (,name ,@(subst `(repeat ,a ,n) x formals))
+       (defthm ,(mksym name '-of-replicate)
+         (equal (,name ,@(subst `(replicate ,n ,a) x formals))
                 (or (and (consp ,a)
                          (,keyp ,@(subst `(car ,a) x key-formals))
                          (,valp ,@(subst `(cdr ,a) x val-formals)))
                     (zp ,n)))
          :hints(("Goal"
-                 :induct (repeat ,a ,n)
-                 :in-theory (enable repeat))))
+                 :induct (replicate ,n ,a)
+                 :in-theory (enable replicate))))
 
        (defthm ,(mksym name '-of-last)
          (implies (,name ,@(subst `(double-rewrite ,x) x formals))
@@ -591,7 +618,7 @@ valp) ")."))))
          :hints(("Goal" :do-not-induct t)))
 
        (defthm ,(mksym name '-of-sfix)
-         ;; This rule is important for sets::under-set-equiv rules to work
+         ;; This rule is important for set::under-set-equiv rules to work
          ;; right in the context of a foo-listp.
          (implies (,name ,@(subst `(double-rewrite ,x) x formals))
                   (equal (,name ,@(subst `(sfix ,x) x formals))
@@ -655,7 +682,7 @@ valp) ")."))))
                             (,name ,@(subst `(union ,x ,y) x formals)))
                    :hints(("Goal"
                            :do-not-induct t
-                           :in-theory (disable sets::union-under-set-equiv
+                           :in-theory (disable set::union-under-set-equiv
                                                deflist-lemma-subsetp-of-union)
                            :use ((:instance deflist-lemma-subsetp-of-union
                                             (x ,x)
@@ -725,6 +752,7 @@ valp) ")."))))
                          (verify-guards 't)
                          (keyp-of-nil ':unknown)
                          (valp-of-nil ':unknown)
+                         already-definedp
                          mode
                          (parents 'nil parents-p)
                          (short 'nil)
@@ -738,6 +766,7 @@ valp) ")."))))
                  (defalist-fn ',name ',formals ',key ',val
                    ',guard ',verify-guards
                    ',keyp-of-nil ',valp-of-nil
+                   ',already-definedp
                    mode
                    parents ',short ',long ',true-listp))))
 
