@@ -1,31 +1,41 @@
-; CUTIL - Centaur Basic Utilities
-; Copyright (C) 2008-2012 Centaur Technology
+; Standard Utilities Library
+; Copyright (C) 2008-2014 Centaur Technology
 ;
 ; Contact:
 ;   Centaur Technology Formal Verification Group
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 ;
 ; Original author: Jared Davis <jared@centtech.com>
 ;
-; Additional copyright notice:
+; Additional Copyright Notice.
 ;
-; This file is adapted from Milawa, which is also released under the GPL.
+; This file is adapted from the Milawa Theorem Prover, Copyright (C) 2005-2009
+; Kookamara LLC, which is also available under an MIT/X11 style license.
 
 (in-package "STD")
 (include-book "xdoc/top" :dir :system)
 (include-book "tools/bstar" :dir :system)
-(include-book "str/cat" :dir :system)
 (include-book "std/osets/top" :dir :system)
 (include-book "defsort/duplicated-members" :dir :system)
 (include-book "std/lists/sets" :dir :system)
@@ -64,7 +74,6 @@ generates basic, automatic @(see xdoc) documentation.</p>
    :guard-hints             nil
    :guard-debug             nil
    :mode                    current defun-mode
-   :already-definedp        nil
    :verbosep                nil
    :parents                 nil
    :short                   nil
@@ -131,11 +140,6 @@ behavior or another; see @(see strict-list-recognizers) for details.</p>
 @(':guard-hints') are options for the @(see defun) we introduce.  These are for
 the guards of the new list recognizer, not the element recognizer.</p>
 
-<p>The optional @(':already-definedp') keyword can be set if you have already
-defined the function.  This can be used to generate all of the ordinary
-@('deflist') theorems without generating a @('defund') event, and is useful
-when you are dealing with mutually recursive recognizers.</p>
-
 <p>The optional @(':mode') keyword can be set to @(':logic') or @(':program')
 to introduce the recognizer in logic or program mode.  The default is whatever
 the current default defun-mode is for ACL2, i.e., if you are already in program
@@ -172,7 +176,15 @@ recognizer is enabled during the @('///') section.  Here is an example:</p>
 <p>Deprecated.  The optional @(':rest') keyword was a precursor to @('///').
 It is still implemented, but its use is now discouraged.  If both @(':rest')
 and @('///') events are used, we arbitrarily put the @(':rest') events
-first.</p>")
+first.</p>
+
+<p>Deprecated.  The optional @(':already-definedp') keyword can be set if you
+have already defined the function.  This was previously useful when you wanted
+to generate the ordinary @('deflist') theorems without generating a @('defund')
+event, e.g., because you are dealing with mutually recursive recognizers.  We
+still accept this option for backwards compatibility but it is useless, because
+@('deflist') is now smart enough to notice that the function is already
+defined.</p>")
 
 (defxdoc strict-list-recognizers
   :parents (deflist)
@@ -291,6 +303,7 @@ of which recognizers require true-listp and which don't.</p>")
   ;; lemmas that we enable so that it will work.
 
   (local (include-book "std/osets/under-set-equiv" :dir :system))
+  (local (in-theory (acl2::enable* set::definitions set::expensive-rules)))
 
   (defthmd deflist-lemma-member-of-car
     (iff (member-equal (car x) x)
@@ -396,9 +409,9 @@ of which recognizers require true-listp and which don't.</p>")
                     (member a x))
            :hints(("Goal"
                    :do-not-induct t
-                   :use ((:instance sets::in-to-member
-                                    (sets::a a)
-                                    (sets::x (sfix x))))))))
+                   :use ((:instance set::in-to-member
+                                    (set::a a)
+                                    (set::x (sfix x))))))))
 
   (defthmd deflist-lemma-subsetp-of-intersect
     (and (subsetp-equal (intersect x y) x)
@@ -463,16 +476,16 @@ of which recognizers require true-listp and which don't.</p>")
     acl2::list-fix-when-not-consp
     acl2::list-fix-when-true-listp
     acl2::list-fix-of-cons
-    sets::sets-are-true-lists
-    sets::mergesort-set
-    sets::union-set
-    sets::intersect-set
-    sets::difference-set
+    set::sets-are-true-lists
+    set::mergesort-set
+    set::union-set
+    set::intersect-set
+    set::difference-set
     acl2::set-equiv-implies-equal-subsetp-1
     acl2::set-equiv-implies-equal-subsetp-2
     acl2::subsetp-refl
     acl2::list-fix-under-list-equiv
-    sets::mergesort-under-set-equiv
+    set::mergesort-under-set-equiv
     acl2::binary-append-without-guard))
 
 
@@ -490,7 +503,10 @@ of which recognizers require true-listp and which don't.</p>")
     :long
     :true-listp
     :rest
-    :verbosep))
+    :verbosep
+    ;; Undocumented option for customizing the theory, mainly meant for
+    ;; problematic cases, e.g., built-in functions where ACL2 "knows too much"
+    :theory-hack))
 
 (defun deflist-fn (name formals element kwd-alist other-events state)
   (declare (xargs :mode :program))
@@ -522,16 +538,38 @@ of which recognizers require true-listp and which don't.</p>")
        (elementp     (car element))
        (elem-formals (cdr element))
 
+       ;; We previously required the user to tell us if the function was
+       ;; already defined.  Now we---you know---actually look to see.  Duh.
+       (looks-already-defined-p
+        (or (not (eq (getprop name 'acl2::formals :none 'acl2::current-acl2-world
+                              (w state))
+                     :none))
+            (not (eq (getprop name 'acl2::macro-args :none 'acl2::current-acl2-world
+                              (w state))
+                     :none))))
+       (already-definedp (getarg :already-definedp :unknown kwd-alist))
+       ((unless (or (eq already-definedp :unknown)
+                    (eq already-definedp looks-already-defined-p)))
+        (raise "Found :already-definedp ~x0, but ~x1 is ~s2."
+               already-definedp name
+               (if looks-already-defined-p
+                   "already defined."
+                 "not defined.")))
+       (already-definedp looks-already-defined-p)
+
        (negatedp         (getarg :negatedp         nil      kwd-alist))
        (true-listp       (getarg :true-listp       nil      kwd-alist))
        (verify-guards    (getarg :verify-guards    t        kwd-alist))
        (guard            (getarg :guard            t        kwd-alist))
        (guard-debug      (getarg :guard-debug      nil      kwd-alist))
        (guard-hints      (getarg :guard-hints      nil      kwd-alist))
-       (already-definedp (getarg :already-definedp nil      kwd-alist))
+
        (elementp-of-nil  (getarg :elementp-of-nil  :unknown kwd-alist))
        (short            (getarg :short            nil      kwd-alist))
        (long             (getarg :long             nil      kwd-alist))
+       (theory-hack      (getarg :theory-hack      nil      kwd-alist))
+
+
 
        (rest             (append
                           (getarg :rest nil kwd-alist)
@@ -565,14 +603,15 @@ of which recognizers require true-listp and which don't.</p>")
 
        (short (or short
                   (and parents
-                       (str::cat "@(call " (symbol-name name)
+                       (concatenate
+                        'string "@(call " (xdoc::full-escape-symbol name)
                                  ") recognizes lists where every element "
                                  (if negatedp
                                      "is rejected by "
                                    "satisfies ")
                                  ;; bozo it'd be better to put the formals in
                                  ;; here, for multi-arity functions.
-                                 "@(see " (symbol-name elementp) ")."))))
+                                 "@(see? " (xdoc::full-escape-symbol elementp) ")."))))
 
        (long (or long
                  (and parents
@@ -660,6 +699,7 @@ of which recognizers require true-listp and which don't.</p>")
                                     deflist-local-booleanp-element-thm
                                     )))
           (local (enable-if-theorem deflist-local-elementp-of-nil-thm))
+          ,@theory-hack
 
           (defthm ,(mksym name '-when-not-consp)
             (implies (not (consp ,x))
@@ -959,16 +999,16 @@ of which recognizers require true-listp and which don't.</p>")
                              (:free (,x ,y)
                                     (,name ,@(subst `(cons ,x ,y) x formals)))))))
 
-          (defthm ,(mksym name '-of-repeat)
-            (equal (,name ,@(subst `(repeat ,x ,n) x formals))
+          (defthm ,(mksym name '-of-replicate)
+            (equal (,name ,@(subst `(replicate ,n ,x) x formals))
                    (or ,(cond (negatedp
                                `(not (,elementp ,@formals)))
                               (t
                                `(,elementp ,@formals)))
                        (zp ,n)))
             :hints(("Goal"
-                    :induct (repeat ,x ,n)
-                    :in-theory (enable repeat deflist-local-booleanp-element-thm)
+                    :induct (replicate ,n ,x)
+                    :in-theory (enable replicate deflist-local-booleanp-element-thm)
                     :expand ((,name ,@formals)
                              (:free (,x ,y)
                                     (,name ,@(subst `(cons ,x ,y) x formals)))))))
@@ -1023,7 +1063,7 @@ of which recognizers require true-listp and which don't.</p>")
             :hints(("Goal" :do-not-induct t)))
 
           (defthm ,(mksym name '-of-sfix)
-            ;; This rule is important for sets::under-set-equiv rules to work
+            ;; This rule is important for set::under-set-equiv rules to work
             ;; right in the context of a foo-listp.
             (implies (,name ,@(subst `(double-rewrite ,x) x formals))
                      (equal (,name ,@(subst `(sfix ,x) x formals))
@@ -1089,7 +1129,7 @@ of which recognizers require true-listp and which don't.</p>")
                                (,name ,@(subst `(union ,x ,y) x formals)))
                       :hints(("Goal"
                               :do-not-induct t
-                              :in-theory (disable sets::union-under-set-equiv
+                              :in-theory (disable set::union-under-set-equiv
                                                   deflist-lemma-subsetp-of-union)
                               :use ((:instance deflist-lemma-subsetp-of-union
                                                (x ,x)
