@@ -6,15 +6,25 @@
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 ;
 ; Original authors: Sol Swords and Jared Davis
 ;                   {sswords,jared}@centtech.com
@@ -91,6 +101,9 @@ precise, it is @('pkg::flag') where @('pkg') is the package of the new flag
 function's name; usually this means you don't have to think about the
 package.</li>
 
+<li>@(':ruler-extenders') lets you give a value for the @(see
+acl2::ruler-extenders) of the new flag function.</li>
+
 </ul>
 
 
@@ -234,6 +247,10 @@ macro and simply leave out the trivial parts:</p>
    :skip-others t)
 })
 
+<p>You may also have more than one defthm form for a given flag.  For the main
+inductive proof, these are all simply conjoined together (and their hints are
+simply appended together), but they are exported as separate theorems and may
+have different rule-classes.</p>
 
 <p>There is an older, alternate syntax for @('make-flag') that is still
 available.  To encourage transitioning to the new syntax, the old syntax is not
@@ -277,7 +294,12 @@ effect.  We simply translate subgoal hints to computed hints:</p>
    --->
  (and (equal id (parse-clause-id \"Subgoal *1/5.2\"))
       '(:in-theory (theory 'foo)))
-})")
+})
+
+<p>As mentioned above, if there is more than one defthm form for a given flag,
+the hints for all such forms are simply appended together; the hints given to
+one such form may affect what you might think of as the proof of another.</p>
+")
 
 ; Examples
 #|
@@ -395,7 +417,7 @@ effect.  We simply translate subgoal hints to computed hints:</p>
           fn)))
 
 (defun get-formals (fn world)
-  (getprop fn 'formals nil 'current-acl2-world world))
+  (getprop fn 'formals :none 'current-acl2-world world))
 
 (defun get-body (fn world)
   ;; This gets the original, normalized or non-normalized body based on what
@@ -602,16 +624,37 @@ effect.  We simply translate subgoal hints to computed hints:</p>
       (extract-keyword-from-args :flag thmpart)
     (car thmpart)))
 
-(defun assoc-flag-in-thmparts (flag thmparts)
+(defun body-from-thmpart (thmpart)
+  (cond ((not thmpart) t)
+        ((eq (car thmpart) 'defthm)
+         ;; (defthm name body ...)
+         (caddr thmpart))
+        (t ;; (flag body ...)
+         (cadr thmpart))))
+
+
+
+(defun collect-thmparts-for-flag (flag thmparts)
   (if (atom thmparts)
       nil
     (if (eq (flag-from-thmpart (car thmparts)) flag)
-        (car thmparts)
-      (assoc-flag-in-thmparts flag (cdr thmparts)))))
+        (cons (car thmparts)
+              (collect-thmparts-for-flag flag (cdr thmparts)))
+      (collect-thmparts-for-flag flag (cdr thmparts)))))
 
 
 
+(defun thmparts-collect-bodies (thmparts)
+  (if (atom thmparts)
+      nil
+    (cons (body-from-thmpart (car thmparts))
+          (thmparts-collect-bodies (cdr thmparts)))))
 
+(defun thmparts-collect-hints (thmparts)
+  (if (atom thmparts)
+      nil
+    (append (extract-keyword-from-args :hints (car thmparts))
+            (thmparts-collect-hints (cdr thmparts)))))
 
 
 
@@ -623,16 +666,14 @@ effect.  We simply translate subgoal hints to computed hints:</p>
 
   (if (consp alist)
       (let* ((flag   (cdar alist))
-             (lookup (assoc-flag-in-thmparts flag thmparts)))
-        (if (and (not lookup) (not skip-ok))
+             (flag-thmparts (collect-thmparts-for-flag flag thmparts)))
+        (if (and (not flag-thmparts) (not skip-ok))
             (er hard 'pair-up-cases-with-thmparts
                 "Expected there to be a case for the flag ~s0.~%" flag)
-          (let ((body (cond ((not lookup) t)
-                            ((eq (car lookup) 'defthm)
-                             ;; (defthm name body ...)
-                             (caddr lookup))
-                            (t ;; (flag body ...)
-                             (cadr lookup)))))
+          (let* ((bodies (thmparts-collect-bodies flag-thmparts))
+                 (body (if (eql (len bodies) 1)
+                           (car bodies)
+                         `(and . ,bodies))))
             (if (consp (cdr alist))
                 (cons `((equal ,flag-var ',flag) ,body)
                       (pair-up-cases-with-thmparts flag-var (cdr alist) thmparts skip-ok))
@@ -649,14 +690,14 @@ effect.  We simply translate subgoal hints to computed hints:</p>
 
   (if (consp alist)
       (let* ((flag   (cdar alist))
-             (lookup (assoc-flag-in-thmparts flag thmparts)))
-        (if (not lookup)
+             (flag-thmparts (collect-thmparts-for-flag flag thmparts)))
+        (if (not flag-thmparts)
             (if skip-ok
                 (cons (cons flag nil)
                       (pair-up-cases-with-hints (cdr alist) thmparts skip-ok))
               (er hard 'pair-up-cases-with-hints
                   "Expected there to be a case for the flag ~s0.~%" flag))
-          (let ((hints (extract-keyword-from-args :hints lookup)))
+          (let ((hints (thmparts-collect-hints flag-thmparts)))
             (cons (cons flag hints)
                   (pair-up-cases-with-hints (cdr alist) thmparts skip-ok)))))
     nil))
@@ -678,35 +719,30 @@ Expected an explicit name for each theorem, since no general name was
 given.  The following theorem does not have a name: ~x0~%" entry)))))
 
 
-(defun make-defthm-macro-fn-aux (lemma-name explicit-name flag-var alist thmparts)
-  ;; We have just proven the lemma and it's time to instantiate it to
-  ;; give us each thm.
-  (if (consp alist)
-      (let* ((flag (cdar alist))
-             (lookup (assoc-flag-in-thmparts flag thmparts)))
-        (if (or (not lookup)
-                (extract-keyword-from-args :skip (cddr lookup)))
-            (make-defthm-macro-fn-aux
-             lemma-name explicit-name flag-var (cdr alist) thmparts)
-          ;; Not checking for lookup, already did it when we did cases.
-          (let ((this-name
-                 (flag-thm-entry-thmname explicit-name flag lookup))
-                (body (if (eq (car lookup) 'defthm)
-                          (caddr lookup)
-                        (cadr lookup)))
-                (rule-classes (let ((mem (member :rule-classes (cddr lookup))))
-                                (if mem (cadr mem) :rewrite)))
-                (doc          (extract-keyword-from-args :doc (cddr lookup))))
-            (cons `(defthm ,this-name
-                     ,body
-                     :rule-classes ,rule-classes
-                     :doc ,doc
-                     :hints(("Goal"
-                             :in-theory (theory 'minimal-theory)
-                             :use ((:instance ,lemma-name (,flag-var ',flag))))))
-                  (make-defthm-macro-fn-aux
-                   lemma-name explicit-name flag-var (cdr alist) thmparts)))))
-    nil))
+(defun flag-defthm-corollaries (lemma-name explicit-name flag-var thmparts)
+  (if (atom thmparts)
+      nil
+    (if (extract-keyword-from-args :skip (car thmparts))
+        (flag-defthm-corollaries lemma-name explicit-name flag-var (cdr thmparts))
+      (let* ((thmpart (car thmparts))
+             (flag    (flag-from-thmpart thmpart))
+             ;; note: this can sometimes cause name conflicts when names are
+             ;; generated from the flags
+             (thmname (flag-thm-entry-thmname explicit-name flag thmpart))
+             (body (body-from-thmpart thmpart))
+             (rule-classes-look (member :rule-classes thmpart))
+             (doc (extract-keyword-from-args :doc thmpart)))
+        (cons `(with-output :stack :pop
+                 (defthm ,thmname
+                   ,body
+                   ,@(and rule-classes-look
+                          `(:rule-classes ,(cadr rule-classes-look)))
+                   :doc ,doc
+                   :hints(("Goal"
+                           :in-theory (theory 'minimal-theory)
+                           :use ((:instance ,lemma-name (,flag-var ',flag)))))))
+              (flag-defthm-corollaries lemma-name explicit-name flag-var (cdr thmparts)))))))
+              
 
 (defun find-first-thm-name (thmparts)
   (if (atom thmparts)
@@ -720,7 +756,7 @@ given.  The following theorem does not have a name: ~x0~%" entry)))))
 
 
 
-(defun make-defthm-macro-fn (args alist flag-var flag-fncall)
+(defun flag-defthm-fn (args alist flag-var flag-fncall)
   (let* ((explicit-name (and (symbolp (car args)) (car args)))
          (args (if explicit-name (cdr args) args))
          (thmparts (throw-away-keyword-parts args))
@@ -762,24 +798,27 @@ given.  The following theorem does not have a name: ~x0~%" entry)))))
                          ,flag-var
                          . ,(pair-up-cases-with-hints alist thmparts skip-ok)))))))
 
-    `(progn
-       (encapsulate
-        ()
-        (local (defthm ,name
-                 (cond . ,(pair-up-cases-with-thmparts
-                           flag-var alist thmparts skip-ok))
-                 :rule-classes nil
-                 :hints ,hints
-                 :instructions ,instructions
-                 :otf-flg ,(extract-keyword-from-args :otf-flg args)))
+    `(with-output :off :all :on (error) :stack :push
+       (progn
+         (encapsulate
+           ()
+           (local
+            (with-output :stack :pop
+              (defthm ,name
+                (cond . ,(pair-up-cases-with-thmparts
+                          flag-var alist thmparts skip-ok))
+                :rule-classes nil
+                :hints ,hints
+                :instructions ,instructions
+                :otf-flg ,(extract-keyword-from-args :otf-flg args))))
 
-        . ,(make-defthm-macro-fn-aux name explicit-name flag-var alist thmparts))
-       (value-triple ',name))))
+           . ,(flag-defthm-corollaries name explicit-name flag-var thmparts))
+         (with-output :stack :pop (value-triple ',name))))))
 
 
 (defun make-defthm-macro (real-macro-name alist flag-var flag-fncall)
   `(defmacro ,real-macro-name (&rest args) ;; was (name &rest args)
-     (make-defthm-macro-fn args ',alist ',flag-var ',flag-fncall)))
+     (flag-defthm-fn args ',alist ',flag-var ',flag-fncall)))
 
 (defun make-cases-for-equiv (alist world)
   (if (consp alist)
@@ -870,7 +909,11 @@ given.  The following theorem does not have a name: ~x0~%" entry)))))
       (if look
           (cons (cdr look) (apply-formals-subst (cdr formals) subst))
         (cons (car formals) (apply-formals-subst (cdr formals) subst))))))
-  
+
+(defun thm-macro-name (flag-fn-name)
+  (intern-in-package-of-symbol
+   (concatenate 'string "DEFTHM-" (symbol-name flag-fn-name))
+   flag-fn-name))
 
 (defun make-flag-fn (flag-fn-name clique-member-name flag-var flag-mapping hints
                                   defthm-macro-name
@@ -882,9 +925,7 @@ given.  The following theorem does not have a name: ~x0~%" entry)))))
                     (pairlis$ (get-clique-members clique-member-name world)
                               (get-clique-members clique-member-name world))))
          (defthm-macro-name (or defthm-macro-name
-                                (intern-in-package-of-symbol
-                                 (concatenate 'string "DEFTHM-" (symbol-name flag-fn-name))
-                                 flag-fn-name)))
+                                (thm-macro-name flag-fn-name)))
          (equiv-thm-name (intern-in-package-of-symbol
                           (concatenate 'string (symbol-name flag-fn-name) "-EQUIVALENCES")
                           flag-fn-name))
@@ -892,7 +933,7 @@ given.  The following theorem does not have a name: ~x0~%" entry)))))
     `(,@(if local '(progn) '(encapsulate nil))
       ;; use encapsulate instead of progn so set-ignore-ok is local to this
       (logic)
-      ,@(and (not local) '((set-ignore-ok t))) ;; can't wrap this in local --- fubar!
+      (set-ignore-ok t) ;; can't wrap this in local --- fubar!
 
       (,(if local 'local 'id)
        ,(make-flag-body flag-fn-name flag-var alist hints ruler-extenders world))
@@ -913,17 +954,21 @@ given.  The following theorem does not have a name: ~x0~%" entry)))))
                             :induct
                             (,flag-fn-name ,flag-var . ,formals)
                             :in-theory
-                            (set-difference-theories
-                             (union-theories (theory 'minimal-theory)
-                                             '((:induction ,flag-fn-name)
-                                               (:rewrite expand-all-hides)))
-                             '(;; Jared found mv-nth to be slowing down a couple of flag
-                               ;; function admissions.  Take it out of the minimal theory.
-                               (:definition mv-nth)
-                               ;; Jared found a case where "linear" forced some goals
-                               ;; from an equality, which were unprovable.  So, turn
-                               ;; off forcing.
-                               (:executable-counterpart force))))
+                            '((:induction ,flag-fn-name))
+                            ;; (set-difference-theories
+                            ;;  (union-theories (theory 'minimal-theory)
+                            ;;                  '((:induction ,flag-fn-name)
+                            ;;                    (:rewrite expand-all-hides)))
+                            ;;  '(;; Jared found mv-nth to be slowing down a couple of flag
+                            ;;    ;; function admissions.  Take it out of the minimal theory.
+                            ;;    (:definition mv-nth)
+                            ;;    ;; Jared found a case where "linear" forced some goals
+                            ;;    ;; from an equality, which were unprovable.  So, turn
+                            ;;    ;; off forcing.
+                            ;;    (:executable-counterpart force)
+                            ;;    ;; Turn of NOT to prevent case-splitting and 
+                            ;;    ))
+                            )
                            (flag-expand-computed-hint stable-under-simplificationp
                                                       ACL2::clause
                                                       ',(cons flag-fn-name
@@ -1010,6 +1055,7 @@ given.  The following theorem does not have a name: ~x0~%" entry)))))
   ;; A few syntactic variations on defining the same theorems:
   (encapsulate
    nil
+   (value-triple 1)
    (local (defthm-pseudo-termp type-of-pseudo-termp
             (term (booleanp (pseudo-termp x))
                   :rule-classes :rewrite
@@ -1018,6 +1064,7 @@ given.  The following theorem does not have a name: ~x0~%" entry)))))
 
   (encapsulate
    nil
+   (value-triple 2)
    (local (defthm-pseudo-termp type-of-pseudo-termp2
             (defthm booleanp-of-pseudo-termp
               (booleanp (pseudo-termp x))
@@ -1029,6 +1076,7 @@ given.  The following theorem does not have a name: ~x0~%" entry)))))
 
   (encapsulate
    nil
+   (value-triple 3)
    (local (in-theory (disable pseudo-termp pseudo-term-listp)))
    (local (defthm-pseudo-termp type-of-pseudo-termp
             (term (booleanp (pseudo-termp x))
@@ -1040,6 +1088,7 @@ given.  The following theorem does not have a name: ~x0~%" entry)))))
 
   (encapsulate
    nil
+   (value-triple 4)
    (local (defthm-pseudo-termp
             (term (booleanp (pseudo-termp x))
                   :rule-classes :rewrite
@@ -1050,6 +1099,7 @@ given.  The following theorem does not have a name: ~x0~%" entry)))))
 
   (encapsulate
    nil
+   (value-triple 5)
    (local (defthm-pseudo-termp
             (defthm type-of-pseudo-termp
               (booleanp (pseudo-termp x))
@@ -1063,9 +1113,16 @@ given.  The following theorem does not have a name: ~x0~%" entry)))))
 
   (encapsulate
    nil
+   (value-triple 6)
    (local (defthm-pseudo-termp
             (defthm type-of-pseudo-termp
               (booleanp (pseudo-termp x))
+              :rule-classes :type-prescription
+              :doc nil
+              :flag term)
+            (defthm pseudo-termp-equal-t
+              (equal (equal (pseudo-termp x) t)
+                     (pseudo-termp x))
               :rule-classes :rewrite
               :doc nil
               :flag term)
@@ -1127,3 +1184,278 @@ given.  The following theorem does not have a name: ~x0~%" entry)))))
                    ignore-test-f)
 
   ))
+
+
+
+
+(defxdoc def-doublevar-induction
+  :parents (mutual-recursion)
+  :short "Create an induction scheme that adds a duplicate variable to the substitution."
+  :long "<p>Certain types of proofs require inductions that are rather simple
+modifications of existing induction schemes.  For example, to prove a
+congruence on some recursive function, typically you want to induct
+<em>almost</em> on that function, but with the simple modification that for
+each substitution in the induction scheme, you want to basically copy the
+substitution of an existing variable into a new variable.</p>
+
+<p>For example, consider our attempt to prove that sum-pairs-list is nat-list congruent:</p>
+@({
+ (defun nat-list-equiv (x y)
+   (if (atom x)
+       (atom y)
+     (and (consp y)
+          (equal (nfix (car x)) (nfix (car y)))
+          (nat-list-equiv (cdr x) (cdr y)))))
+ 
+ (defun sum-pairs-list (x)
+   (if (atom x)
+       nil
+     (if (atom (cdr x))
+         (list (nfix (car x)))
+       (cons (+ (nfix (car x)) (nfix (cadr x)))
+             (sum-pairs-list (cddr x))))))
+ 
+ (defequiv nat-list-equiv)
+
+ (defthm sum-pairs-list-nat-list-equiv-congruence
+   (implies (nat-list-equiv x y)
+            (equal (sum-pairs-list x) (sum-pairs-list y)))
+   :rule-classes :congruence)
+})
+
+<p>The proof of the congruence rule fails with no hint, and neither of the
+following induction hints don't help either:</p>
+
+@({
+  :hints ((\"goal\" :induct (nat-list-equiv x y))))
+  :hints ((\"goal\" :induct (list (sum-pairs-list x)
+                                  (sum-pairs-list y))))
+})
+
+<p>What we really want is an induction scheme that inducts as sum-pairs-list
+on (say) x, but does a similar substitution on y, e.g.,</p>
+
+@({
+ (defun sum-pairs-list-double-manual (x y)
+   (declare (ignorable y))
+   (if (atom x)
+       nil
+     (if (atom (cdr x))
+         (list (nfix (car x)))
+       (cons (+ (nfix (car x)) (nfix (cadr x)))
+             (sum-pairs-list-double-manual (cddr x) (cddr y))))))
+ 
+ (defthm sum-pairs-list-nat-list-equiv-congruence ;; sum-pairs-list-double-manual works
+   (implies (nat-list-equiv x y)
+            (equal (sum-pairs-list x) (sum-pairs-list y)))
+   :hints ((\"goal\" :induct (sum-pairs-list-double-manual x y)))
+   :rule-classes :congruence)
+})
+
+<p>Def-doublevar-ind automatically generates a function like this, e.g.:</p>
+
+@({
+ (def-doublevar-induction sum-pairs-list-double
+   :orig-fn sum-pairs-list
+   :old-var x :new-var y)
+ 
+ (defthm sum-pairs-list-nat-list-equiv-congruence ;; sum-pairs-list-double works
+   (implies (nat-list-equiv x y)
+            (equal (sum-pairs-list x) (sum-pairs-list y)))
+   :hints ((\"goal\" :induct (sum-pairs-list-double x y)))
+   :rule-classes :congruence)
+})
+
+<p>This can be used with flag functions and their defthm macros (see @(see make-flag)): use def-doublevar-ind to define a new induction scheme based on the flag function, and give a hint to the flag defthm macro to use that induction scheme. For example,</p>
+@({
+ (flag::make-flag foo-flag foo-mutualrec ...)
+
+ (flag::def-doublevar-ind foo-doublevar-ind
+   :orig-fn foo-flag
+   :old-var x :new-var y)
+
+ (defthm-foo-flag
+  (defthm foo1-thm ...)
+  (defthm foo2-thm ...)
+  :hints ((\"goal\" :induct (foo-doublevar-ind flag x a b y))))
+})
+")
+
+
+(defun doublevar-transform-calls (calls fnname old-var-index old-var new-var)
+  (if (atom calls)
+      nil
+    (let ((actuals (cdr (car calls))))
+      (cons (cons fnname (append actuals
+                                 (list
+                                  (acl2::subst-var new-var old-var (nth old-var-index actuals)))))
+            (doublevar-transform-calls (cdr calls) fnname old-var-index old-var new-var)))))
+
+(defun doublevar-different-equals-p (test1 test2)
+  (and (consp test1)
+       (consp test2)
+       (eq (car test1) 'equal)
+       (eq (car test2) 'equal)
+       (let* ((quote1 (if (quotep (cadr test1))
+                          (cadr test1)
+                        (and (quotep (caddr test1))
+                             (caddr test1))))
+              (quote2 (if (quotep (cadr test2))
+                          (cadr test2)
+                        (and (quotep (caddr test2))
+                             (caddr test2)))))
+         (and quote1
+              quote2
+              (not (equal quote1 quote2))
+              (or (equal (cadr test1) (cadr test2))
+                  (equal (cadr test1) (caddr test2))
+                  (equal (caddr test1) (cadr test2))
+                  (equal (caddr test1) (caddr test2)))))))
+
+(defun doublevar-place-calls-in-body (tests term calls-term)
+  (declare (xargs :measure (make-ord 1 (+ 1 (acl2-count term))
+                                     (acl2-count tests))
+                  :mode :program))
+  (if (atom tests)
+      calls-term
+    (let* ((negp (and (consp (car tests))
+                      (eq (caar tests) 'not)))
+           (test-term (if negp (cadar tests) (car tests)))
+           (diff-equals (and (not negp)
+                             (doublevar-different-equals-p
+                              test-term (cadr term))))
+           (matchp (and (eq (car term) 'if)
+                        (or (equal (cadr term) test-term)
+                            diff-equals)))
+           (new-subterm (doublevar-place-calls-in-body
+                         (if diff-equals tests (cdr tests))
+                         (if diff-equals
+                             (cadddr term)
+                           (if matchp
+                               (if negp
+                                   (cadddr term)
+                                 (caddr term))
+                             nil))
+                         calls-term)))
+      (if diff-equals
+          (list 'if (cadr term) (caddr term) new-subterm)
+        (if negp
+            (list 'if test-term (and matchp (caddr term)) new-subterm)
+          (list 'if test-term new-subterm (and matchp (cadddr term))))))))
+
+(defun doublevar-ind-body-add-tests/calls (tests term calls-term)
+  (declare (xargs :mode :program))
+  (doublevar-place-calls-in-body tests term calls-term))
+
+
+(defun doublevar-ind-body (ind-machine fnname old-var-index old-var new-var term)
+  (declare (xargs :mode :program))
+  (if (atom ind-machine)
+      term
+    (let* ((tests (access acl2::tests-and-calls (car ind-machine) :tests))
+           (calls (access acl2::tests-and-calls (car ind-machine) :calls))
+           (calls-term `(list ,(len ind-machine)
+                              . ,(doublevar-transform-calls
+                                  calls fnname old-var-index old-var new-var)))
+           (new-term (doublevar-ind-body-add-tests/calls tests term calls-term)))
+      (doublevar-ind-body (cdr ind-machine) fnname old-var-index old-var new-var new-term))))
+  
+
+
+(defun def-doublevar-induction-fn (name f old-var new-var hints w)
+  (declare (xargs :mode :program))
+  (let* ((formals (get-formals f w))
+         (ind-machine (getprop f 'acl2::induction-machine :none 'current-acl2-world w)))
+    (cond ((eq formals :none)
+           (er hard? 'def-doublevar-induction-fn "Not a function -- no formals~%"))
+          ((not (member-eq old-var formals))
+           (er hard? 'def-doublevar-induction-fn "~x0 is not an existing formal of ~x1~%"
+               old-var f))
+          ((eq ind-machine :none)
+           (er hard? 'def-doublevar-induction-fn "No induction machine -- not singly recursive?~%"))
+          (t
+           (let* ((measure (get-measure f w))
+                  (wfr (get-wfr f w))
+                  (old-var-index (search (list old-var) formals))
+                  (all-formals (append formals (list new-var))))
+             `(defun-nx ,name ,all-formals
+                (declare (xargs :verify-guards nil
+                                :measure ,measure
+                                :hints ,hints
+                                :well-founded-relation ,wfr
+                                :mode :logic)
+                         (ignorable . ,all-formals))
+                ,(doublevar-ind-body ind-machine name old-var-index old-var new-var nil)))))))
+
+(defmacro def-doublevar-induction (name &key orig-fn old-var new-var hints)
+  `(make-event
+    (def-doublevar-induction-fn ',name ',orig-fn ',old-var ',new-var ',hints (w state))))
+
+
+(local
+ (progn
+
+   (defun nat-list-equiv (x y)
+     (if (atom x)
+         (atom y)
+       (and (consp y)
+            (equal (nfix (car x)) (nfix (car y)))
+            (nat-list-equiv (cdr x) (cdr y)))))
+
+   (defun sum-pairs-list (x)
+     (if (atom x)
+         nil
+       (if (atom (cdr x))
+           (list (nfix (car x)))
+         (cons (+ (nfix (car x)) (nfix (cadr x)))
+               (sum-pairs-list (cddr x))))))
+
+   ;; It is a *MIRACLE* that this works given how many sub-inductions it does.
+   (defequiv nat-list-equiv)
+
+   ;; ;; no hint fails
+   ;; (defthm sum-pairs-list-nat-list-equiv-congruence 
+   ;;   (implies (nat-list-equiv x y)
+   ;;            (equal (sum-pairs-list x) (sum-pairs-list y)))
+   ;;   :rule-classes :congruence)
+
+   ;; ;; induct equiv hint fails
+   ;; (defthm sum-pairs-list-nat-list-equiv-congruence
+   ;;   (implies (nat-list-equiv x y)
+   ;;            (equal (sum-pairs-list x) (sum-pairs-list y)))
+   ;;   :rule-classes :congruence
+   ;;   :hints (("goal" :induct (nat-list-equiv x y))))
+
+   ;; ;; induct both hint fails
+   ;; (defthm sum-pairs-list-nat-list-equiv-congruence
+   ;;   (implies (nat-list-equiv x y)
+   ;;            (equal (sum-pairs-list x) (sum-pairs-list y)))
+   ;;   :rule-classes :congruence
+   ;;   :hints (("goal" :induct (list (sum-pairs-list x) (sum-pairs-list y)))))
+
+   ;; (defun sum-pairs-list-double-manual (x y)
+   ;;   (declare (ignorable y))
+   ;;   (if (atom x)
+   ;;       nil
+   ;;     (if (atom (cdr x))
+   ;;         (list (nfix (car x)))
+   ;;       (cons (+ (nfix (car x)) (nfix (cadr x)))
+   ;;             (sum-pairs-list-double-manual (cddr x) (cddr y))))))
+
+   ;; ;; sum-pairs-list-double-manual works but is not what we want to test
+   ;; (defthm sum-pairs-list-nat-list-equiv-congruence
+   ;;   (implies (nat-list-equiv x y)
+   ;;            (equal (sum-pairs-list x) (sum-pairs-list y)))
+   ;;   :rule-classes :congruence
+   ;;   :hints (("goal" :induct (sum-pairs-list-double-manual x y))))
+
+   (def-doublevar-induction sum-pairs-list-double
+     :orig-fn sum-pairs-list
+     :old-var x :new-var y)
+
+   (defthm sum-pairs-list-nat-list-equiv-congruence ;; sum-pairs-list-double works
+     (implies (nat-list-equiv x y)
+              (equal (sum-pairs-list x) (sum-pairs-list y)))
+     :rule-classes :congruence
+     :hints (("goal" :induct (sum-pairs-list-double x y))))))
+
