@@ -6,15 +6,25 @@
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 ;
 ; Original author: Jared Davis <jared@centtech.com>
 
@@ -48,10 +58,10 @@
 (defconst *throwaway-tags*
   ;; We leave "img" and "icon" in this list even thouggh we process them in
   ;; merge-text below, because we don't want to process </img> or </icon>.
-  (list "b" "i" "u" "tt" "v" "color" "sf" "a" "box" "img" "icon"
+  (list "b" "i" "u" "tt" "v" "color" "sf" "box" "img" "icon"
         "page"))
 
-(defun merge-text (x acc codes)
+(defun merge-text (x acc codes href)
   ;; CODES is number of open <code> tags -- we don't normalize whitespace
   ;; within them, but entities still get converted.
   (b* (((when (atom x))
@@ -65,40 +75,42 @@
                       codes)))
           (cond ((equal name "img")
                  (b* ((tok  (list :TEXT "{IMAGE}")))
-                   (merge-text (cons tok rest) acc codes)))
+                   (merge-text (cons tok rest) acc codes href)))
                 ((equal name "icon")
                  (b* ((tok  (list :TEXT "{ICON}")))
-                   (merge-text (cons tok rest) acc codes)))
+                   (merge-text (cons tok rest) acc codes href)))
                 ((member-equal name *throwaway-tags*)
-                 (merge-text rest acc codes))
+                 (merge-text rest acc codes nil))
                 ((equal name "a")
-                 (b* ((href (cdr (assoc :href (opentok-atts tok1))))
-                      (href (if (stringp href) href ""))
-                      (tok  (list :TEXT (str::cat "[" href " "))))
-                   (merge-text (cons tok rest) acc codes)))
+                 (b* ((href (cdr (assoc-equal "href" (opentok-atts tok1))))
+                      (tok  (list :TEXT (str::cat "{"))))
+                   (merge-text (cons tok rest) acc codes href)))
                 ((equal name "see")
                  (b* ((tok  (list :TEXT "[")))
-                   (merge-text (cons tok rest) acc codes)))
+                   (merge-text (cons tok rest) acc codes href)))
                 ((equal name "srclink")
                  (b* ((tok  (list :TEXT "<")))
-                   (merge-text (cons tok rest) acc codes)))
+                   (merge-text (cons tok rest) acc codes href)))
                 (t
-                 (merge-text rest (cons tok1 acc) codes)))))
+                 (merge-text rest (cons tok1 acc) codes href)))))
        ((when (closetok-p tok1))
         (b* ((name  (closetok-name tok1))
              (codes (if (equal name "code")
                         (- 1 codes)
                       codes)))
           (cond ((member-equal name *throwaway-tags*)
-                 (merge-text rest acc codes))
-                ((member-equal name '("a" "see"))
+                 (merge-text rest acc codes href))
+                ((member-equal name '("see"))
                  (let ((tok (list :TEXT "]")))
-                   (merge-text (cons tok rest) acc codes)))
+                   (merge-text (cons tok rest) acc codes href)))
+                ((member-equal name '("a"))
+                 (let ((tok (list :TEXT (str::cat " | " (or href "") "}"))))
+                   (merge-text (cons tok rest) acc codes href)))
                 ((equal name "srclink")
                  (let ((tok (list :TEXT ">")))
-                   (merge-text (cons tok rest) acc codes)))
+                   (merge-text (cons tok rest) acc codes href)))
                 (t
-                 (merge-text rest (cons tok1 acc) codes)))))
+                 (merge-text rest (cons tok1 acc) codes href)))))
        (tok1
         ;; Goofy.  Convert any entities into ordinary text.  Normalize
         ;; whitespace for any non-code tokens.
@@ -111,11 +123,11 @@
                ;; Inside a <code> block, so don't touch ws.
                tok1)))
        ((unless (texttok-p (car acc)))
-        (merge-text rest (cons tok1 acc) codes))
+        (merge-text rest (cons tok1 acc) codes href))
 
        (merged-tok (list :TEXT (str::cat (texttok-text (car acc))
                                          (texttok-text tok1)))))
-    (merge-text rest (cons merged-tok (cdr acc)) codes)))
+    (merge-text rest (cons merged-tok (cdr acc)) codes href)))
 
 
 (defun has-tag-above (tag open-tags)
@@ -339,10 +351,14 @@
   (b* ((name (cdr (assoc :name x)))
 ;       (- (cw "Preprocessing...~%"))
        ;; Use NIL as the topics-fal as a simple way to suppress autolinks...
+       (topics-fal (topics-fal all-topics))
        ((mv text state)
         (preprocess-topic
          (acons :parents nil x) ;; horrible hack so we can control this better
-         all-topics nil nil state))
+         all-topics topics-fal
+         t ;; disable-autolinking to avoid auto links in text-mode code
+         state))
+       (- (fast-alist-free topics-fal))
 ;       (- (cw "Text is ~x0.~%" text))
 ;       (- (cw "Parsing xml...~%"))
        ((mv err tokens) (parse-xml text))
@@ -358,7 +374,7 @@
           state))
 ;       (- (cw "Tokens are ~x0.~%" tokens))
 ;       (- (cw "Merging tokens...~%"))
-       (merged-tokens (reverse (merge-text tokens nil 0)))
+       (merged-tokens (reverse (merge-text tokens nil 0 nil)))
 ;       (- (cw "Merged tokens are ~x0.~%" merged-tokens))
        (terminal (str::rchars-to-string
                   (tokens-to-terminal merged-tokens 70 nil nil nil)))
@@ -416,8 +432,7 @@
        (short    (cdr (assoc :short x)))
 ;       (- (cw "Preprocessing...~%"))
        ;; Use NIL as the topics-fal as a simple way to suppress autolinks...
-       ((mv short-acc state) (preprocess-main short
-                                              nil ;; no directory is needed here
+       ((mv short-acc state) (preprocess-main short name
                                               nil ;; no topics-fal, just keep it simple
                                               base-pkg
                                               state
@@ -437,7 +452,7 @@
           state))
 ;       (- (cw "Tokens are ~x0.~%" tokens))
 ;       (- (cw "Merging tokens...~%"))
-       (merged-tokens (reverse (merge-text tokens nil 0)))
+       (merged-tokens (reverse (merge-text tokens nil 0 nil)))
 ;       (- (cw "Merged tokens are ~x0.~%" merged-tokens))
        (terminal (str::rchars-to-string (tokens-to-terminal merged-tokens 70 nil nil nil)))
        (state (princ$ "    " *standard-co* state))
