@@ -6,15 +6,25 @@
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 ;
 ; Original author: Jared Davis <jared@centtech.com>
 
@@ -171,7 +181,7 @@
   (b* ((short    (or (cdr (assoc :short topic)) ""))
        (base-pkg (cdr (assoc :base-pkg topic)))
        (name     (cdr (assoc :name topic)))
-       ((mv short-rchars state) (preprocess-main short nil topics-fal base-pkg state nil))
+       ((mv short-rchars state) (preprocess-main short name topics-fal base-pkg state nil))
        (short-str (str::rchars-to-string short-rchars))
        ((mv err &) (parse-xml short-str))
        (state
@@ -200,7 +210,7 @@
   (b* ((long     (or (cdr (assoc :long topic)) ""))
        (base-pkg (cdr (assoc :base-pkg topic)))
        (name     (cdr (assoc :name topic)))
-       ((mv long-rchars state) (preprocess-main long nil topics-fal base-pkg state nil))
+       ((mv long-rchars state) (preprocess-main long name topics-fal base-pkg state nil))
        (long-str (str::rchars-to-string long-rchars))
        ((mv err &) (parse-xml long-str))
        (state
@@ -226,28 +236,15 @@
 
 
 (defun json-encode-index-entry (topic topics-fal uid-map state acc)
-  (b* ((name     (cdr (assoc :name topic)))
+  (b* ((- (check-topic-syntax topic))
+
+       (name     (cdr (assoc :name topic)))
        (base-pkg (cdr (assoc :base-pkg topic)))
-       (short    (or (cdr (assoc :short topic)) ""))
-       (long     (or (cdr (assoc :long topic)) ""))
        (parents  (cdr (assoc :parents topic)))
-       ((unless (symbolp name))
-        (mv (er hard? 'preprocess-topic "Name is not a string: ~x0.~%" topic)
-            state))
-       ((unless (symbolp base-pkg))
-        (mv (er hard? 'preprocess-topic "Base-pkg is not a symbol: ~x0.~%" topic)
-            state))
-       ((unless (symbol-listp parents))
-        (mv (er hard? 'preprocess-topic "Parents are not a symbol-listp: ~x0.~%" topic)
-            state))
-       ((unless (stringp short))
-        (mv (er hard? 'preprocess-topic "Short is not a string: ~x0.~%" topic)
-            state))
-       ((unless (stringp long))
-        (mv (er hard? 'preprocess-topic "Long is not a string: ~x0.~%" topic)
-            state))
+       (suborder (cdr (assoc :suborder topic)))
 
        (parent-uids (collect-uids parents uid-map))
+       (suborder    (and suborder (collect-uids suborder uid-map)))
 
        ((mv short-str state) (short-xml-for-topic topic topics-fal state))
 
@@ -278,6 +275,13 @@
        (acc (cons #\, acc))
        (acc (json-encode-string short-str acc))
 
+       ;; most topics don't have a suborder, so only include this if they do have one.
+       (acc (b* (((unless suborder)
+                  acc)
+                 (acc (cons #\, acc))
+                 (acc (json-encode-uids suborder acc)))
+              acc))
+
        (acc (cons #\] acc)))
     (mv acc state)))
 
@@ -318,31 +322,11 @@
 
 
 (defun json-encode-data-entry (topic topics-fal state acc)
-  (b* ((__function__ 'json-encode-data-entry)
+  (b* ((- (check-topic-syntax topic))
        (name     (cdr (assoc :name topic)))
        (base-pkg (cdr (assoc :base-pkg topic)))
-       (short    (or (cdr (assoc :short topic)) ""))
-       (long     (or (cdr (assoc :long topic)) ""))
        (parents  (cdr (assoc :parents topic)))
        (from     (or (cdr (assoc :from topic)) "Unknown"))
-       ((unless (symbolp name))
-        (mv (er hard? __function__ "Name is not a string: ~x0.~%" topic)
-            state))
-       ((unless (symbolp base-pkg))
-        (mv (er hard? __function__ "Base-pkg is not a symbol: ~x0.~%" topic)
-            state))
-       ((unless (symbol-listp parents))
-        (mv (er hard? __function__ "Parents are not a symbol-listp: ~x0.~%" topic)
-            state))
-       ((unless (stringp short))
-        (mv (er hard? __function__ "Short is not a string: ~x0.~%" topic)
-            state))
-       ((unless (stringp long))
-        (mv (er hard? __function__ "Long is not a string: ~x0.~%" topic)
-            state))
-       ((unless (stringp from))
-        (mv (er hard? __function__ "From is not a string: ~x0.~%" topic)
-            state))
 
        ((mv long-str state) (long-xml-for-topic topic topics-fal state))
 
@@ -411,11 +395,16 @@
                   (clean-topics topics)))))
 
        (- (cw "; Saving JSON files for ~x0 topics.~%" (len topics)))
-       ((mv topics xtopics state)
+       ((mv topics xtopics sitemap state)
         (time$ (order-topics-by-importance topics state)
                :msg "; Importance sorting topics: ~st sec, ~sa bytes.~%"
                :mintime 1/2))
 
+       (smfile (oslib::catpath dir "sitemap.xml"))
+       ((mv channel state) (open-output-channel smfile :character state))
+       (state (princ$ sitemap channel state))
+       (state (close-output-channel channel state))
+
        (lcfile (oslib::catpath dir "linkcheck.html"))
        ((mv channel state) (open-output-channel lcfile :character state))
        (state (princ$ (linkcheck xtopics) channel state))
@@ -497,6 +486,7 @@
                                                 "print.html"
                                                 "printer.png"
                                                 "render.js"
+                                                "render-html.xsl"
                                                 "style.css"
                                                 "view_flat.png"
                                                 "view_tree.png"
@@ -505,15 +495,16 @@
                                                 "xslt.js"
                                                 "xdoc_index.js"
                                                 "xdataget.pl"
+                                                "xdata2html.pl"
                                                 "xdata2sql.pl"
                                                 "zip.sh"
+                                                ".htaccess"
                                                 )
                                           dir state))
 
        (- (cw "Copying fancy viewer library files...~%"))
        (state          (stupid-copy-files xdoc/fancy/lib
-                                          (list "hogan-2.0.0.js"
-                                                "jquery-2.0.3.js"
+                                          (list "jquery-2.0.3.js"
                                                 "jquery-2.0.3.min.js"
                                                 "jquery.base64.js"
                                                 "jquery.powertip.css"
@@ -530,11 +521,19 @@
                                           dir/images state)))
     state))
 
+(defttag :xdoc) ; for sys-call+ call below
 
 (defun run-fancy-zip (dir state)
   (b* ((- (cw "; XDOC: Running zip.sh to create download/ directory.~%"))
+       ;; We formerly used oslib::catpath here.  However, David Rager reported
+       ;; (2014-06-24) that this caused failures when using directories such as
+       ;; "~/public_html", because the ~ doesn't get wildcard expanded in all
+       ;; Lisps, e.g., CCL.  So, we now use ACL2's extend-pathname, since it
+       ;; gets rid of the ~ characters.
+       (dir-fix (acl2::extend-pathname dir "." state))
+       (zip.sh  (acl2::extend-pathname dir-fix "zip.sh" state))
        ((mv erp val state)
-        (time$ (sys-call+ "sh" (list (oslib::catpath dir "zip.sh") dir) state)
+        (time$ (sys-call+ "sh" (list zip.sh dir-fix) state)
                :msg "; XDOC zip.sh: ~st sec, ~sa bytes.~%"))
        ((when erp)
         (er hard? 'run-fancy-zip
@@ -543,9 +542,11 @@
         state))
     state))
 
-(defun save-fancy (all-topics dir state)
+(defun save-fancy (all-topics dir zip-p state)
   (b* ((state (prepare-fancy-dir dir state))
        (state (save-json-files all-topics dir state))
-       (state (run-fancy-zip dir state)))
+       (state (if zip-p
+                  (run-fancy-zip dir state)
+                state)))
     state))
 
