@@ -6,15 +6,25 @@
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 ;
 ; Original author: Jared Davis <jared@centtech.com>
 
@@ -58,7 +68,8 @@
        (progn
          (colon-xdoc-init)
          (make-event
-          (b* (((mv all-xdoc-topics state) (all-xdoc-topics state))
+          (b* (((mv all-xdoc-topics state) 
+                (with-guard-checking t (all-xdoc-topics state)))
                ((mv & & state) (colon-xdoc-fn ',name all-xdoc-topics state)))
             (value '(value-triple :invisible))))))))
 
@@ -67,47 +78,50 @@
 (add-ld-keyword-alias! :doc '(1 xdoc))
 
 (defmacro xdoc-extend (name long)
-  ;; Extend an existing xdoc topic with more content.  Long is an xdoc
-  ;; format string, e.g., "<p>blah blah @(def blah) blah.</p>" that will
-  ;; be appended onto the end of the current :long for this topic.
   `(table xdoc 'doc
           (let* ((all-topics   (xdoc::get-xdoc-table world))
                  (old-topic    (xdoc::find-topic ',name all-topics))
                  (long         (or ,long "")))
             (cond ((not old-topic)
                    (prog2$
-                    (er hard? 'xdoc-extend
-                        "Trying to extend topic ~x0, but this topic wasn't found."
-                        ',name)
+                    (er hard? 'xdoc-extend "Topic ~x0 wasn't found." ',name)
                     all-topics))
                    (t
                     (let* ((other-topics (remove-equal old-topic all-topics))
-                           (old-long     (cdr (assoc :long old-topic)))
+                           (old-long     (or (cdr (assoc :long old-topic)) ""))
                            (new-long     (concatenate 'string old-long long))
                            (new-topic    (acons :long new-long
                                                 (delete-assoc :long old-topic))))
                       (cons new-topic other-topics)))))))
 
 (defmacro xdoc-prepend (name long)
-  ;; Extend an existing xdoc topic with more content.  Long is an xdoc
-  ;; format string, e.g., "<p>blah blah @(def blah) blah.</p>" that will
-  ;; be prepended to the front of the current :long for this topic.
   `(table xdoc 'doc
           (let* ((all-topics   (xdoc::get-xdoc-table world))
                  (old-topic    (xdoc::find-topic ',name all-topics))
                  (long         (or ,long "")))
             (cond ((not old-topic)
-                   (prog2$
-                    (er hard? 'xdoc-prepend
-                        "Trying to prepend to topic ~x0, but this topic wasn't
-                         found." ',name)
-                    all-topics))
-                   (t
+                   (er hard? 'xdoc-prepend "Topic ~x0 wasn't found." ',name))
+                  (t
+                   (let* ((other-topics (remove-equal old-topic all-topics))
+                          (old-long     (cdr (assoc :long old-topic)))
+                          (new-long     (concatenate 'string long old-long))
+                          (new-topic    (acons :long new-long
+                                               (delete-assoc :long old-topic))))
+                     (cons new-topic other-topics)))))))
+
+(defmacro order-subtopics (name order)
+  `(table xdoc 'doc
+          (let* ((all-topics (xdoc::get-xdoc-table world))
+                 (old-topic  (xdoc::find-topic ',name all-topics))
+                 (order      ',order))
+            (cond ((not old-topic)
+                   (er hard? 'order-subtopics "Topic ~x0 wasn't found." ',name))
+                  ((not (symbol-listp order))
+                   (er hard? 'order-subtopics "Subtopics are not a symbol list: ~x0" order))
+                  (t
                     (let* ((other-topics (remove-equal old-topic all-topics))
-                           (old-long     (cdr (assoc :long old-topic)))
-                           (new-long     (concatenate 'string long old-long))
-                           (new-topic    (acons :long new-long
-                                                (delete-assoc :long old-topic))))
+                           (new-topic    (acons :suborder order
+                                                (delete-assoc :suborder old-topic))))
                       (cons new-topic other-topics)))))))
 
 (defund extract-keyword-from-args (kwd args)
