@@ -1,20 +1,30 @@
 ; XDOC Documentation System for ACL2
-; Copyright (C) 2009-2011 Centaur Technology
+; Copyright (C) 2009-2014 Centaur Technology
 ;
 ; Contact:
 ;   Centaur Technology Formal Verification Group
 ;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
 ;   http://www.centtech.com/
 ;
-; This program is free software; you can redistribute it and/or modify it under
-; the terms of the GNU General Public License as published by the Free Software
-; Foundation; either version 2 of the License, or (at your option) any later
-; version.  This program is distributed in the hope that it will be useful but
-; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-; more details.  You should have received a copy of the GNU General Public
-; License along with this program; if not, write to the Free Software
-; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+; License: (An MIT/X11-style license)
+;
+;   Permission is hereby granted, free of charge, to any person obtaining a
+;   copy of this software and associated documentation files (the "Software"),
+;   to deal in the Software without restriction, including without limitation
+;   the rights to use, copy, modify, merge, publish, distribute, sublicense,
+;   and/or sell copies of the Software, and to permit persons to whom the
+;   Software is furnished to do so, subject to the following conditions:
+;
+;   The above copyright notice and this permission notice shall be included in
+;   all copies or substantial portions of the Software.
+;
+;   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+;   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+;   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+;   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+;   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+;   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+;   DEALINGS IN THE SOFTWARE.
 ;
 ; Original author: Jared Davis <jared@centtech.com>
 
@@ -29,6 +39,7 @@
 (include-book "import-acl2doc")  ;; For base acl2 documentation
 
 (defxdoc xdoc
+  :parents (documentation)
   :short "<i>XDOC</i> is a tool for documenting ACL2 books.  You can use it to
 access documentation about ACL2 and its books, to document your own books, and
 to create custom web-based manuals.  It is intended as a replacement for ACL2
@@ -61,19 +72,19 @@ See @(see acl2::acl2-doc) for details.</li>
 
 </ul>
 
-<p>You can also build your own copy of the manual as follows.
-(This has been tested using CCL on Linux and Mac OS X, but may work for other
-OS/Lisp combinations.)  To do this, you first need to build ACL2(h), then
-certify the @('centaur/doc') book, e.g., as follows:</p>
+<p>You can also build your own copy of the manual as follows. (This has been
+tested using CCL on Linux and Mac OS X, but may work for other OS/Lisp
+combinations.)  To do this, you first need to build ACL2(h), then certify the
+@('doc/top') book, e.g., as follows:</p>
 
 @({
   cd acl2-sources/books
-  make USE_QUICKLISP=1 ACL2=my-acl2h centaur/doc.cert
+  make USE_QUICKLISP=1 ACL2=my-acl2h doc/top.cert
 })
 
 <p>After this is built, the web-based manual should be available at:</p>
 
-@({acl2-sources/books/centaur/manual/index.html})
+@({acl2-sources/books/doc/manual/index.html})
 
 <p>See also @(see acl2::acl2-doc) for details about how to build your own
 Emacs-based manual.</p>
@@ -178,8 +189,8 @@ markup) language, and may also use @(see preprocessor) commands to insert
 function definitions, theorems, topic links, and so on.</p>
 
 <p>Many examples of using XDOC can be found throughout the ACL2 books.  See for
-instance the @(see acl2::std), @(see acl2::str) or @(see acl2::osets)
-libraries.</p>
+instance the @(see acl2::std), @(see acl2::std/strings) or @(see
+acl2::std/osets) libraries.</p>
 
 <h3>Note for Advanced Users</h3>
 
@@ -196,8 +207,8 @@ use @(see defxdoc-raw) instead.</p>")
 language that is the basis of XDOC documentation strings."
 
   :long "<p>XDOC uses an XML markup language that is similar to a subset of <a
-href='http://en.wikipedia.org/wiki/HTML'>HTML</a>.  Note that in XML, beginning
-and ending tags really need to be balanced.</p>
+href='http://en.wikipedia.org/wiki/HTML'>HTML</a>.  Note that unlike HTML,
+beginning and ending tags are case-sensitive and must match exactly.</p>
 
 <h3>Formatting Text</h3>
 
@@ -461,7 +472,7 @@ documentation.  For instance:</p>
 </ul>
 
 <p>By default, the backtick syntax introduces @('<tt>') tags and automatically
-escaped any special XML characters like &lt;.  Sometimes this isn't what you
+escapes any special XML characters like @('<').  Sometimes this isn't what you
 want.  When the result of your evaluation is large, you can use a spcial
 @(':code') prefix to insert @('<code>') tags instead.  For instance:
 @('@(`(:code (make-list 100))`)') produces:</p>
@@ -472,7 +483,7 @@ want.  When the result of your evaluation is large, you can use a spcial
 sophisticated uses of evaluation, e.g., to generate hyperlinks, etc.  The
 special @(':raw') prefix can be used to insert exactly the contents of a
 string, with no automatic escaping.  For instance:
-  @('  @(`(:raw \"a &lt; b\")`)  ') produces 
+  @('  @(`(:raw \"a &lt; b\")`)  ') produces
        @(`(:raw \"a &lt; b\")`).
 Note that it is quite easy to use @(':raw') incorrectly; you have to escape
 things properly yourself!</p>
@@ -498,7 +509,8 @@ that are generated by macros.  You can suppress the automatic source links on
 <p>The easiest way to link between topics is to use @('@(see name)'), which
 expands into a link to @('name').  The text shown to the reader is just the
 name of the topic, in lower case.  Note also that @('@(csee name)') can be used
-for links whose first letter should be capitalized.</p>
+for links whose first letter should be capitalized, and that @('@(tsee name)')
+can be used for links that should appear in type-writer font.</p>
 
 <p>For most purposes, @('@(see name)') is adequate and it is also recommended.
 Finer-grained control (e.g., changing the link text) is also possible, but then
@@ -601,9 +613,10 @@ manual with others, you should read about @(see deploying-manuals).</li>
 
 @({
     (save <target-dir>
-          [:type      type]      ;; default is :fancy
           [:import    import]    ;; default is t
-          <classic-options>)
+          [:redef-okp bool]      ;; default is nil
+          [:zip-p     bool]      ;; default is t
+          )
 })
 
 <p>The only (required) argument to the @('save') command is the name of a
@@ -618,13 +631,6 @@ overwritten</color>.</li>
 
 </ul>
 
-<p>XDOC can generate two kinds of manuals.  By default, it generates a
-so-called <i>fancy</i> manual, with a rich JavaScript interface that has, e.g.,
-jump-to and search capabilities.  Alternately you can generate a much more
-plain @(see classic-manual) by using @(':type :classic'), but we may move to
-deprecate classic manuals in the future.</p>
-
-
 <h3>Avoiding Unwanted Documentation</h3>
 
 <p>By default, the @('save') command will automatically import:</p>
@@ -639,8 +645,8 @@ deprecate classic manuals in the future.</p>
 
 <p>However, you may find that even after setting @(':import nil'), some
 extraneous documentation is still being included!  For instance, you may find
-documentation from libraries like @(see str::str) and @(see oslib::oslib) in
-your output.</p>
+documentation from libraries like @(see acl2::std/strings) and @(see
+oslib::oslib) in your output.</p>
 
 <p>This is because @('xdoc/save') includes some supporting books that are,
 themselves, documented.  If you really want precise control over what goes into
@@ -700,7 +706,7 @@ sites, then you may wish to read on.</p>
 
 <p>The basic reason that the default manuals are slow is that they work by
 simply loading the data for <b>every</b> topic, at startup.  As of October
-2013, this comes to around 25 MB of data for the basic @('centaur/doc.lisp')
+2013, this comes to around 25 MB of data for the basic @('doc/top.lisp')
 manual.  It's no big deal to load 25 MB from a local hard drive or a fast
 intranet connection, but it can be quite slow over the internet.</p>
 
@@ -832,65 +838,6 @@ just need to change:</p>
 <p>At this point, your manual should load topic data dynamically as needed.
 The result should be much faster for users on slow connections.</p>")
 
-
-(defxdoc classic-manual
-  :parents (save)
-  :short "Description of @(':type :classic') manuals."
-
-  :long "<box><p><b>Note</b>: we generally discourage the use of classic
-manuals.  We may deprecate them in the future.</p></box>
-
-<p>If you run @(see save) with @(':type :classic'), it will write out
-a manual in the \"classic\" format.  In this case, the resulting manual
-directory will include:</p>
-
-<ul>
-
-<li>@('xml/'), a subdirectory with a @('.xml') file for each topic and some
-supporting files, and</li>
-
-<li>@('Makefile'), a Makefile for converting these files into other formats,
-and</li>
-
-<li>@('preview.html'), a web page that lets you directly view the XML files
-using your web browser.</li>
-
-</ul>
-
-<p>Many web browsers can directly display XML files, so you may be able to view
-@('preview.html') without any additional steps.</p>
-
-
-<h3>HTML and Other Formats</h3>
-
-<p>You can generate a plain HTML or TEXT version of your manual by using
-@('make html') or @('make text') from within the directory target of the above
-mentioned @('xdoc::save') command (in this example, the @('mylib-manual')
-directory).  We might add support for TEXINFO or other formats in the
-future.</p>
-
-<p>After running @('make html'), you may wish to open @('frames2.html') and
-@('frames3.html'), which allow you to navigate the HTML manual much like
-@('preview.html') allows you to navigate the XML version.  These pages accept
-an optional argument named <tt>topic</tt> that tells the browser to
-automatically go to a particular topic.  For example, one can go to the
-<tt>XDOC::save</tt> topic by using the url
-<tt>frames3.html?topic=XDOC____SAVE.html</tt>.</p>
-
-<p>Converting to HTML is a good idea because it ensures that all of your tags
-are balanced on every page.  Without this sanity check, your manual might
-contain errors that will prevent some topics from being loaded in a web
-browser.</p>
-
-<p>Creating the HTML code requires Xalan-C++.  Xalan is distributed with many
-Linux distributions.  For example, on Ubuntu, one can run <tt>sudo apt-get
-install xalan</tt> to install it.  Alternatively, see <a
-href=\"http://xml.apache.org/xalan-c/\">Apache Xalan</a> to download.  We have
-accomodated the various versions of Xalan that we know about and use, but we
-welcome modifications to file <tt>support/Makefile-trans</tt> if you wish to
-use a version we do not currently support.</p>")
-
-
 (defxdoc emacs-links
   :short "Instructions for integrating XDOC web pages with <a
   href='http://www.gnu.org/software/emacs/'>Emacs</a>."
@@ -1153,9 +1100,9 @@ any of these keywords are provided, they will be used to introduce a @(see
 defxdoc) command; otherwise no documentation will be generated.</p>
 
 <p>By default, the @(':long') string you give will be automatically extended
-with a \"Definitions and Theorems\" part that lists all the (non-local)
-definitions and theorems introduced in the section.  In the above example, the
-@(':long') field would be extended with:</p>
+with a \"Definitions and Theorems\" part that lists all the (non-local,
+non-redundant) definitions and theorems introduced in the section.  In the
+above example, the @(':long') field would be extended with:</p>
 
 @({
  <h3>Definition and Theorems</h3>
@@ -1212,28 +1159,6 @@ not introduce a new local scope, but a @('defsection') does.</p>")
   :short "Placeholder for undocumented topics.")
 
 
-(defxdoc test-of-entities
-  :short "Placeholder topic for testing out HTML entity support in XDOC."
-  :long "<p>Here are the entities that XDOC allows:</p>
-
-<p>Normal XML entities:</p>
-<ul>
-<li>@('&amp;')   becomes &amp;</li>
-<li>@('&lt;')    becomes &lt;</li>
-<li>@('&gt;')    becomes &gt;</li>
-<li>@('&quot;')  becomes &quot;</li>
-<li>@('&apos;')  becomes &apos;</li>
-</ul>
-
-<p>Additional entities allowed by XDOC:</p>
-<ul>
-<li>@('&nbsp;')  becomes &nbsp; (this one can be hard to see)</li>
-<li>@('&mdash;') becomes &mdash;</li>
-<li>@('&rarr;')  becomes &rarr;</li>
-</ul>")
-
-
-
 (defxdoc set-default-parents
   :short "Set up default parents to use for @(see xdoc)."
 
@@ -1357,6 +1282,50 @@ the back.</p>
 
 <p>See @(see xdoc-extend) for related commentary.</p>")
 
+
+(defxdoc order-subtopics
+  :short "Control the ordering of subtopics in the XDOC display."
+
+  :long "<p>@(call order-subtopics) allows you to specify what order the
+subtopics of @('name') will be presented in.  For instance:</p>
+
+@({
+    (xdoc::order-subtopics mini-tutorial
+      (example syntax semantics gotchas references))
+})
+
+<p>By default, XDOC shows subtopics in an alphabetical order.  For most
+reference-manual material this is generally fine.  But, especially for tutorial
+guides, you sometimes intend your topics to be read in some particular order,
+and alphabetizing things gets in the way.</p>
+
+<p>The @('order-subtopics') command lets you specify the exact subtopic
+ordering that should be used for a particular topic.  The general form is:</p>
+
+@({
+    (xdoc::order-subtopics parent
+      (subtopic1 subtopic2 ...))
+})
+
+<p>You don't have to give a complete order.  Any subtopics that aren't
+mentioned will be listed last, in the usual alphabetical order.</p>
+
+<p>We require @('parent') to refer to some defined topic, but the subtopics
+don't need to be defined at @('order-subtopics') time.  This makes it easy to
+keep the subtopic ordering close to the defxdoc command, e.g., you can
+write:</p>
+
+@({
+     (defxdoc mini-tutorial ...))
+     (order-subtopics mini-tutorial (example syntax ...))
+
+     (defxdoc example ...)
+     (defxdoc syntax ...)
+})
+
+<p>We do at least warn about undefined topics when you @(see save) a
+manual.</p>")
+
 #||
 
 (in-package "ACL2")
@@ -1388,3 +1357,44 @@ the back.</p>
 ")
 
 ||#
+
+
+(defxdoc xdoc-tests
+  :short "Topics that exist only to test XDOC functionality.")
+
+(local (set-default-parents xdoc-tests))
+
+(defxdoc test-of-entities
+  :short "Placeholder topic for testing out HTML entity support in XDOC."
+  :long "<p>Here are the entities that XDOC allows:</p>
+
+<p>Normal XML entities:</p>
+<ul>
+<li>@('&amp;')   becomes &amp;</li>
+<li>@('&lt;')    becomes &lt;</li>
+<li>@('&gt;')    becomes &gt;</li>
+<li>@('&quot;')  becomes &quot;</li>
+<li>@('&apos;')  becomes &apos;</li>
+</ul>
+
+<p>Additional entities allowed by XDOC:</p>
+<ul>
+<li>@('&nbsp;')  becomes &nbsp; (this one can be hard to see)</li>
+<li>@('&mdash;') becomes &mdash;</li>
+<li>@('&rarr;')  becomes &rarr;</li>
+</ul>")
+
+(order-subtopics xdoc-tests
+  (order-test-w
+   order-test-o
+   order-test-r
+   order-test-z-intentionally-missing
+   order-test-k
+   order-test-s))
+
+(defxdoc order-test-s :short "S")
+(defxdoc order-test-w :short "W")
+(defxdoc order-test-r :short "R")
+(defxdoc order-test-o :short "O")
+(defxdoc order-test-k :short "K")
+
